@@ -1,3 +1,9 @@
+# postgresql/psycopg2.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
 """Support for the PostgreSQL database via the psycopg2 driver.
 
 Driver
@@ -96,8 +102,9 @@ from sqlalchemy.sql import expression
 from sqlalchemy.sql import operators as sql_operators
 from sqlalchemy import types as sqltypes
 from sqlalchemy.dialects.postgresql.base import PGDialect, PGCompiler, \
-                                            PGIdentifierPreparer, PGExecutionContext, \
-                                            ENUM, ARRAY, _DECIMAL_TYPES, _FLOAT_TYPES
+                                PGIdentifierPreparer, PGExecutionContext, \
+                                ENUM, ARRAY, _DECIMAL_TYPES, _FLOAT_TYPES,\
+                                _INT_TYPES
 
 
 logger = logging.getLogger('sqlalchemy.dialects.postgresql')
@@ -111,19 +118,21 @@ class _PGNumeric(sqltypes.Numeric):
         if self.asdecimal:
             if coltype in _FLOAT_TYPES:
                 return processors.to_decimal_processor_factory(decimal.Decimal)
-            elif coltype in _DECIMAL_TYPES:
+            elif coltype in _DECIMAL_TYPES or coltype in _INT_TYPES:
                 # pg8000 returns Decimal natively for 1700
                 return None
             else:
-                raise exc.InvalidRequestError("Unknown PG numeric type: %d" % coltype)
+                raise exc.InvalidRequestError(
+                            "Unknown PG numeric type: %d" % coltype)
         else:
             if coltype in _FLOAT_TYPES:
                 # pg8000 returns float natively for 701
                 return None
-            elif coltype in _DECIMAL_TYPES:
+            elif coltype in _DECIMAL_TYPES or coltype in _INT_TYPES:
                 return processors.to_float
             else:
-                raise exc.InvalidRequestError("Unknown PG numeric type: %d" % coltype)
+                raise exc.InvalidRequestError(
+                            "Unknown PG numeric type: %d" % coltype)
 
 class _PGEnum(ENUM):
     def __init__(self, *arg, **kw):
@@ -149,7 +158,7 @@ SERVER_SIDE_CURSOR_RE = re.compile(
 class PGExecutionContext_psycopg2(PGExecutionContext):
     def create_cursor(self):
         # TODO: coverage for server side cursors + select.for_update()
-        
+
         if self.dialect.server_side_cursors:
             is_server_side = \
                 self.execution_options.get('stream_results', True) and (
@@ -176,7 +185,7 @@ class PGExecutionContext_psycopg2(PGExecutionContext):
     def get_result_proxy(self):
         if logger.isEnabledFor(logging.INFO):
             self._log_notices(self.cursor)
-        
+
         if self.__is_server_side:
             return base.BufferedRowResultProxy(self)
         else:
@@ -194,7 +203,7 @@ class PGExecutionContext_psycopg2(PGExecutionContext):
 class PGCompiler_psycopg2(PGCompiler):
     def visit_mod(self, binary, **kw):
         return self.process(binary.left) + " %% " + self.process(binary.right)
-    
+
     def post_process_text(self, text):
         return text.replace('%', '%%')
 
@@ -228,12 +237,12 @@ class PGDialect_psycopg2(PGDialect):
         self.server_side_cursors = server_side_cursors
         self.use_native_unicode = use_native_unicode
         self.supports_unicode_binds = use_native_unicode
-        
+
     @classmethod
     def dbapi(cls):
         psycopg = __import__('psycopg2')
         return psycopg
-    
+
     def on_connect(self):
         if self.isolation_level is not None:
             extensions = __import__('psycopg2.extensions').extensions
@@ -242,7 +251,7 @@ class PGDialect_psycopg2(PGDialect):
             'READ_UNCOMMITTED':extensions.ISOLATION_LEVEL_READ_UNCOMMITTED, 
             'REPEATABLE_READ':extensions.ISOLATION_LEVEL_REPEATABLE_READ,
             'SERIALIZABLE':extensions.ISOLATION_LEVEL_SERIALIZABLE
-            
+
             }
             def base_on_connect(conn):
                 try:
@@ -253,7 +262,7 @@ class PGDialect_psycopg2(PGDialect):
                                 self.isolation_level)
         else:
             base_on_connect = None
-            
+
         if self.dbapi and self.use_native_unicode:
             extensions = __import__('psycopg2.extensions').extensions
             def connect(conn):
@@ -273,14 +282,22 @@ class PGDialect_psycopg2(PGDialect):
 
     def is_disconnect(self, e):
         if isinstance(e, self.dbapi.OperationalError):
-            return 'closed the connection' in str(e) or 'connection not open' in str(e)
+            # these error messages from libpq: interfaces/libpq/fe-misc.c.
+            # TODO: these are sent through gettext in libpq and we can't 
+            # check within other locales - consider using connection.closed 
+            return 'closed the connection' in str(e) or \
+                    'connection not open' in str(e) or \
+                    'could not receive data from server' in str(e)
         elif isinstance(e, self.dbapi.InterfaceError):
-            return 'connection already closed' in str(e) or 'cursor already closed' in str(e)
+            # psycopg2 client errors, psycopg2/conenction.h, psycopg2/cursor.h
+            return 'connection already closed' in str(e) or \
+                    'cursor already closed' in str(e)
         elif isinstance(e, self.dbapi.ProgrammingError):
-            # yes, it really says "losed", not "closed"
+            # not sure where this path is originally from, it may 
+            # be obsolete.   It really says "losed", not "closed".
             return "losed the connection unexpectedly" in str(e)
         else:
             return False
 
 dialect = PGDialect_psycopg2
-    
+
