@@ -1,5 +1,5 @@
 # engine/base.py
-# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -69,19 +69,19 @@ class Dialect(object):
       a tuple containing a version number for the DB backend in use.
       This value is only available for supporting dialects, and is
       typically populated during the initial connection to the database.
-    
+
     default_schema_name
      the name of the default schema.  This value is only available for
      supporting dialects, and is typically populated during the
      initial connection to the database.
-     
+
     execution_ctx_cls
       a :class:`ExecutionContext` class used to handle statement execution
 
     execute_sequence_format
       either the 'tuple' or 'list' type, depending on what cursor.execute()
       accepts for the second argument (they vary).
-      
+
     preparer
       a :class:`~sqlalchemy.sql.compiler.IdentifierPreparer` class used to
       quote identifiers.
@@ -113,7 +113,7 @@ class Dialect(object):
       True if 'implicit' primary key functions must be executed separately
       in order to get their value.   This is currently oriented towards
       Postgresql.
-      
+
     implicit_returning
       use RETURNING or equivalent during INSERT execution in order to load 
       newly generated primary keys and other column defaults in one execution,
@@ -121,7 +121,7 @@ class Dialect(object):
       If an insert statement has returning() specified explicitly, 
       the "implicit" functionality is not used and inserted_primary_key
       will not be available.
-      
+
     dbapi_type_map
       A mapping of DB-API type objects present in this Dialect's
       DB-API implementation mapped to TypeEngine implementations used
@@ -141,17 +141,17 @@ class Dialect(object):
     supports_default_values
       Indicates if the construct ``INSERT INTO tablename DEFAULT
       VALUES`` is supported
-    
+
     supports_sequences
       Indicates if the dialect supports CREATE SEQUENCE or similar.
-    
+
     sequences_optional
       If True, indicates if the "optional" flag on the Sequence() construct
       should signal to not generate a CREATE SEQUENCE. Applies only to
       dialects that support sequences. Currently used only to allow Postgresql
       SERIAL to be used on a column that specifies Sequence() for usage on
       other backends.
-        
+
     supports_native_enum
       Indicates if the dialect supports a native ENUM construct.
       This will prevent types.Enum from generating a CHECK
@@ -161,7 +161,7 @@ class Dialect(object):
       Indicates if the dialect supports a native boolean construct.
       This will prevent types.Boolean from generating a CHECK
       constraint when that type is used.
-      
+
     """
 
     def create_connect_args(self, url):
@@ -170,7 +170,7 @@ class Dialect(object):
         Given a :class:`~sqlalchemy.engine.url.URL` object, returns a tuple
         consisting of a `*args`/`**kwargs` suitable to send directly
         to the dbapi's connect function.
-        
+
         """
 
         raise NotImplementedError()
@@ -185,7 +185,7 @@ class Dialect(object):
 
         The returned result is cached *per dialect class* so can
         contain no dialect-instance state.
-        
+
         """
 
         raise NotImplementedError()
@@ -196,13 +196,13 @@ class Dialect(object):
 
         Allows dialects to configure options based on server version info or
         other properties.
-        
+
         The connection passed here is a SQLAlchemy Connection object, 
         with full capabilities.
-        
+
         The initalize() method of the base dialect should be called via
         super().
-        
+
         """
 
         pass
@@ -215,12 +215,12 @@ class Dialect(object):
         properties from the database.  If include_columns (a list or
         set) is specified, limit the autoload to the given column
         names.
-        
+
         The default implementation uses the 
         :class:`~sqlalchemy.engine.reflection.Inspector` interface to 
         provide the output, building upon the granular table/column/
         constraint etc. methods of :class:`Dialect`.
-        
+
         """
 
         raise NotImplementedError()
@@ -354,7 +354,7 @@ class Dialect(object):
     def normalize_name(self, name):
         """convert the given name to lowercase if it is detected as 
         case insensitive.
-    
+
         this method is only used if the dialect defines
         requires_name_normalize=True.
 
@@ -364,13 +364,13 @@ class Dialect(object):
     def denormalize_name(self, name):
         """convert the given name to a case insensitive identifier
         for the backend if it is an all-lowercase name.
-        
+
         this method is only used if the dialect defines
         requires_name_normalize=True.
 
         """
         raise NotImplementedError()
-        
+
     def has_table(self, connection, table_name, schema=None):
         """Check the existence of a particular table in the database.
 
@@ -394,15 +394,15 @@ class Dialect(object):
 
     def _get_server_version_info(self, connection):
         """Retrieve the server version info from the given connection.
-        
+
         This is used by the default implementation to populate the
         "server_version_info" attribute and is called exactly
         once upon first connect.
-        
+
         """
 
         raise NotImplementedError()
-        
+
     def _get_default_schema_name(self, connection):
         """Return the string name of the currently selected schema from 
         the given connection.
@@ -410,7 +410,7 @@ class Dialect(object):
         This is used by the default implementation to populate the
         "default_schema_name" attribute and is called exactly
         once upon first connect.
-        
+
         """
 
         raise NotImplementedError()
@@ -510,7 +510,7 @@ class Dialect(object):
 
         The callable accepts a single argument "conn" which is the 
         DBAPI connection itself.  It has no return value.
-        
+
         This is used to set dialect-wide per-connection options such as
         isolation modes, unicode modes, etc.
 
@@ -617,7 +617,7 @@ class ExecutionContext(object):
         raise NotImplementedError()
 
     def handle_dbapi_exception(self, e):
-        """Receive a DBAPI exception which occured upon execute, result 
+        """Receive a DBAPI exception which occurred upon execute, result 
         fetch, etc."""
 
         raise NotImplementedError()
@@ -661,7 +661,7 @@ class ExecutionContext(object):
         in some dialects; this is indicated by the 
         ``supports_sane_rowcount`` and ``supports_sane_multi_rowcount``
         dialect attributes.
-        
+
         """
 
         raise NotImplementedError()
@@ -704,13 +704,13 @@ class Compiled(object):
     @property
     def sql_compiler(self):
         """Return a Compiled that is capable of processing SQL expressions.
-        
+
         If this compiler is one, it would likely just return 'self'.
-        
+
         """
-        
+
         raise NotImplementedError()
-        
+
     def process(self, obj, **kwargs):
         return obj._compiler_dispatch(self, **kwargs)
 
@@ -806,22 +806,19 @@ class Connection(Connectable):
     shared among threads using properly synchronized access, it is still
     possible that the underlying DBAPI connection may not support shared
     access between threads.  Check the DBAPI documentation for details.
-    
+
     The Connection object represents a single dbapi connection checked out
     from the connection pool. In this state, the connection pool has no affect
     upon the connection, including its expiration or timeout state. For the
     connection pool to properly manage connections, connections should be
     returned to the connection pool (i.e. ``connection.close()``) whenever the
-    connection is not in use. If your application has a need for management
-    of multiple connections or is otherwise long running (this includes all
-    web applications, threaded or not), don't hold a single connection open at
-    the module level.
+    connection is not in use.
 
     .. index::
       single: thread safety; Connection
-      
+
     """
-    
+
     def __init__(self, engine, connection=None, close_with_result=False,
                  _branch=False, _execution_options=None):
         """Construct a new Connection.
@@ -829,7 +826,7 @@ class Connection(Connectable):
         The constructor here is not public and is only called only by an
         :class:`.Engine`. See :meth:`.Engine.connect` and
         :meth:`.Engine.contextual_connect` methods.
-        
+
         """
         self.engine = engine
         self.__connection = connection or engine.raw_connection()
@@ -865,21 +862,21 @@ class Connection(Connectable):
         c = self.__class__.__new__(self.__class__)
         c.__dict__ = self.__dict__.copy()
         return c
-    
+
     def execution_options(self, **opt):
         """ Set non-SQL options for the connection which take effect 
         during execution.
-        
+
         The method returns a copy of this :class:`Connection` which references
         the same underlying DBAPI connection, but also defines the given
         execution options which will take effect for a call to
         :meth:`execute`. As the new :class:`Connection` references the same
         underlying resource, it is probably best to ensure that the copies
         would be discarded immediately, which is implicit if used as in::
-        
+
             result = connection.execution_options(stream_results=True).\
                                 execute(stmt)
-            
+
         The options are the same as those accepted by 
         :meth:`sqlalchemy.sql.expression.Executable.execution_options`.
 
@@ -887,7 +884,7 @@ class Connection(Connectable):
         c = self._clone()
         c._execution_options = c._execution_options.union(opt)
         return c
-    
+
     @property
     def dialect(self):
         "Dialect used by this Connection."
@@ -929,7 +926,7 @@ class Connection(Connectable):
         # use getattr() for is_valid to support exceptions raised in
         # dialect initializer, where the connection is not wrapped in
         # _ConnectionFairy
-        
+
         return getattr(self.__connection, 'is_valid', False)
 
     @property
@@ -978,7 +975,7 @@ class Connection(Connectable):
         """
         if self.invalidated:
             return
-            
+
         if self.closed:
             raise exc.ResourceClosedError("This Connection is closed")
 
@@ -986,8 +983,8 @@ class Connection(Connectable):
             self.__connection.invalidate(exception)
         del self.__connection
         self.__invalid = True
-    
-        
+
+
     def detach(self):
         """Detach the underlying DB-API connection from its connection pool.
 
@@ -1072,7 +1069,7 @@ class Connection(Connectable):
 
     def _begin_impl(self):
         if self._echo:
-            self.engine.logger.info("BEGIN")
+            self.engine.logger.info("BEGIN (implicit)")
         try:
             self.engine.dialect.do_begin(self.connection)
         except Exception, e:
@@ -1158,7 +1155,7 @@ class Connection(Connectable):
         self.__invalid = False
         del self.__connection
         self.__transaction = None
-        
+
     def scalar(self, object, *multiparams, **params):
         """Executes and returns the first column of the first row.
 
@@ -1169,9 +1166,9 @@ class Connection(Connectable):
 
     def execute(self, object, *multiparams, **params):
         """Executes the given construct and returns a :class:`.ResultProxy`.
-        
+
         The construct can be one of:
-        
+
         * a textual SQL string
         * any :class:`.ClauseElement` construct that is also
           a subclass of :class:`.Executable`, such as a 
@@ -1182,7 +1179,7 @@ class Connection(Connectable):
         * a :class:`.DDLElement` object
         * a :class:`.DefaultGenerator` object
         * a :class:`.Compiled` object
-            
+
         """
 
         for c in type(object).__mro__:
@@ -1204,7 +1201,7 @@ class Connection(Connectable):
 
         In the case of 'raw' execution which accepts positional parameters,
         it may be a list of tuples or lists.
-        
+
         """
 
         if not multiparams:
@@ -1292,7 +1289,7 @@ class Connection(Connectable):
     def __execute_context(self, context):
         if context.compiled:
             context.pre_exec()
-            
+
         if context.executemany:
             self._cursor_executemany(
                             context.cursor, 
@@ -1303,25 +1300,25 @@ class Connection(Connectable):
                             context.cursor, 
                             context.statement, 
                             context.parameters[0], context=context)
-            
+
         if context.compiled:
             context.post_exec()
-            
+
             if context.isinsert and not context.executemany:
                 context.post_insert()
-        
+
         # create a resultproxy, get rowcount/implicit RETURNING
         # rows, close cursor if no further results pending
         r = context.get_result_proxy()._autoclose()
 
         if self.__transaction is None and context.should_autocommit:
             self._commit_impl()
-        
+
         if r.closed and self.should_close_with_result:
             self.close()
-        
+
         return r
-        
+
     def _handle_dbapi_exception(self, 
                                     e, 
                                     statement, 
@@ -1368,7 +1365,7 @@ class Connection(Connectable):
                                     connection_invalidated=is_disconnect), \
                                     None, sys.exc_info()[2]
             # end Py2K
-            
+
         finally:
             del self._reentrant_error
 
@@ -1459,9 +1456,9 @@ class Connection(Connectable):
         This is a shortcut for explicitly calling `begin()` and `commit()`
         and optionally `rollback()` when exceptions are raised.  The
         given `*args` and `**kwargs` will be passed to the function.
-        
+
         See also transaction() on engine.
-        
+
         """
 
         trans = self.begin()
@@ -1485,7 +1482,7 @@ class Transaction(object):
     also implements a context manager interface so that 
     the Python ``with`` statement can be used with the 
     :meth:`.Connection.begin` method.
-    
+
     The Transaction object is **not** threadsafe.
 
     .. index::
@@ -1496,7 +1493,7 @@ class Transaction(object):
         """The constructor for :class:`.Transaction` is private
         and is called from within the :class:`.Connection.begin` 
         implementation.
-        
+
         """
         self.connection = connection
         self._parent = parent or self
@@ -1511,7 +1508,7 @@ class Transaction(object):
 
         This is used to cancel a Transaction without affecting the scope of
         an enclosing transaction.
-        
+
         """
         if not self._parent.is_active:
             return
@@ -1520,7 +1517,7 @@ class Transaction(object):
 
     def rollback(self):
         """Roll back this :class:`.Transaction`.
-        
+
         """
         if not self._parent.is_active:
             return
@@ -1532,7 +1529,7 @@ class Transaction(object):
 
     def commit(self):
         """Commit this :class:`.Transaction`."""
-        
+
         if not self._parent.is_active:
             raise exc.InvalidRequestError("This transaction is inactive")
         self._do_commit()
@@ -1606,7 +1603,7 @@ class Engine(Connectable, log.Identified):
     Connects a :class:`~sqlalchemy.pool.Pool` and 
     :class:`~sqlalchemy.engine.base.Dialect` together to provide a source 
     of database connectivity and behavior.
-    
+
     An :class:`Engine` object is instantiated publically using the 
     :func:`~sqlalchemy.create_engine` function.
 
@@ -1632,15 +1629,15 @@ class Engine(Connectable, log.Identified):
             self.Connection = Connection
         if execution_options:
             self.update_execution_options(**execution_options)
-    
+
     def update_execution_options(self, **opt):
         """update the execution_options dictionary of this :class:`Engine`.
-        
+
         For details on execution_options, see
         :meth:`Connection.execution_options` as well as
         :meth:`sqlalchemy.sql.expression.Executable.execution_options`.
-        
-        
+
+
         """
         self._execution_options = \
                 self._execution_options.union(opt)
@@ -1670,23 +1667,23 @@ class Engine(Connectable, log.Identified):
         A new connection pool is created immediately after the old one has
         been disposed.   This new pool, like all SQLAlchemy connection pools,
         does not make any actual connections to the database until one is 
-        first requested.   
-        
+        first requested.
+
         This method has two general use cases:
-        
+
          * When a dropped connection is detected, it is assumed that all
            connections held by the pool are potentially dropped, and 
            the entire pool is replaced.
-           
+
          * An application may want to use :meth:`dispose` within a test 
            suite that is creating multiple engines.
-           
+
         It is critical to note that :meth:`dispose` does **not** guarantee
         that the application will release all open database connections - only
-        those connections that are checked into the pool are closed.   
+        those connections that are checked into the pool are closed.
         Connections which remain checked out or have been detached from
         the engine are not affected. 
-        
+
         """
         self.pool.dispose()
         self.pool = self.pool.recreate()
@@ -1723,11 +1720,11 @@ class Engine(Connectable, log.Identified):
     def text(self, text, *args, **kwargs):
         """Return a :func:`~sqlalchemy.sql.expression.text` construct, 
         bound to this engine.
-        
+
         This is equivalent to::
-        
+
             text("SELECT * FROM table", bind=engine)
-            
+
         """
 
         return expression.text(text, bind=self, *args, **kwargs)
@@ -1751,13 +1748,13 @@ class Engine(Connectable, log.Identified):
         This is a shortcut for explicitly calling `begin()` and `commit()`
         and optionally `rollback()` when exceptions are raised.  The
         given `*args` and `**kwargs` will be passed to the function.
-        
+
         The connection used is that of contextual_connect().
-        
+
         See also the similar method on Connection itself.
-        
+
         """
-        
+
         conn = self.contextual_connect()
         try:
             return conn.transaction(callable_, *args, **kwargs)
@@ -1773,10 +1770,10 @@ class Engine(Connectable, log.Identified):
 
     def execute(self, statement, *multiparams, **params):
         """Executes the given construct and returns a :class:`.ResultProxy`.
-        
+
         The arguments are the same as those used by
         :meth:`.Connection.execute`.
-        
+
         Here, a :class:`.Connection` is acquired using the
         :meth:`~.Engine.contextual_connect` method, and the statement executed
         with that connection. The returned :class:`.ResultProxy` is flagged
@@ -1784,7 +1781,7 @@ class Engine(Connectable, log.Identified):
         underlying cursor is closed, the :class:`.Connection` created here
         will also be closed, which allows its associated DBAPI connection
         resource to be returned to the connection pool.
-        
+
         """
 
         connection = self.contextual_connect(close_with_result=True)
@@ -1803,29 +1800,29 @@ class Engine(Connectable, log.Identified):
 
     def connect(self, **kwargs):
         """Return a new :class:`.Connection` object.
-        
+
         The :class:`.Connection`, upon construction, will procure a DBAPI connection
         from the :class:`.Pool` referenced by this :class:`.Engine`,
         returning it back to the :class:`.Pool` after the :meth:`.Connection.close`
         method is called.
-        
+
         """
 
         return self.Connection(self, **kwargs)
 
     def contextual_connect(self, close_with_result=False, **kwargs):
         """Return a :class:`.Connection` object which may be part of some ongoing context.
-        
+
         By default, this method does the same thing as :meth:`.Engine.connect`.
         Subclasses of :class:`.Engine` may override this method
         to provide contextual behavior.
 
         :param close_with_result: When True, the first :class:`.ResultProxy` created
           by the :class:`.Connection` will call the :meth:`.Connection.close` method
-          of that connection as soon as any pending result rows are exhausted.  
+          of that connection as soon as any pending result rows are exhausted.
           This is used to supply the "connectionless execution" behavior provided
           by the :meth:`.Engine.execute` method.
-          
+
         """
 
         return self.Connection(self, 
@@ -1903,7 +1900,7 @@ def _proxy_connection_cls(cls, proxy):
 
         def _begin_impl(self):
             return proxy.begin(self, super(ProxyConnection, self)._begin_impl)
-            
+
         def _rollback_impl(self):
             return proxy.rollback(self, 
                                 super(ProxyConnection, self)._rollback_impl)
@@ -1922,7 +1919,7 @@ def _proxy_connection_cls(cls, proxy):
                         super(ProxyConnection,
                                 self)._rollback_to_savepoint_impl, 
                                 name, context)
-            
+
         def _release_savepoint_impl(self, name, context):
             return proxy.release_savepoint(self, 
                         super(ProxyConnection, self)._release_savepoint_impl, 
@@ -2093,11 +2090,19 @@ class RowProxy(BaseRowProxy):
     def itervalues(self):
         return iter(self)
 
+try:
+    # Register RowProxy with Sequence, 
+    # so sequence protocol is implemented
+    from collections import Sequence
+    Sequence.register(RowProxy)
+except ImportError:
+    pass
+
 
 class ResultMetaData(object):
     """Handle cursor.description, applying additional info from an execution
     context."""
-    
+
     def __init__(self, parent, metadata):
         self._processors = processors = []
 
@@ -2135,14 +2140,14 @@ class ResultMetaData(object):
 
             processor = type_.dialect_impl(dialect).\
                             result_processor(dialect, coltype)
-            
+
             processors.append(processor)
             rec = (processor, i)
 
             # indexes as keys. This is only needed for the Python version of
             # RowProxy (the C version uses a faster path for integer indexes).
             keymap[i] = rec
-            
+
             # Column names as keys 
             if keymap.setdefault(name.lower(), rec) is not rec: 
                 # We do not raise an exception directly because several
@@ -2155,10 +2160,10 @@ class ResultMetaData(object):
             if origname and \
                     keymap.setdefault(origname.lower(), rec) is not rec:
                 keymap[origname.lower()] = (processor, None)
-            
+
             if dialect.requires_name_normalize:
                 colname = dialect.normalize_name(colname)
-                
+
             self.keys.append(colname)
             if obj:
                 for o in obj:
@@ -2169,7 +2174,7 @@ class ResultMetaData(object):
             self.logger.debug(
                 "Col %r", tuple(x[0] for x in metadata))
 
-    def _key_fallback(self, key):
+    def _key_fallback(self, key, raiseerr=True):
         map = self._keymap
         result = None
         if isinstance(key, basestring):
@@ -2183,8 +2188,12 @@ class ResultMetaData(object):
             elif hasattr(key, 'name') and key.name.lower() in map:
                 result = map[key.name.lower()]
         if result is None:
-            raise exc.NoSuchColumnError(
-                "Could not locate column in row for column '%s'" % key)
+            if raiseerr:
+                raise exc.NoSuchColumnError(
+                    "Could not locate column in row for column '%s'" % 
+                        expression._string_or_unprintable(key))
+            else:
+                return None
         else:
             map[key] = result
         return result
@@ -2193,11 +2202,7 @@ class ResultMetaData(object):
         if key in self._keymap:
             return True
         else:
-            try:
-                self._key_fallback(key)
-                return True
-            except exc.NoSuchColumnError:
-                return False
+            return self._key_fallback(key, False) is not None
 
     def __len__(self):
         return len(self.keys)
@@ -2211,7 +2216,7 @@ class ResultMetaData(object):
             ),
             'keys': self.keys
         }
-    
+
     def __setstate__(self, state):
         # the row has been processed at pickling time so we don't need any
         # processor anymore
@@ -2222,7 +2227,7 @@ class ResultMetaData(object):
         self.keys = state['keys']
         self._echo = False
 
-       
+
 class ResultProxy(object):
     """Wraps a DB-API cursor object to provide easier access to row columns.
 
@@ -2247,12 +2252,12 @@ class ResultProxy(object):
     _process_row = RowProxy
     out_parameters = None
     _can_close_connection = False
-    
+
     def __init__(self, context):
         self.context = context
         self.dialect = context.dialect
         self.closed = False
-        self.cursor = context.cursor
+        self.cursor = self._saved_cursor = context.cursor
         self.connection = context.root_connection
         self._echo = self.connection._echo and \
                         context.engine._should_log_debug()
@@ -2264,23 +2269,23 @@ class ResultProxy(object):
             self._metadata = None
         else:
             self._metadata = ResultMetaData(self, metadata)
-        
+
     def keys(self):
         """Return the current set of string keys for rows."""
         if self._metadata:
             return self._metadata.keys
         else:
             return []
-        
+
     @util.memoized_property
     def rowcount(self):
         """Return the 'rowcount' for this result.
-        
+
         The 'rowcount' reports the number of rows affected
         by an UPDATE or DELETE statement.  It has *no* other
         uses and is not intended to provide the number of rows
         present from a SELECT.
-        
+
         Note that this row count may not be properly implemented in some
         dialects; this is indicated by
         :meth:`~sqlalchemy.engine.base.ResultProxy.supports_sane_rowcount()`
@@ -2288,37 +2293,67 @@ class ResultProxy(object):
         :meth:`~sqlalchemy.engine.base.ResultProxy.supports_sane_multi_rowcount()`.
         ``rowcount()`` also may not work at this time for a statement that
         uses ``returning()``.
-        
+
         """
         return self.context.rowcount
 
     @property
     def lastrowid(self):
         """return the 'lastrowid' accessor on the DBAPI cursor.
-        
+
         This is a DBAPI specific method and is only functional
         for those backends which support it, for statements
         where it is appropriate.  It's behavior is not 
         consistent across backends.
-        
+
         Usage of this method is normally unnecessary; the
         :attr:`~ResultProxy.inserted_primary_key` attribute provides a
         tuple of primary key values for a newly inserted row,
         regardless of database backend.
+
+        """
+        return self._saved_cursor.lastrowid
+
+    @property
+    def returns_rows(self):
+        """True if this :class:`.ResultProxy` returns rows.
+        
+        I.e. if it is legal to call the methods 
+        :meth:`~.ResultProxy.fetchone`, 
+        :meth:`~.ResultProxy.fetchmany`
+        :meth:`~.ResultProxy.fetchall`.
+        
+        New in 0.6.7.
+
+        """
+        return self._metadata is not None
+
+    @property
+    def is_insert(self):
+        """True if this :class:`.ResultProxy` is the result
+        of a executing an expression language compiled 
+        :func:`.expression.insert` construct.
+        
+        When True, this implies that the 
+        :attr:`inserted_primary_key` attribute is accessible,
+        assuming the statement did not include
+        a user defined "returning" construct.
+
+        New in 0.6.7.
         
         """
-        return self.cursor.lastrowid
-    
+        return self.context.isinsert
+
     def _cursor_description(self):
         """May be overridden by subclasses."""
-        
-        return self.cursor.description
-            
+
+        return self._saved_cursor.description
+
     def _autoclose(self):
         """called by the Connection to autoclose cursors that have no pending
         results beyond those used by an INSERT/UPDATE/DELETE with no explicit
         RETURNING clause.
-        
+
         """
         if self.context.isinsert:
             if self.context._is_implicit_returning:
@@ -2332,14 +2367,14 @@ class ResultProxy(object):
             # such as kintersbasdb, mxodbc),
             self.rowcount
             self.close(_autoclose_connection=False)
-        
+
         return self
-    
+
     def close(self, _autoclose_connection=True):
         """Close this ResultProxy.
 
         Closes the underlying DBAPI cursor corresponding to the execution.
-        
+
         Note that any data cached within this ResultProxy is still available.
         For some types of results, this may include buffered rows.
 
@@ -2351,7 +2386,7 @@ class ResultProxy(object):
 
         * all result rows are exhausted using the fetchXXX() methods.
         * cursor.description is None.
-        
+
         """
 
         if not self.closed:
@@ -2362,7 +2397,7 @@ class ResultProxy(object):
                 self.connection.close()
             # allow consistent errors
             self.cursor = None
-            
+
     def __iter__(self):
         while True:
             row = self.fetchone()
@@ -2370,11 +2405,11 @@ class ResultProxy(object):
                 raise StopIteration
             else:
                 yield row
-    
+
     @util.memoized_property
     def inserted_primary_key(self):
         """Return the primary key for the row just inserted.
-        
+
         This only applies to single row insert() constructs which
         did not explicitly specify returning().
 
@@ -2386,15 +2421,15 @@ class ResultProxy(object):
             raise exc.InvalidRequestError(
                         "Can't call inserted_primary_key when returning() "
                         "is used.")
-            
+
         return self.context._inserted_primary_key
 
     @util.deprecated("0.6", "Use :attr:`.ResultProxy.inserted_primary_key`")
     def last_inserted_ids(self):
         """Return the primary key for the row just inserted."""
-        
+
         return self.inserted_primary_key
-        
+
     def last_updated_params(self):
         """Return ``last_updated_params()`` from the underlying
         ExecutionContext.
@@ -2460,7 +2495,7 @@ class ResultProxy(object):
             return self.cursor.fetchall()
         except AttributeError:
             self._non_result()
-    
+
     def _non_result(self):
         if self._metadata is None:
             raise exc.ResourceClosedError(
@@ -2469,7 +2504,7 @@ class ResultProxy(object):
             )
         else:
             raise exc.ResourceClosedError("This result object is closed.")
-        
+
     def process_rows(self, rows):
         process_row = self._process_row
         metadata = self._metadata
@@ -2502,10 +2537,10 @@ class ResultProxy(object):
     def fetchmany(self, size=None):
         """Fetch many rows, just like DB-API
         ``cursor.fetchmany(size=cursor.arraysize)``.
-        
+
         If rows are present, the cursor remains open after this is called.
         Else the cursor is automatically closed and an empty list is returned.
-        
+
         """
 
         try:
@@ -2521,10 +2556,10 @@ class ResultProxy(object):
 
     def fetchone(self):
         """Fetch one row, just like DB-API ``cursor.fetchone()``.
-        
+
         If a row is present, the cursor remains open after this is called.
         Else the cursor is automatically closed and None is returned.
-        
+
         """
         try:
             row = self._fetchone_impl()
@@ -2541,9 +2576,9 @@ class ResultProxy(object):
 
     def first(self):
         """Fetch the first row and then close the result set unconditionally.
-        
+
         Returns None if no row is present.
-        
+
         """
         if self._metadata is None:
             self._non_result()
@@ -2563,12 +2598,12 @@ class ResultProxy(object):
                 return None
         finally:
             self.close()
-        
+
     def scalar(self):
         """Fetch the first column of the first row, and close the result set.
-        
+
         Returns None if no row is present.
-        
+
         """
         row = self.first()
         if row is not None:
@@ -2637,11 +2672,11 @@ class BufferedRowResultProxy(ResultProxy):
 
 class FullyBufferedResultProxy(ResultProxy):
     """A result proxy that buffers rows fully upon creation.
-    
+
     Used for operations where a result is to be delivered
     after the database conversation can not be continued,
     such as MSSQL INSERT...OUTPUT after an autocommit.
-    
+
     """
     def _init_metadata(self):
         super(FullyBufferedResultProxy, self)._init_metadata()
@@ -2649,7 +2684,7 @@ class FullyBufferedResultProxy(ResultProxy):
 
     def _buffer_rows(self):
         return self.cursor.fetchall()
-        
+
     def _fetchone_impl(self):
         if self.__rowbuffer:
             return self.__rowbuffer.pop(0)
@@ -2683,7 +2718,7 @@ class BufferedColumnRow(RowProxy):
         row = tuple(row)
         super(BufferedColumnRow, self).__init__(parent, row,
                                                 processors, keymap)
-        
+
 class BufferedColumnResultProxy(ResultProxy):
     """A ResultProxy with column buffering behavior.
 
@@ -2693,7 +2728,7 @@ class BufferedColumnResultProxy(ResultProxy):
     databases where result rows contain "live" results that fall out
     of scope unless explicitly fetched.  Currently this includes
     cx_Oracle LOB objects.
-    
+
     """
 
     _process_row = BufferedColumnRow
