@@ -1,13 +1,14 @@
-# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Michael Bayer mike_mp@zzzcomputing.com
+# sqlalchemy/exc.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
 
 """Exceptions used with SQLAlchemy.
 
-The base exception class is SQLAlchemyError.  Exceptions which are raised as a
+The base exception class is :class:`.SQLAlchemyError`.  Exceptions which are raised as a
 result of DBAPI exceptions are all subclasses of
-:class:`~sqlalchemy.exc.DBAPIError`.
+:class:`.DBAPIError`.
 
 """
 
@@ -26,7 +27,11 @@ class ArgumentError(SQLAlchemyError):
 
 class CircularDependencyError(SQLAlchemyError):
     """Raised by topological sorts when a circular dependency is detected"""
-
+    def __init__(self, message, cycles, edges):
+        message += ": cycles: %r all edges: %r" % (cycles, edges)
+        SQLAlchemyError.__init__(self, message)
+        self.cycles = cycles
+        self.edges = edges
 
 class CompileError(SQLAlchemyError):
     """Raised when an error occurs during SQL compilation"""
@@ -34,7 +39,7 @@ class CompileError(SQLAlchemyError):
 class IdentifierError(SQLAlchemyError):
     """Raised when a schema name is beyond the max character limit"""
 
-# Moved to orm.exc; compatability definition installed by orm import until 0.6
+# Moved to orm.exc; compatibility definition installed by orm import until 0.6
 ConcurrentModificationError = None
 
 class DisconnectionError(SQLAlchemyError):
@@ -46,7 +51,7 @@ class DisconnectionError(SQLAlchemyError):
     """
 
 
-# Moved to orm.exc; compatability definition installed by orm import until 0.6
+# Moved to orm.exc; compatibility definition installed by orm import until 0.6
 FlushError = None
 
 class TimeoutError(SQLAlchemyError):
@@ -60,12 +65,16 @@ class InvalidRequestError(SQLAlchemyError):
 
     """
 
+class ResourceClosedError(InvalidRequestError):
+    """An operation was requested from a connection, cursor, or other
+    object that's in a closed state."""
+
 class NoSuchColumnError(KeyError, InvalidRequestError):
     """A nonexistent column is requested from a ``RowProxy``."""
 
 class NoReferenceError(InvalidRequestError):
     """Raised by ``ForeignKey`` to indicate a reference cannot be resolved."""
-    
+
 class NoReferencedTableError(NoReferenceError):
     """Raised by ``ForeignKey`` when the referred ``Table`` cannot be located."""
 
