@@ -1,3 +1,9 @@
+# ext/declarative.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
 """
 Synopsis
 ========
@@ -33,7 +39,7 @@ The resulting table and mapper are accessible via
 
     # access the mapped Table
     SomeClass.__table__
-    
+
     # access the Mapper
     SomeClass.__mapper__
 
@@ -51,7 +57,7 @@ just give the column a name.  Below, column "some_table_id" is mapped to the
     class SomeClass(Base):
         __tablename__ = 'some_table'
         id = Column("some_table_id", Integer, primary_key=True)
-    
+
 Attributes may be added to the class after its construction, and they will be
 added to the underlying :class:`.Table` and
 :func:`.mapper()` definitions as appropriate::
@@ -60,7 +66,7 @@ added to the underlying :class:`.Table` and
     SomeClass.related = relationship(RelatedInfo)
 
 Classes which are constructed using declarative can interact freely
-with classes that are mapped explicitly with :func:`mapper`.   
+with classes that are mapped explicitly with :func:`mapper`.
 
 It is recommended, though not required, that all tables 
 share the same underlying :class:`~sqlalchemy.schema.MetaData` object,
@@ -173,7 +179,7 @@ the :class:`.MetaData` object used by the declarative base::
         Column('author_id', Integer, ForeignKey('authors.id')),
         Column('keyword_id', Integer, ForeignKey('keywords.id'))
         )
-                
+
     class Author(Base):
         __tablename__ = 'authors'
         id = Column(Integer, primary_key=True)
@@ -205,11 +211,11 @@ using Python 2.6 style properties::
         @property
         def attr(self):
             return self._attr
-        
+
         @attr.setter
         def attr(self, attr):
             self._attr = attr
-            
+
         attr = synonym('_attr', descriptor=attr)
 
 The above synonym is then usable as an instance attribute as well as a
@@ -224,7 +230,7 @@ conjunction with ``@property``::
 
     class MyClass(Base):
         __tablename__ = 'sometable'
-    
+
         id = Column(Integer, primary_key=True)
         _attr = Column('attr', String)
 
@@ -271,19 +277,19 @@ need either from the local class definition or from remote
 classes::
 
     from sqlalchemy.sql import func
-    
+
     class Address(Base):
         __tablename__ = 'address'
 
         id = Column('id', Integer, primary_key=True)
         user_id = Column(Integer, ForeignKey('user.id'))
-        
+
     class User(Base):
         __tablename__ = 'user'
 
         id = Column(Integer, primary_key=True)
         name = Column(String)
-        
+
         address_count = column_property(
             select([func.count(Address.id)]).\\
                 where(Address.user_id==id)
@@ -351,17 +357,17 @@ to a table::
 table metadata, while still getting most of the benefits of using declarative.
 An application that uses reflection might want to load table metadata elsewhere
 and simply pass it to declarative classes::
-    
+
     from sqlalchemy.ext.declarative import declarative_base
-    
+
     Base = declarative_base()
     Base.metadata.reflect(some_engine)
-    
+
     class User(Base):
-        __table__ = metadata['user']
-    
+        __table__ = metadata.tables['user']
+
     class Address(Base):
-        __table__ = metadata['address']
+        __table__ = metadata.tables['address']
 
 Some configuration schemes may find it more appropriate to use ``__table__``, 
 such as those which already take advantage of the data-driven nature of 
@@ -380,13 +386,13 @@ mapped columns can reference them directly from within the
 class declaration::
 
     from datetime import datetime
-    
+
     class Widget(Base):
         __tablename__ = 'widgets'
-        
+
         id = Column(Integer, primary_key=True)
         timestamp = Column(DateTime, nullable=False)
-        
+
         __mapper_args__ = {
                         'version_id_col': timestamp,
                         'version_id_generator': lambda v:datetime.now()
@@ -482,7 +488,7 @@ Concrete is defined as a subclass which has its own table and sets the
         __tablename__ = 'people'
         id = Column(Integer, primary_key=True)
         name = Column(String(50))
-        
+
     class Engineer(Person):
         __tablename__ = 'engineers'
         __mapper_args__ = {'concrete':True}
@@ -503,16 +509,16 @@ requires usage of :func:`~sqlalchemy.orm.util.polymorphic_union`::
                     Column('name', String(50)),
                     Column('golf_swing', String(50))
                 )
-    
+
     punion = polymorphic_union({
         'engineer':engineers,
         'manager':managers
     }, 'type', 'punion')
-    
+
     class Person(Base):
         __table__ = punion
         __mapper_args__ = {'polymorphic_on':punion.c.type}
-        
+
     class Engineer(Person):
         __table__ = engineers
         __mapper_args__ = {'polymorphic_identity':'engineer', 'concrete':True}
@@ -520,7 +526,7 @@ requires usage of :func:`~sqlalchemy.orm.util.polymorphic_union`::
     class Manager(Person):
         __table__ = managers
         __mapper_args__ = {'polymorphic_identity':'manager', 'concrete':True}
-    
+
 
 Mixin Classes
 ==============
@@ -535,10 +541,10 @@ using a "mixin class". A mixin class is one that isn't mapped to a
 table and doesn't subclass the declarative :class:`Base`. For example::
 
     class MyMixin(object):
-    
+
         __table_args__ = {'mysql_engine': 'InnoDB'}
         __mapper_args__= {'always_refresh': True}
-        
+
         id =  Column(Integer, primary_key=True)
 
 
@@ -589,33 +595,34 @@ keys, as a :class:`ForeignKey` itself contains references to columns
 which can't be properly recreated at this level.  For columns that 
 have foreign keys, as well as for the variety of mapper-level constructs
 that require destination-explicit context, the
-:func:`~sqlalchemy.util.classproperty` decorator is provided so that
+:func:`~.declared_attr` decorator (renamed from ``sqlalchemy.util.classproperty`` in 0.6.5) 
+is provided so that
 patterns common to many classes can be defined as callables::
 
-    from sqlalchemy.util import classproperty
-    
+    from sqlalchemy.ext.declarative import declared_attr
+
     class ReferenceAddressMixin(object):
-        @classproperty
+        @declared_attr
         def address_id(cls):
             return Column(Integer, ForeignKey('address.id'))
-            
+
     class User(Base, ReferenceAddressMixin):
         __tablename__ = 'user'
         id = Column(Integer, primary_key=True)
-        
+
 Where above, the ``address_id`` class-level callable is executed at the 
 point at which the ``User`` class is constructed, and the declarative
 extension can use the resulting :class:`Column` object as returned by
 the method without the need to copy it.
 
-Columns generated by :func:`~sqlalchemy.util.classproperty` can also be
+Columns generated by :func:`~.declared_attr` can also be
 referenced by ``__mapper_args__`` to a limited degree, currently 
 by ``polymorphic_on`` and ``version_id_col``, by specifying the 
 classdecorator itself into the dictionary - the declarative extension
 will resolve them at class construction time::
 
     class MyMixin:
-        @classproperty
+        @declared_attr
         def type_(cls):
             return Column(String(50))
 
@@ -624,63 +631,56 @@ will resolve them at class construction time::
     class MyModel(Base,MyMixin):
         __tablename__='test'
         id =  Column(Integer, primary_key=True)
-    
-.. note:: The usage of :func:`~sqlalchemy.util.classproperty` with mixin 
-   columns is a new feature as of SQLAlchemy 0.6.2.
 
 Mixing in Relationships
 ~~~~~~~~~~~~~~~~~~~~~~~
 
 Relationships created by :func:`~sqlalchemy.orm.relationship` are provided
 with declarative mixin classes exclusively using the
-:func:`~sqlalchemy.util.classproperty` approach, eliminating any ambiguity
+:func:`.declared_attr` approach, eliminating any ambiguity
 which could arise when copying a relationship and its possibly column-bound
 contents. Below is an example which combines a foreign key column and a
 relationship so that two classes ``Foo`` and ``Bar`` can both be configured to
 reference a common target class via many-to-one::
 
     class RefTargetMixin(object):
-        @classproperty
+        @declared_attr
         def target_id(cls):
             return Column('target_id', ForeignKey('target.id'))
-    
-        @classproperty
+
+        @declared_attr
         def target(cls):
             return relationship("Target")
-    
+
     class Foo(Base, RefTargetMixin):
         __tablename__ = 'foo'
         id = Column(Integer, primary_key=True)
-    
+
     class Bar(Base, RefTargetMixin):
         __tablename__ = 'bar'
         id = Column(Integer, primary_key=True)
-                    
+
     class Target(Base):
         __tablename__ = 'target'
         id = Column(Integer, primary_key=True)
 
 :func:`~sqlalchemy.orm.relationship` definitions which require explicit
 primaryjoin, order_by etc. expressions should use the string forms 
-for these arguments, so that they are evaluated as late as possible.  
+for these arguments, so that they are evaluated as late as possible.
 To reference the mixin class in these expressions, use the given ``cls``
 to get it's name::
 
     class RefTargetMixin(object):
-        @classproperty
+        @declared_attr
         def target_id(cls):
             return Column('target_id', ForeignKey('target.id'))
-        
-        @classproperty
+
+        @declared_attr
         def target(cls):
             return relationship("Target",
                 primaryjoin="Target.id==%s.target_id" % cls.__name__
             )
 
-.. note:: The usage of :func:`~sqlalchemy.util.classproperty` with mixin 
-   relationships is a new feature as of SQLAlchemy 0.6.2.
-
-
 Mixing in deferred(), column_property(), etc.
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
@@ -688,21 +688,18 @@ Like :func:`~sqlalchemy.orm.relationship`, all
 :class:`~sqlalchemy.orm.interfaces.MapperProperty` subclasses such as
 :func:`~sqlalchemy.orm.deferred`, :func:`~sqlalchemy.orm.column_property`,
 etc. ultimately involve references to columns, and therefore, when 
-used with declarative mixins, have the :func:`~sqlalchemy.util.classproperty` 
+used with declarative mixins, have the :func:`.declared_attr` 
 requirement so that no reliance on copying is needed::
 
     class SomethingMixin(object):
 
-        @classproperty
+        @declared_attr
         def dprop(cls):
             return deferred(Column(Integer))
 
     class Something(Base, SomethingMixin):
         __tablename__ = "something"
 
-.. note:: The usage of :func:`~sqlalchemy.util.classproperty` with mixin 
-   mapper properties is a new feature as of SQLAlchemy 0.6.2.
-
 
 Controlling table inheritance with mixins
 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@@ -721,10 +718,10 @@ where you wanted to use that mixin in a single table inheritance
 hierarchy, you can explicitly specify ``__tablename__`` as ``None`` to
 indicate that the class should not have a table mapped::
 
-    from sqlalchemy.util import classproperty
+    from sqlalchemy.ext.declarative import declared_attr
 
     class Tablename:
-        @classproperty
+        @declared_attr
         def __tablename__(cls):
             return cls.__name__.lower()
 
@@ -748,11 +745,11 @@ has a mapped table.
 As an example, here's a mixin that will only allow single table
 inheritance::
 
-    from sqlalchemy.util import classproperty
+    from sqlalchemy.ext.declarative import declared_attr
     from sqlalchemy.ext.declarative import has_inherited_table
 
     class Tablename:
-        @classproperty
+        @declared_attr
         def __tablename__(cls):
             if has_inherited_table(cls):
                 return None
@@ -772,11 +769,11 @@ table inheritance, you would need a slightly different mixin and use
 it on any joined table child classes in addition to their parent
 classes::
 
-    from sqlalchemy.util import classproperty
+    from sqlalchemy.ext.declarative import declared_attr
     from sqlalchemy.ext.declarative import has_inherited_table
 
     class Tablename:
-        @classproperty
+        @declared_attr
         def __tablename__(cls):
             if (has_inherited_table(cls) and
                 Tablename not in cls.__bases__):
@@ -806,14 +803,14 @@ In the case of ``__table_args__`` or ``__mapper_args__``
 specified with declarative mixins, you may want to combine
 some parameters from several mixins with those you wish to
 define on the class iteself. The
-:func:`~sqlalchemy.util.classproperty` decorator can be used
+:func:`.declared_attr` decorator can be used
 here to create user-defined collation routines that pull
 from multiple collections::
 
-    from sqlalchemy.util import classproperty
+    from sqlalchemy.ext.declarative import declared_attr
 
     class MySQLSettings:
-        __table_args__ = {'mysql_engine':'InnoDB'}             
+        __table_args__ = {'mysql_engine':'InnoDB'}
 
     class MyOtherMixin:
         __table_args__ = {'info':'foo'}
@@ -821,7 +818,7 @@ from multiple collections::
     class MyModel(Base,MySQLSettings,MyOtherMixin):
         __tablename__='my_model'
 
-        @classproperty
+        @declared_attr
         def __table_args__(self):
             args = dict()
             args.update(MySQLSettings.__table_args__)
@@ -830,6 +827,7 @@ from multiple collections::
 
         id =  Column(Integer, primary_key=True)
 
+
 Class Constructor
 =================
 
@@ -845,7 +843,7 @@ Sessions
 Note that ``declarative`` does nothing special with sessions, and is
 only intended as an easier way to configure mappers and
 :class:`~sqlalchemy.schema.Table` objects.  A typical application
-setup using :func:`~sqlalchemy.orm.scoped_session` might look like::  
+setup using :func:`~sqlalchemy.orm.scoped_session` might look like::
 
     engine = create_engine('postgresql://scott:tiger@localhost/test')
     Session = scoped_session(sessionmaker(autocommit=False,
@@ -864,8 +862,8 @@ from sqlalchemy.orm import synonym as _orm_synonym, mapper,\
 from sqlalchemy.orm.interfaces import MapperProperty
 from sqlalchemy.orm.properties import RelationshipProperty, ColumnProperty
 from sqlalchemy.orm.util import _is_mapped_class
-from sqlalchemy import util, exceptions
-from sqlalchemy.sql import util as sql_util
+from sqlalchemy import util, exc
+from sqlalchemy.sql import util as sql_util, expression
 
 
 __all__ = 'declarative_base', 'synonym_for', \
@@ -875,10 +873,10 @@ def instrument_declarative(cls, registry, metadata):
     """Given a class, configure the class declaratively,
     using the given registry, which can be any dictionary, and
     MetaData object. 
-    
+
     """
     if '_decl_class_registry' in cls.__dict__:
-        raise exceptions.InvalidRequestError(
+        raise exc.InvalidRequestError(
                             "Class %r already has been "
                             "instrumented declaratively" % cls)
     cls._decl_class_registry = registry
@@ -901,80 +899,106 @@ def _as_declarative(cls, classname, dict_):
 
     column_copies = {}
     potential_columns = {}
-    
+
     mapper_args = {}
     table_args = inherited_table_args = None
     tablename = None
     parent_columns = ()
-    
+
+    declarative_props = (declared_attr, util.classproperty)
+
     for base in cls.__mro__:
-        if _is_mapped_class(base):
+        class_mapped = _is_mapped_class(base)
+        if class_mapped:
             parent_columns = base.__table__.c.keys()
-        else:
-            for name,obj in vars(base).items():
-                if name == '__mapper_args__':
-                    if not mapper_args:
-                        mapper_args = cls.__mapper_args__
-                elif name == '__tablename__':
-                    if not tablename:
-                        tablename = cls.__tablename__
-                elif name == '__table_args__':
-                    if not table_args:                        
-                        table_args = cls.__table_args__
-                        if base is not cls:
-                            inherited_table_args = True
-                elif base is not cls:
-                    # we're a mixin.
-                    
-                    if isinstance(obj, Column):
-                        if obj.foreign_keys:
-                            raise exceptions.InvalidRequestError(
-                            "Columns with foreign keys to other columns "
-                            "must be declared as @classproperty callables "
-                            "on declarative mixin classes. ")
-                        if name not in dict_ and not (
-                                '__table__' in dict_ and 
-                                name in dict_['__table__'].c
-                                ):
-                            potential_columns[name] = \
-                                    column_copies[obj] = \
-                                    obj.copy()
-                            column_copies[obj]._creation_order = \
-                                    obj._creation_order
-                    elif isinstance(obj, MapperProperty):
-                        raise exceptions.InvalidRequestError(
-                            "Mapper properties (i.e. deferred,"
-                            "column_property(), relationship(), etc.) must "
-                            "be declared as @classproperty callables "
-                            "on declarative mixin classes.")
-                    elif isinstance(obj, util.classproperty):
-                        dict_[name] = ret = \
-                                column_copies[obj] = getattr(cls, name)
-                        if isinstance(ret, (Column, MapperProperty)) and \
-                            ret.doc is None:
-                            ret.doc = obj.__doc__
+
+        for name,obj in vars(base).items():
+            if name == '__mapper_args__':
+                if not mapper_args and (
+                                        not class_mapped or 
+                                        isinstance(obj, declarative_props)
+                                    ):
+                    mapper_args = cls.__mapper_args__
+            elif name == '__tablename__':
+                if not tablename and (
+                                        not class_mapped or 
+                                        isinstance(obj, declarative_props)
+                                    ):
+                    tablename = cls.__tablename__
+            elif name == '__table_args__':
+                if not table_args and (
+                                        not class_mapped or 
+                                        isinstance(obj, declarative_props)
+                                    ):
+                    table_args = cls.__table_args__
+                    if not isinstance(table_args, (tuple, dict, type(None))):
+                        raise exc.ArgumentError(
+                                "__table_args__ value must be a tuple, "
+                                "dict, or None")
+                    if base is not cls:
+                        inherited_table_args = True
+            elif class_mapped:
+                continue
+            elif base is not cls:
+                # we're a mixin.
+
+                if isinstance(obj, Column):
+                    if obj.foreign_keys:
+                        raise exc.InvalidRequestError(
+                        "Columns with foreign keys to other columns "
+                        "must be declared as @declared_attr callables "
+                        "on declarative mixin classes. ")
+                    if name not in dict_ and not (
+                            '__table__' in dict_ and 
+                            (obj.name or name) in dict_['__table__'].c
+                            ) and name not in potential_columns:
+                        potential_columns[name] = \
+                                column_copies[obj] = \
+                                obj.copy()
+                        column_copies[obj]._creation_order = \
+                                obj._creation_order
+                elif isinstance(obj, MapperProperty):
+                    raise exc.InvalidRequestError(
+                        "Mapper properties (i.e. deferred,"
+                        "column_property(), relationship(), etc.) must "
+                        "be declared as @declared_attr callables "
+                        "on declarative mixin classes.")
+                elif isinstance(obj, declarative_props):
+                    dict_[name] = ret = \
+                            column_copies[obj] = getattr(cls, name)
+                    if isinstance(ret, (Column, MapperProperty)) and \
+                        ret.doc is None:
+                        ret.doc = obj.__doc__
 
     # apply inherited columns as we should
     for k, v in potential_columns.items():
-        if tablename or k not in parent_columns:
+        if tablename or (v.name or k) not in parent_columns:
             dict_[k] = v
-            
+
     if inherited_table_args and not tablename:
         table_args = None
 
     # make sure that column copies are used rather 
     # than the original columns from any mixins
-    for k, v in mapper_args.iteritems():
-        mapper_args[k] = column_copies.get(v,v)
-    
+    for k in ('version_id_col', 'polymorphic_on',):
+        if k in mapper_args:
+            v = mapper_args[k]
+            mapper_args[k] = column_copies.get(v,v)
+
+    if classname in cls._decl_class_registry:
+        util.warn("The classname %r is already in the registry of this"
+                  " declarative base, mapped to %r" % (
+                classname,
+                cls._decl_class_registry[classname]
+                ))
     cls._decl_class_registry[classname] = cls
     our_stuff = util.OrderedDict()
 
     for k in dict_:
         value = dict_[k]
-        if isinstance(value, util.classproperty):
+        if isinstance(value, declarative_props):
             value = getattr(cls, k)
-            
+
         if (isinstance(value, tuple) and len(value) == 1 and
             isinstance(value[0], (Column, MapperProperty))):
             util.warn("Ignoring declarative-like tuple value of attribute "
@@ -983,6 +1007,12 @@ def _as_declarative(cls, classname, dict_):
             continue
         if not isinstance(value, (Column, MapperProperty)):
             continue
+        if k == 'metadata':
+            raise exc.InvalidRequestError(
+                "Attribute name 'metadata' is reserved "
+                "for the MetaData instance when using a "
+                "declarative base class."
+            )
         prop = _deferred_relationship(cls, value)
         our_stuff[k] = prop
 
@@ -1011,14 +1041,14 @@ def _as_declarative(cls, classname, dict_):
     table = None
     if '__table__' not in dict_:
         if tablename is not None:
-            
+
             if isinstance(table_args, dict):
                 args, table_kw = (), table_args
             elif isinstance(table_args, tuple):
                 args = table_args[0:-1]
                 table_kw = table_args[-1]
                 if len(table_args) < 2 or not isinstance(table_kw, dict):
-                    raise exceptions.ArgumentError(
+                    raise exc.ArgumentError(
                         "Tuple form of __table_args__ is "
                         "(arg1, arg2, arg3, ..., {'kw1':val1, "
                         "'kw2':val2, ...})"
@@ -1038,11 +1068,11 @@ def _as_declarative(cls, classname, dict_):
         if cols:
             for c in cols:
                 if not table.c.contains_column(c):
-                    raise exceptions.ArgumentError(
+                    raise exc.ArgumentError(
                         "Can't add additional column %r when "
                         "specifying __table__" % c.key
                     )
-    
+
     if 'inherits' not in mapper_args:
         for c in cls.__bases__:
             if _is_mapped_class(c):
@@ -1056,7 +1086,7 @@ def _as_declarative(cls, classname, dict_):
         mapper_cls = mapper
 
     if table is None and 'inherits' not in mapper_args:
-        raise exceptions.InvalidRequestError(
+        raise exc.InvalidRequestError(
             "Class %r does not have a __table__ or __tablename__ "
             "specified and does not inherit from an existing "
             "table-mapped class." % cls
@@ -1066,39 +1096,31 @@ def _as_declarative(cls, classname, dict_):
         inherited_mapper = class_mapper(mapper_args['inherits'],
                                             compile=False)
         inherited_table = inherited_mapper.local_table
-        if 'inherit_condition' not in mapper_args and table is not None:
-            # figure out the inherit condition with relaxed rules
-            # about nonexistent tables, to allow for ForeignKeys to
-            # not-yet-defined tables (since we know for sure that our
-            # parent table is defined within the same MetaData)
-            mapper_args['inherit_condition'] = sql_util.join_condition(
-                mapper_args['inherits'].__table__, table,
-                ignore_nonexistent_tables=True)
 
         if table is None:
             # single table inheritance.
             # ensure no table args
             if table_args:
-                raise exceptions.ArgumentError(
+                raise exc.ArgumentError(
                     "Can't place __table_args__ on an inherited class "
                     "with no table."
                     )
-        
+
             # add any columns declared here to the inherited table.
             for c in cols:
                 if c.primary_key:
-                    raise exceptions.ArgumentError(
+                    raise exc.ArgumentError(
                         "Can't place primary key columns on an inherited "
                         "class with no table."
                         )
                 if c.name in inherited_table.c:
-                    raise exceptions.ArgumentError(
+                    raise exc.ArgumentError(
                         "Column '%s' on class %s conflicts with "
                         "existing column '%s'" % 
                         (c, cls, inherited_table.c[c.name])
                     )
                 inherited_table.append_column(c)
-    
+
         # single or joined inheritance
         # exclude any cols on the inherited table which are not mapped on the
         # parent class, to avoid
@@ -1106,13 +1128,32 @@ def _as_declarative(cls, classname, dict_):
         inherited_mapper = class_mapper(mapper_args['inherits'],
                                             compile=False)
         inherited_table = inherited_mapper.local_table
-        
+
         if 'exclude_properties' not in mapper_args:
             mapper_args['exclude_properties'] = exclude_properties = \
                 set([c.key for c in inherited_table.c
                      if c not in inherited_mapper._columntoproperty])
             exclude_properties.difference_update([c.key for c in cols])
-    
+
+        # look through columns in the current mapper that 
+        # are keyed to a propname different than the colname
+        # (if names were the same, we'd have popped it out above,
+        # in which case the mapper makes this combination).
+        # See if the superclass has a similar column property.
+        # If so, join them together.
+        for k, col in our_stuff.items():
+            if not isinstance(col, expression.ColumnElement):
+                continue
+            if k in inherited_mapper._props:
+                p = inherited_mapper._props[k]
+                if isinstance(p, ColumnProperty):
+                    # note here we place the superclass column
+                    # first.  this corresponds to the 
+                    # append() in mapper._configure_property().
+                    # change this ordering when we do [ticket:1892]
+                    our_stuff[k] = p.columns + [col]
+
+
     cls.__mapper__ = mapper_cls(cls, 
                                 table, 
                                 properties=our_stuff, 
@@ -1153,16 +1194,16 @@ class _GetColumns(object):
     def __init__(self, cls):
         self.cls = cls
     def __getattr__(self, key):
-        
+
         mapper = class_mapper(self.cls, compile=False)
         if mapper:
             prop = mapper.get_property(key, raiseerr=False)
             if prop is None:
-                raise exceptions.InvalidRequestError(
+                raise exc.InvalidRequestError(
                             "Class %r does not have a mapped column named %r"
                             % (self.cls, key))
             elif not isinstance(prop, ColumnProperty):
-                raise exceptions.InvalidRequestError(
+                raise exc.InvalidRequestError(
                             "Property %r is not an instance of"
                             " ColumnProperty (i.e. does not correspond"
                             " directly to a Column)." % key)
@@ -1172,7 +1213,7 @@ class _GetColumns(object):
 def _deferred_relationship(cls, prop):
     def resolve_arg(arg):
         import sqlalchemy
-        
+
         def access_cls(key):
             if key in cls._decl_class_registry:
                 return _GetColumns(cls._decl_class_registry[key])
@@ -1185,14 +1226,14 @@ def _deferred_relationship(cls, prop):
         def return_cls():
             try:
                 x = eval(arg, globals(), d)
-                
+
                 if isinstance(x, _GetColumns):
                     return x.cls
                 else:
                     return x
             except NameError, n:
-                raise exceptions.InvalidRequestError(
-                    "When compiling mapper %s, expression %r failed to "
+                raise exc.InvalidRequestError(
+                    "When initializing mapper %s, expression %r failed to "
                     "locate a name (%r). If this is a class name, consider "
                     "adding this relationship() to the %r class after "
                     "both dependent classes have been defined." % 
@@ -1261,6 +1302,63 @@ def comparable_using(comparator_factory):
         return comparable_property(comparator_factory, fn)
     return decorate
 
+class declared_attr(property):
+    """Mark a class-level method as representing the definition of
+    a mapped property or special declarative member name.
+
+    .. note:: @declared_attr is available as 
+      ``sqlalchemy.util.classproperty`` for SQLAlchemy versions
+      0.6.2, 0.6.3, 0.6.4.
+
+    @declared_attr turns the attribute into a scalar-like
+    property that can be invoked from the uninstantiated class.
+    Declarative treats attributes specifically marked with 
+    @declared_attr as returning a construct that is specific
+    to mapping or declarative table configuration.  The name
+    of the attribute is that of what the non-dynamic version
+    of the attribute would be.
+
+    @declared_attr is more often than not applicable to mixins,
+    to define relationships that are to be applied to different
+    implementors of the class::
+
+        class ProvidesUser(object):
+            "A mixin that adds a 'user' relationship to classes."
+
+            @declared_attr
+            def user(self):
+                return relationship("User")
+
+    It also can be applied to mapped classes, such as to provide
+    a "polymorphic" scheme for inheritance::
+
+        class Employee(Base):
+            id = Column(Integer, primary_key=True)
+            type = Column(String(50), nullable=False)
+
+            @declared_attr
+            def __tablename__(cls):
+                return cls.__name__.lower()
+
+            @declared_attr
+            def __mapper_args__(cls):
+                if cls.__name__ == 'Employee':
+                    return {
+                            "polymorphic_on":cls.type, 
+                            "polymorphic_identity":"Employee"
+                    }
+                else:
+                    return {"polymorphic_identity":cls.__name__}
+
+    """
+
+    def __init__(self, fget, *arg, **kw):
+        super(declared_attr, self).__init__(fget, *arg, **kw)
+        self.__doc__ = fget.__doc__
+
+    def __get__(desc, self, cls):
+        return desc.fget(cls)
+
 def _declarative_constructor(self, **kwargs):
     """A simple constructor that allows initialization from kwargs.
 
