@@ -1,6 +1,5 @@
-# properties.py
-# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Michael Bayer
-# mike_mp@zzzcomputing.com
+# orm/properties.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -60,7 +59,8 @@ class ColumnProperty(StrategizedProperty):
                                             self.__class__.Comparator)
         self.descriptor = kwargs.pop('descriptor', None)
         self.extension = kwargs.pop('extension', None)
-        
+        self.active_history = kwargs.pop('active_history', False)
+
         if 'doc' in kwargs:
             self.doc = kwargs.pop('doc')
         else:
@@ -71,7 +71,7 @@ class ColumnProperty(StrategizedProperty):
                     break
             else:
                 self.doc = None
-        
+
         if kwargs:
             raise TypeError(
                 "%s received unexpected keyword argument(s): %s" % (
@@ -85,11 +85,11 @@ class ColumnProperty(StrategizedProperty):
             self.strategy_class = strategies.DeferredColumnLoader
         else:
             self.strategy_class = strategies.ColumnLoader
-    
+
     def instrument_class(self, mapper):
         if not self.instrument:
             return
-        
+
         attributes.register_descriptor(
             mapper.class_, 
             self.key, 
@@ -98,7 +98,7 @@ class ColumnProperty(StrategizedProperty):
             property_=self,
             doc=self.doc
             )
-        
+
     def do_init(self):
         super(ColumnProperty, self).do_init()
         if len(self.columns) > 1 and \
@@ -114,10 +114,11 @@ class ColumnProperty(StrategizedProperty):
         return ColumnProperty(
                         deferred=self.deferred, 
                         group=self.group, 
+                        active_history=self.active_history,
                         *self.columns)
 
-    def _getattr(self, state, dict_, column):
-        return state.get_impl(self.key).get(state, dict_)
+    def _getattr(self, state, dict_, column, passive=False):
+        return state.get_impl(self.key).get(state, dict_, passive=passive)
 
     def _getcommitted(self, state, dict_, column, passive=False):
         return state.get_impl(self.key).\
@@ -130,7 +131,7 @@ class ColumnProperty(StrategizedProperty):
                                 dest_dict, load, _recursive):
         if self.key in source_dict:
             value = source_dict[self.key]
-        
+
             if not load:
                 dest_dict[self.key] = value
             else:
@@ -139,7 +140,7 @@ class ColumnProperty(StrategizedProperty):
         else:
             if dest_state.has_identity and self.key not in dest_dict:
                 dest_state.expire_attributes(dest_dict, [self.key])
-                
+
     def get_col_value(self, column, value):
         return value
 
@@ -152,17 +153,17 @@ class ColumnProperty(StrategizedProperty):
                 return self.prop.columns[0]._annotate({
                                                 "parententity": self.mapper,
                                                 "parentmapper":self.mapper})
-                
+
         def operate(self, op, *other, **kwargs):
             return op(self.__clause_element__(), *other, **kwargs)
 
         def reverse_operate(self, op, other, **kwargs):
             col = self.__clause_element__()
             return op(col._bind_param(op, other), col, **kwargs)
-    
+
     # TODO: legacy..do we need this ? (0.5)
     ColumnComparator = Comparator
-    
+
     def __str__(self):
         return str(self.parent.class_.__name__) + "." + self.key
 
@@ -170,7 +171,7 @@ log.class_logger(ColumnProperty)
 
 class CompositeProperty(ColumnProperty):
     """subclasses ColumnProperty to provide composite type support."""
-    
+
     def __init__(self, class_, *columns, **kwargs):
         super(CompositeProperty, self).__init__(*columns, **kwargs)
         self._col_position_map = util.column_dict(
@@ -184,6 +185,7 @@ class CompositeProperty(ColumnProperty):
                         deferred=self.deferred, 
                         group=self.group,
                         composite_class=self.composite_class, 
+                        active_history=self.active_history,
                         *self.columns)
 
     def do_init(self):
@@ -191,8 +193,8 @@ class CompositeProperty(ColumnProperty):
         # which issues assertions that do not apply to CompositeColumnProperty
         super(ColumnProperty, self).do_init()
 
-    def _getattr(self, state, dict_, column):
-        obj = state.get_impl(self.key).get(state, dict_)
+    def _getattr(self, state, dict_, column, passive=False):
+        obj = state.get_impl(self.key).get(state, dict_, passive=passive)
         return self.get_col_value(column, obj)
 
     def _getcommitted(self, state, dict_, column, passive=False):
@@ -214,12 +216,12 @@ class CompositeProperty(ColumnProperty):
             obj.__set_composite_values__(*values)
         else:
             setattr(obj, column.key, value)
-            
+
     def get_col_value(self, column, value):
         if value is None:
             return None
         for a, b in zip(self.columns, value.__composite_values__()):
-            if a is column:
+            if a.shares_lineage(column): 
                 return b
 
     class Comparator(PropComparator):
@@ -230,9 +232,9 @@ class CompositeProperty(ColumnProperty):
                             *[self.adapter(x) for x in self.prop.columns])
             else:
                 return expression.ClauseList(*self.prop.columns)
-        
+
         __hash__ = None
-        
+
         def __eq__(self, other):
             if other is None:
                 values = [None] * len(self.prop.columns)
@@ -240,7 +242,7 @@ class CompositeProperty(ColumnProperty):
                 values = other.__composite_values__()
             return sql.and_(
                     *[a==b for a, b in zip(self.prop.columns, values)])
-            
+
         def __ne__(self, other):
             return sql.not_(self.__eq__(other))
 
@@ -255,7 +257,7 @@ class DescriptorProperty(MapperProperty):
         pass
 
     def create_row_processor(self, selectcontext, path, mapper, row, adapter):
-        return (None, None)
+        return None, None, None
 
     def merge(self, session, source_state, source_dict, 
                 dest_state, dest_dict, load, _recursive):
@@ -266,9 +268,9 @@ class ConcreteInheritedProperty(DescriptorProperty):
     """A 'do nothing' :class:`MapperProperty` that disables 
     an attribute on a concrete subclass that is only present
     on the inherited mapper, not the concrete classes' mapper.
-    
+
     Cases where this occurs include:
-    
+
     * When the superclass mapper is mapped against a 
       "polymorphic union", which includes all attributes from 
       all subclasses.
@@ -276,9 +278,9 @@ class ConcreteInheritedProperty(DescriptorProperty):
       but not on the subclass mapper.  Concrete mappers require
       that relationship() is configured explicitly on each 
       subclass. 
-    
+
     """
-    
+
     def instrument_class(self, mapper):
         def warn():
             raise AttributeError("Concrete %s does not implement "
@@ -393,7 +395,7 @@ class SynonymProperty(DescriptorProperty):
             doc=self.doc
             )
 
-        
+
 class ComparableProperty(DescriptorProperty):
     """Instruments a Python property for use in query expressions."""
 
@@ -405,7 +407,7 @@ class ComparableProperty(DescriptorProperty):
 
     def instrument_class(self, mapper):
         """Set up a proxy to the unmanaged descriptor."""
-        
+
         if self.descriptor is None:
             desc = getattr(mapper.class_, self.key, None)
             if mapper._is_userland_descriptor(desc):
@@ -444,7 +446,11 @@ class RelationshipProperty(StrategizedProperty):
         comparator_factory=None,
         single_parent=False, innerjoin=False,
         doc=None,
-        strategy_class=None, _local_remote_pairs=None, query_class=None):
+        active_history=False,
+        cascade_backrefs=True,
+        load_on_pending=False,
+        strategy_class=None, _local_remote_pairs=None, 
+        query_class=None):
 
         self.uselist = uselist
         self.argument = argument
@@ -459,20 +465,23 @@ class RelationshipProperty(StrategizedProperty):
         self._user_defined_foreign_keys = foreign_keys
         self.collection_class = collection_class
         self.passive_deletes = passive_deletes
+        self.cascade_backrefs = cascade_backrefs
         self.passive_updates = passive_updates
         self.remote_side = remote_side
         self.enable_typechecks = enable_typechecks
         self.query_class = query_class
         self.innerjoin = innerjoin
         self.doc = doc
+        self.active_history = active_history
         self.join_depth = join_depth
         self.local_remote_pairs = _local_remote_pairs
         self.extension = extension
+        self.load_on_pending = load_on_pending
         self.comparator_factory = comparator_factory or \
                                     RelationshipProperty.Comparator
         self.comparator = self.comparator_factory(self, None)
         util.set_creation_order(self)
-        
+
         if strategy_class:
             self.strategy_class = strategy_class
         elif self.lazy== 'dynamic':
@@ -480,7 +489,7 @@ class RelationshipProperty(StrategizedProperty):
             self.strategy_class = dynamic.DynaLoader
         else:
             self.strategy_class = strategies.factory(self.lazy)
-            
+
         self._reverse_property = set()
 
         if cascade is not False:
@@ -530,13 +539,13 @@ class RelationshipProperty(StrategizedProperty):
             """Return a copy of this PropComparator which will use the
             given adaption function on the local side of generated
             expressions.
-            
+
             """
 
             return self.__class__(self.property, self.mapper,
                                   getattr(self, '_of_type', None),
                                   adapter)
-        
+
         @property
         def parententity(self):
             return self.property.parent
@@ -564,9 +573,9 @@ class RelationshipProperty(StrategizedProperty):
             raise NotImplementedError('in_() not yet supported for '
                     'relationships.  For a simple many-to-one, use '
                     'in_() against the set of foreign key values.')
-            
+
         __hash__ = None
-        
+
         def __eq__(self, other):
             if isinstance(other, (NoneType, expression._Null)):
                 if self.property.direction in [ONETOMANY, MANYTOMANY]:
@@ -602,7 +611,7 @@ class RelationshipProperty(StrategizedProperty):
                 source_selectable = self.__clause_element__()
             else:
                 source_selectable = None
-            
+
             pj, sj, source, dest, secondary, target_adapter = \
                 self.property._create_joins(dest_polymorphic=True,
                         dest_selectable=to_selectable,
@@ -614,7 +623,7 @@ class RelationshipProperty(StrategizedProperty):
                     criterion = crit
                 else:
                     criterion = criterion & crit
-            
+
             # annotate the *local* side of the join condition, in the case
             # of pj + sj this is the full primaryjoin, in the case of just
             # pj its the local side of the primaryjoin.
@@ -622,7 +631,7 @@ class RelationshipProperty(StrategizedProperty):
                 j = _orm_annotate(pj) & sj
             else:
                 j = _orm_annotate(pj, exclude=self.property.remote_side)
-            
+
             if criterion is not None and target_adapter:
                 # limit this adapter to annotated only?
                 criterion = target_adapter.traverse(criterion)
@@ -634,9 +643,9 @@ class RelationshipProperty(StrategizedProperty):
             # to anything in the enclosing query.
             if criterion is not None:
                 criterion = criterion._annotate({'_halt_adapt': True})
-            
+
             crit = j & criterion
-            
+
             return sql.exists([1], crit, from_obj=dest).correlate(source)
 
         def any(self, criterion=None, **kwargs):
@@ -672,26 +681,26 @@ class RelationshipProperty(StrategizedProperty):
         def __negated_contains_or_equals(self, other):
             if self.property.direction == MANYTOONE:
                 state = attributes.instance_state(other)
-                
+
                 def state_bindparam(state, col):
                     o = state.obj() # strong ref
                     return lambda : \
                         self.property.mapper._get_committed_attr_by_column(o,
                             col)
-                
+
                 def adapt(col):
                     if self.adapter:
                         return self.adapter(col)
                     else:
                         return col
-                        
+
                 if self.property._use_get:
                     return sql.and_(*[
                         sql.or_(
                         adapt(x) != state_bindparam(state, y),
                         adapt(x) == None)
                         for (x, y) in self.property.local_remote_pairs])
-                    
+
             criterion = sql.and_(*[x==y for (x, y) in 
                                 zip(
                                     self.property.mapper.primary_key,
@@ -722,8 +731,7 @@ class RelationshipProperty(StrategizedProperty):
 
     def compare(self, op, value, 
                             value_is_parent=False, 
-                            alias_secondary=True,
-                            detect_transient_pending=False):
+                            alias_secondary=True):
         if op == operators.eq:
             if value is None:
                 if self.uselist:
@@ -731,26 +739,22 @@ class RelationshipProperty(StrategizedProperty):
                 else:
                     return self._optimized_compare(None, 
                                     value_is_parent=value_is_parent,
-                                    detect_transient_pending=detect_transient_pending,
                                     alias_secondary=alias_secondary)
             else:
                 return self._optimized_compare(value, 
                                 value_is_parent=value_is_parent,
-                                detect_transient_pending=detect_transient_pending,
                                 alias_secondary=alias_secondary)
         else:
             return op(self.comparator, value)
 
     def _optimized_compare(self, value, value_is_parent=False, 
                                     adapt_source=None, 
-                                    detect_transient_pending=False,
                                     alias_secondary=True):
         if value is not None:
             value = attributes.instance_state(value)
         return self._get_strategy(strategies.LazyLoader).lazy_clause(value,
                 reverse_direction=not value_is_parent,
                 alias_secondary=alias_secondary,
-                detect_transient_pending=detect_transient_pending,
                 adapt_source=adapt_source)
 
     def __str__(self):
@@ -785,11 +789,11 @@ class RelationshipProperty(StrategizedProperty):
             if load:
                 # for a full merge, pre-load the destination collection,
                 # so that individual _merge of each item pulls from identity
-                # map for those already present.  
+                # map for those already present.
                 # also assumes CollectionAttrbiuteImpl behavior of loading
                 # "old" list in any case
                 dest_state.get_impl(self.key).get(dest_state, dest_dict)
-                
+
             dest_list = []
             for current in instances:
                 current_state = attributes.instance_state(current)
@@ -799,7 +803,7 @@ class RelationshipProperty(StrategizedProperty):
                         load=load, _recursive=_recursive)
                 if obj is not None:
                     dest_list.append(obj)
-                    
+
             if not load:
                 coll = attributes.init_state_collection(dest_state,
                         dest_dict, self.key)
@@ -842,14 +846,14 @@ class RelationshipProperty(StrategizedProperty):
                     passive=passive)
         skip_pending = type_ == 'refresh-expire' and 'delete-orphan' \
             not in self.cascade
-        
+
         if instances:
             for c in instances:
                 if c is not None and \
                     c is not attributes.PASSIVE_NO_RESULT and \
                     c not in visited_instances and \
                     (halt_on is None or not halt_on(c)):
-                    
+
                     if not isinstance(c, self.mapper.class_):
                         raise AssertionError("Attribute '%s' on class '%s' "
                                             "doesn't handle objects "
@@ -859,22 +863,23 @@ class RelationshipProperty(StrategizedProperty):
                                                 str(c.__class__)
                                             ))
                     instance_state = attributes.instance_state(c)
-                    
+
                     if skip_pending and not instance_state.key:
                         continue
-                        
+
                     visited_instances.add(c)
 
                     # cascade using the mapper local to this 
                     # object, so that its individual properties are located
                     instance_mapper = instance_state.manager.mapper
-                    yield (c, instance_mapper, instance_state)
+                    yield c, instance_mapper, instance_state
+
 
     def _add_reverse_property(self, key):
         other = self.mapper.get_property(key, _compile_mappers=False)
         self._reverse_property.add(other)
         other._reverse_property.add(self)
-        
+
         if not other._get_target().common_parent(self.parent):
             raise sa_exc.ArgumentError('reverse_property %r on '
                     'relationship %s references relationship %s, which '
@@ -886,7 +891,7 @@ class RelationshipProperty(StrategizedProperty):
                     'both of the same direction %r.  Did you mean to '
                     'set remote_side on the many-to-one side ?'
                     % (other, self, self.direction))
-        
+
     def do_init(self):
         self._get_target()
         self._assert_is_primary()
@@ -919,7 +924,7 @@ class RelationshipProperty(StrategizedProperty):
                         % (self.key, type(self.argument)))
             assert isinstance(self.mapper, mapper.Mapper), self.mapper
         return self.mapper
-        
+
     def _process_dependent_arguments(self):
 
         # accept callables for other attributes which may require
@@ -961,7 +966,7 @@ class RelationshipProperty(StrategizedProperty):
                 if inheriting is not self.parent \
                     and inheriting.has_property(self.key):
                     util.warn("Warning: relationship '%s' on mapper "
-                              "'%s' supercedes the same relationship "
+                              "'%s' supersedes the same relationship "
                               "on inherited mapper '%s'; this can "
                               "cause dependency issues during flush"
                               % (self.key, self.parent, inheriting))
@@ -1033,25 +1038,25 @@ class RelationshipProperty(StrategizedProperty):
 
     def _sync_pairs_from_join(self, join_condition, primary):
         """Given a join condition, figure out what columns are foreign
-        and are part of a binary "equated" condition to their referecned
+        and are part of a binary "equated" condition to their referenced
         columns, and convert into a list of tuples of (primary col->foreign col).
-        
+
         Make several attempts to determine if cols are compared using 
         "=" or other comparators (in which case suggest viewonly), 
         columns are present but not part of the expected mappings, columns
         don't have any :class:`ForeignKey` information on them, or 
         the ``foreign_keys`` attribute is being used incorrectly.
-        
+
         """
         eq_pairs = criterion_as_pairs(join_condition,
                 consider_as_foreign_keys=self._user_defined_foreign_keys,
                 any_operator=self.viewonly)
-                
+
         eq_pairs = [(l, r) for (l, r) in eq_pairs
                     if self._col_is_part_of_mappings(l)
                     and self._col_is_part_of_mappings(r)
                     or self.viewonly and r in self._user_defined_foreign_keys]
-        
+
         if not eq_pairs and \
                 self.secondary is not None and \
                 not self._user_defined_foreign_keys:
@@ -1074,21 +1079,34 @@ class RelationshipProperty(StrategizedProperty):
                                 join_condition,
                                 self
                             ))
-                    
+
         if not eq_pairs:
             if not self.viewonly and criterion_as_pairs(join_condition,
                     consider_as_foreign_keys=self._user_defined_foreign_keys,
                     any_operator=True):
-                raise sa_exc.ArgumentError("Could not locate any "
-                        "equated, locally mapped column pairs for %s "
-                        "condition '%s' on relationship %s. For more "
-                        "relaxed rules on join conditions, the "
-                        "relationship may be marked as viewonly=True."
-                        % (
+
+                err = "Could not locate any "\
+                        "foreign-key-equated, locally mapped column "\
+                        "pairs for %s "\
+                        "condition '%s' on relationship %s." % (
                             primary and 'primaryjoin' or 'secondaryjoin', 
                             join_condition, 
                             self
-                        ))
+                        )
+
+                if not self._user_defined_foreign_keys:
+                    err += "  Ensure that the "\
+                            "referencing Column objects have a "\
+                            "ForeignKey present, or are otherwise part "\
+                            "of a ForeignKeyConstraint on their parent "\
+                            "Table, or specify the foreign_keys parameter "\
+                            "to this relationship."
+
+                err += "  For more "\
+                        "relaxed rules on join conditions, the "\
+                        "relationship may be marked as viewonly=True."
+
+                raise sa_exc.ArgumentError(err)
             else:
                 if self._user_defined_foreign_keys:
                     raise sa_exc.ArgumentError("Could not determine "
@@ -1115,7 +1133,8 @@ class RelationshipProperty(StrategizedProperty):
                             "referencing Column objects have a "
                             "ForeignKey present, or are otherwise part "
                             "of a ForeignKeyConstraint on their parent "
-                            "Table." 
+                            "Table, or specify the foreign_keys parameter " 
+                            "to this relationship."
                             % (
                                 primary and 'primaryjoin' or 'secondaryjoin', 
                                 join_condition, 
@@ -1215,7 +1234,11 @@ class RelationshipProperty(StrategizedProperty):
                       'when single_parent is not set.   Set '
                       'single_parent=True on the relationship().'
                       % self)
-        
+        if self.direction is MANYTOONE and self.passive_deletes:
+            util.warn("On %s, 'passive_deletes' is normally configured "
+                      "on one-to-many, one-to-one, many-to-many relationships only."
+                       % self)
+
     def _determine_local_remote_pairs(self):
         if not self.local_remote_pairs:
             if self.remote_side:
@@ -1288,7 +1311,7 @@ class RelationshipProperty(StrategizedProperty):
                     "created for class '%s' " % (self.key,
                     self.parent.class_.__name__,
                     self.parent.class_.__name__))
-            
+
     def _generate_backref(self):
         if not self.is_primary():
             return
@@ -1337,7 +1360,7 @@ class RelationshipProperty(StrategizedProperty):
             self.extension.append(
                     attributes.GenericBackrefExtension(self.back_populates))
             self._add_reverse_property(self.back_populates)
-        
+
     def _post_init(self):
         self.logger.info('%s setup primary join %s', self,
                          self.primaryjoin)
@@ -1359,7 +1382,7 @@ class RelationshipProperty(StrategizedProperty):
         if not self.viewonly:
             self._dependency_processor = \
                 dependency.DependencyProcessor.from_relationship(self)
-    
+
     @util.memoized_property
     def _use_get(self):
         """memoize the 'use_get' attribute of this RelationshipLoader's
@@ -1367,10 +1390,17 @@ class RelationshipProperty(StrategizedProperty):
 
         strategy = self._get_strategy(strategies.LazyLoader)
         return strategy.use_get
-        
+
     def _refers_to_parent_table(self):
+        pt = self.parent.mapped_table
+        mt = self.mapper.mapped_table
         for c, f in self.synchronize_pairs:
-            if c.table is f.table:
+            if (
+                pt.is_derived_from(c.table) and \
+                pt.is_derived_from(f.table) and \
+                mt.is_derived_from(c.table) and \
+                mt.is_derived_from(f.table)
+            ):
                 return True
         else:
             return False
@@ -1407,21 +1437,21 @@ class RelationshipProperty(StrategizedProperty):
 
         primaryjoin, secondaryjoin, secondary = self.primaryjoin, \
             self.secondaryjoin, self.secondary
-        
+
         # adjust the join condition for single table inheritance,
         # in the case that the join is to a subclass
-        # this is analgous to the "_adjust_for_single_table_inheritance()"
+        # this is analogous to the "_adjust_for_single_table_inheritance()"
         # method in Query.
 
         dest_mapper = of_type or self.mapper
-        
+
         single_crit = dest_mapper._single_table_criterion
         if single_crit is not None:
             if secondaryjoin is not None:
                 secondaryjoin = secondaryjoin & single_crit
             else:
                 primaryjoin = primaryjoin & single_crit
-            
+
         if aliased:
             if secondary is not None:
                 secondary = secondary.alias()
@@ -1478,6 +1508,3 @@ class RelationshipProperty(StrategizedProperty):
 PropertyLoader = RelationProperty = RelationshipProperty
 log.class_logger(RelationshipProperty)
 
-mapper.ColumnProperty = ColumnProperty
-mapper.RelationshipProperty = RelationshipProperty
-mapper.ConcreteInheritedProperty = ConcreteInheritedProperty
