@@ -1,10 +1,16 @@
+# orm/state.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
 from sqlalchemy.util import EMPTY_SET
 import weakref
 from sqlalchemy import util
 from sqlalchemy.orm.attributes import PASSIVE_NO_RESULT, PASSIVE_OFF, \
-                                        NEVER_SET, NO_VALUE, manager_of_class, \
-                                        ATTR_WAS_SET
-from sqlalchemy.orm import attributes, exc as orm_exc, interfaces
+                        NEVER_SET, NO_VALUE, manager_of_class, ATTR_WAS_SET
+from sqlalchemy.orm import attributes, exc as orm_exc, interfaces, \
+                        util as orm_util
 
 import sys
 attributes.state = sys.modules['sqlalchemy.orm.state']
@@ -22,7 +28,8 @@ class InstanceState(object):
     _strong_obj = None
     modified = False
     expired = False
-    
+    deleted = False
+
     def __init__(self, obj, manager):
         self.class_ = obj.__class__
         self.manager = manager
@@ -31,7 +38,7 @@ class InstanceState(object):
     @util.memoized_property
     def committed_state(self):
         return {}
-    
+
     @util.memoized_property
     def parents(self):
         return {}
@@ -47,7 +54,7 @@ class InstanceState(object):
     @property
     def has_identity(self):
         return bool(self.key)
-        
+
     def detach(self):
         if self.session_id:
             try:
@@ -58,7 +65,7 @@ class InstanceState(object):
     def dispose(self):
         self.detach()
         del self.obj
-    
+
     def _cleanup(self, ref):
         instance_dict = self._instance_dict()
         if instance_dict:
@@ -69,10 +76,10 @@ class InstanceState(object):
         # remove possible cycles
         self.__dict__.pop('callables', None)
         self.dispose()
-    
+
     def obj(self):
         return None
-    
+
     @property
     def dict(self):
         o = self.obj()
@@ -80,7 +87,7 @@ class InstanceState(object):
             return attributes.instance_dict(o)
         else:
             return {}
-        
+
     @property
     def sort_key(self):
         return self.key and self.key[1] or (self.insert_order, )
@@ -91,7 +98,7 @@ class InstanceState(object):
 
         for fn in manager.events.on_init:
             fn(self, instance, args, kwargs)
-            
+
         # LESSTHANIDEAL:
         # adjust for the case where the InstanceState was created before
         # mapper compilation, and this actually needs to be a MutableAttrInstanceState
@@ -99,7 +106,7 @@ class InstanceState(object):
             self.__class__ = MutableAttrInstanceState
             self.obj = weakref.ref(self.obj(), self._cleanup)
             self.mutable_dict = {}
-            
+
         try:
             return manager.events.original_init(*mixed[1:], **kwargs)
         except:
@@ -139,7 +146,7 @@ class InstanceState(object):
 
     def _run_on_load(self, instance):
         self.manager.events.run('on_load', instance)
-    
+
     def __getstate__(self):
         d = {'instance':self.obj()}
 
@@ -152,7 +159,7 @@ class InstanceState(object):
         if self.load_path:
             d['load_path'] = interfaces.serialize_path(self.load_path)
         return d
-        
+
     def __setstate__(self, state):
         self.obj = weakref.ref(state['instance'], self._cleanup)
         self.class_ = state['instance'].__class__
@@ -165,17 +172,17 @@ class InstanceState(object):
                         self.class_)
         elif manager.is_mapped and not manager.mapper.compiled:
             manager.mapper.compile()
-            
+
         self.committed_state = state.get('committed_state', {})
         self.pending = state.get('pending', {})
         self.parents = state.get('parents', {})
         self.modified = state.get('modified', False)
         self.expired = state.get('expired', False)
         self.callables = state.get('callables', {})
-            
+
         if self.modified:
             self._strong_obj = state['instance']
-            
+
         self.__dict__.update([
             (k, state[k]) for k in (
                 'key', 'load_options', 'mutable_dict'
@@ -188,7 +195,7 @@ class InstanceState(object):
     def initialize(self, key):
         """Set this attribute to an empty value or collection, 
            based on the AttributeImpl in use."""
-        
+
         self.manager.get_impl(key).initialize(self, self.dict)
 
     def reset(self, dict_, key):
@@ -213,15 +220,15 @@ class InstanceState(object):
     def set_callable(self, dict_, key, callable_):
         """Remove the given attribute and set the given callable
            as a loader."""
-           
+
         dict_.pop(key, None)
         self.callables[key] = callable_
-    
+
     def expire_attributes(self, dict_, attribute_names, instance_dict=None):
         """Expire all or a group of attributes.
-        
+
         If all attributes are expired, the "expired" flag is set to True.
-        
+
         """
         # we would like to assert that 'self.key is not None' here, 
         # but there are many cases where the mapper will expire
@@ -230,7 +237,7 @@ class InstanceState(object):
         # occurs fully, within the flush(), before this key is assigned.
         # the key is assigned late within the flush() to assist in
         # "key switch" bookkeeping scenarios.
-        
+
         if attribute_names is None:
             attribute_names = self.manager.keys()
             self.expired = True
@@ -252,14 +259,14 @@ class InstanceState(object):
             self.__dict__.get('committed_state', None),
             self.mutable_dict
         )
-        
+
         for key in attribute_names:
             impl = self.manager[key].impl
             if impl.accepts_scalar_loader and \
                 (not filter_deferred or impl.expire_missing or key in dict_):
                 self.callables[key] = self
             dict_.pop(key, None)
-            
+
             for d in to_clear:
                 if d is not None:
                     d.pop(key, None)
@@ -273,10 +280,10 @@ class InstanceState(object):
 
         if kw.get('passive') is attributes.PASSIVE_NO_FETCH:
             return attributes.PASSIVE_NO_RESULT
-        
+
         toload = self.expired_attributes.\
                         intersection(self.unmodified)
-        
+
         self.manager.deferred_scalar_loader(self, toload)
 
         # if the loader failed, or this 
@@ -285,13 +292,13 @@ class InstanceState(object):
         # dict.  ensure they are removed.
         for k in toload.intersection(self.callables):
             del self.callables[k]
-            
+
         return ATTR_WAS_SET
 
     @property
     def unmodified(self):
         """Return the set of keys which have no uncommitted changes"""
-        
+
         return set(self.manager).difference(self.committed_state)
 
     @property
@@ -310,11 +317,11 @@ class InstanceState(object):
     def expired_attributes(self):
         """Return the set of keys which are 'expired' to be loaded by
            the manager's deferred scalar loader, assuming no pending 
-           changes.  
-           
+           changes.
+
            see also the ``unmodified`` collection which is intersected
            against this set when a refresh operation occurs.
-           
+
         """
         return set([k for k, v in self.callables.items() if v is self])
 
@@ -323,7 +330,7 @@ class InstanceState(object):
 
     def _is_really_none(self):
         return self.obj()
-        
+
     def modified_event(self, dict_, attr, should_copy, previous, passive=PASSIVE_OFF):
         if attr.key not in self.committed_state:
             if previous is NEVER_SET:
@@ -338,13 +345,28 @@ class InstanceState(object):
 
             self.committed_state[attr.key] = previous
 
-        if not self.modified:
+
+        # the "or not self.modified" is defensive at 
+        # this point.  The assertion below is expected
+        # to be True:
+        # assert self._strong_obj is None or self.modified
+
+        if self._strong_obj is None or not self.modified:
             instance_dict = self._instance_dict()
             if instance_dict:
                 instance_dict._modified.add(self)
 
             self._strong_obj = self.obj()
-
+            if self._strong_obj is None:
+                util.warn(
+                        "Can't emit change event for attribute '%s.%s' "
+                        "- parent object of type %s has been garbage "
+                        "collected." 
+                        % (
+                            self.class_.__name__, 
+                            attr.key, 
+                            orm_util.state_class_str(self)
+                        ))
             self.modified = True
 
     def commit(self, dict_, keys):
@@ -363,14 +385,14 @@ class InstanceState(object):
                 self.committed_state[key] = self.manager[key].impl.copy(dict_[key])
             else:
                 self.committed_state.pop(key, None)
-        
+
         self.expired = False
-        
+
         for key in set(self.callables).\
                             intersection(keys).\
                             intersection(dict_):
             del self.callables[key]
-    
+
     def commit_all(self, dict_, instance_dict=None):
         """commit all attributes unconditionally.
 
@@ -386,7 +408,7 @@ class InstanceState(object):
         if a value was not populated in state.dict.
 
         """
-        
+
         self.__dict__.pop('committed_state', None)
         self.__dict__.pop('pending', None)
 
@@ -399,26 +421,26 @@ class InstanceState(object):
         for key in self.manager.mutable_attributes:
             if key in dict_:
                 self.committed_state[key] = self.manager[key].impl.copy(dict_[key])
-                
+
         if instance_dict and self.modified:
             instance_dict._modified.discard(self)
-        
+
         self.modified = self.expired = False
         self._strong_obj = None
 
 class MutableAttrInstanceState(InstanceState):
     """InstanceState implementation for objects that reference 'mutable' 
     attributes.
-    
+
     Has a more involved "cleanup" handler that checks mutable attributes
     for changes upon dereference, resurrecting if needed.
-    
+
     """
-    
+
     @util.memoized_property
     def mutable_dict(self):
         return {}
-        
+
     def _get_modified(self, dict_=None):
         if self.__dict__.get('modified', False):
             return True
@@ -430,18 +452,18 @@ class MutableAttrInstanceState(InstanceState):
                     return True
             else:
                 return False
-    
+
     def _set_modified(self, value):
         self.__dict__['modified'] = value
-        
+
     modified = property(_get_modified, _set_modified)
-    
+
     @property
     def unmodified(self):
         """a set of keys which have no uncommitted changes"""
 
         dict_ = self.dict
-        
+
         return set([
             key for key in self.manager
             if (key not in self.committed_state or
@@ -450,12 +472,12 @@ class MutableAttrInstanceState(InstanceState):
 
     def _is_really_none(self):
         """do a check modified/resurrect.
-        
+
         This would be called in the extremely rare
         race condition that the weakref returned None but
         the cleanup handler had not yet established the 
         __resurrect callable as its replacement.
-        
+
         """
         if self.modified:
             self.obj = self.__resurrect
@@ -466,19 +488,19 @@ class MutableAttrInstanceState(InstanceState):
     def reset(self, dict_, key):
         self.mutable_dict.pop(key, None)
         InstanceState.reset(self, dict_, key)
-    
+
     def _cleanup(self, ref):
         """weakref callback.
-        
+
         This method may be called by an asynchronous
         gc.
-        
+
         If the state shows pending changes, the weakref
         is replaced by the __resurrect callable which will
         re-establish an object reference on next access,
         else removes this InstanceState from the owning
         identity map, if any.
-        
+
         """
         if self._get_modified(self.mutable_dict):
             self.obj = self.__resurrect
@@ -490,13 +512,13 @@ class MutableAttrInstanceState(InstanceState):
                 except AssertionError:
                     pass
             self.dispose()
-            
+
     def __resurrect(self):
         """A substitute for the obj() weakref function which resurrects."""
-        
+
         # store strong ref'ed version of the object; will revert
         # to weakref when changes are persisted
-        
+
         obj = self.manager.new_instance(state=self)
         self.obj = weakref.ref(obj, self._cleanup)
         self._strong_obj = obj
@@ -504,12 +526,12 @@ class MutableAttrInstanceState(InstanceState):
 
         # re-establishes identity attributes from the key
         self.manager.events.run('on_resurrect', self, obj)
-        
+
         # TODO: don't really think we should run this here.
         # resurrect is only meant to preserve the minimal state needed to
         # do an UPDATE, not to produce a fully usable object
         self._run_on_load(obj)
-        
+
         return obj
 
 class PendingCollection(object):
@@ -527,10 +549,12 @@ class PendingCollection(object):
     def append(self, value):
         if value in self.deleted_items:
             self.deleted_items.remove(value)
-        self.added_items.add(value)
+        else:
+            self.added_items.add(value)
 
     def remove(self, value):
         if value in self.added_items:
             self.added_items.remove(value)
-        self.deleted_items.add(value)
+        else:
+            self.deleted_items.add(value)
 
