@@ -1,8 +1,14 @@
+# sqlalchemy/queue.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
 """An adaptation of Py2.3/2.4's Queue module which supports reentrant
 behavior, using RLock instead of Lock for its mutex object.
 
 This is to support the connection pool's usage of weakref callbacks to return
-connections to the underlying Queue, which can apparently in extremely
+connections to the underlying Queue, which can in extremely
 rare cases be invoked within the ``get()`` method of the Queue itself,
 producing a ``put()`` inside the ``get()`` and therefore a reentrant
 condition."""
@@ -52,7 +58,8 @@ class Queue:
         return n
 
     def empty(self):
-        """Return True if the queue is empty, False otherwise (not reliable!)."""
+        """Return True if the queue is empty, False otherwise (not
+        reliable!)."""
 
         self.mutex.acquire()
         n = self._empty()
@@ -60,7 +67,8 @@ class Queue:
         return n
 
     def full(self):
-        """Return True if the queue is full, False otherwise (not reliable!)."""
+        """Return True if the queue is full, False otherwise (not
+        reliable!)."""
 
         self.mutex.acquire()
         n = self._full()
