@@ -5,6 +5,7 @@ from sqlalchemy.test import *
 class CompileTest(TestBase, AssertsExecutionResults):
     @classmethod
     def setup_class(cls):
+
         global t1, t2, metadata
         metadata = MetaData()
         t1 = Table('t1', metadata,
@@ -15,6 +16,10 @@ class CompileTest(TestBase, AssertsExecutionResults):
             Column('c1', Integer, primary_key=True),
             Column('c2', String(30)))
 
+        # do a "compile" ahead of time to load
+        # deferred imports
+        t1.insert().compile()
+
         # go through all the TypeEngine
         # objects in use and pre-load their _type_affinity
         # entries.
@@ -24,13 +29,13 @@ class CompileTest(TestBase, AssertsExecutionResults):
         from sqlalchemy import types
         for t in types.type_map.values():
             t._type_affinity
-            
-    @profiling.function_call_count(69, {'2.4': 44, 
+
+    @profiling.function_call_count(73, {'2.4': 44, 
                                             '3.0':77, '3.1':77})
     def test_insert(self):
         t1.insert().compile()
 
-    @profiling.function_call_count(69, {'2.4': 45})
+    @profiling.function_call_count(73, {'2.4': 45})
     def test_update(self):
         t1.update().compile()
 
