@@ -13,13 +13,53 @@ from sqlalchemy.test.testing import eq_
 from sqlalchemy.test import *
 from sqlalchemy.test.engines import utf8_engine
 import datetime
+from sqlalchemy.engine.url import make_url
+
+
+class CompileTest(TestBase, AssertsCompiledSQL):
+
+    __dialect__ = mysql.dialect()
+
+    def test_reserved_words(self):
+        table = Table("mysql_table", MetaData(),
+            Column("col1", Integer),
+            Column("master_ssl_verify_server_cert", Integer))
+        x = select([table.c.col1, table.c.master_ssl_verify_server_cert])
+
+        self.assert_compile(x, 
+            '''SELECT mysql_table.col1, mysql_table.`master_ssl_verify_server_cert` FROM mysql_table''')
+
+class DialectTest(TestBase):
+    __only_on__ = 'mysql'
+
+    @testing.only_on(['mysql+mysqldb', 'mysql+oursql'], 
+                    'requires particular SSL arguments')
+    def test_ssl_arguments(self):
+        dialect = testing.db.dialect
+        kwarg = dialect.create_connect_args(
+            make_url("mysql://scott:tiger@localhost:3306/test"
+                "?ssl_ca=/ca.pem&ssl_cert=/cert.pem&ssl_key=/key.pem")
+        )[1]
+        # args that differ among mysqldb and oursql
+        for k in ('use_unicode', 'found_rows', 'client_flag'):
+            kwarg.pop(k, None)
+        eq_(
+            kwarg, 
+            {
+                'passwd': 'tiger', 'db': 'test', 
+                'ssl': {'ca': '/ca.pem', 'cert': '/cert.pem', 
+                        'key': '/key.pem'}, 
+                'host': 'localhost', 'user': 'scott', 
+                'port': 3306
+            }
+        )
 
 class TypesTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
     "Test MySQL column types"
 
     __only_on__ = 'mysql'
     __dialect__ = mysql.dialect()
-    
+
     @testing.uses_deprecated('Manually quoting ENUM value literals')
     def test_basic(self):
         meta1 = MetaData(testing.db)
@@ -358,7 +398,7 @@ class TypesTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             meta.drop_all()
 
     def test_boolean(self):
-        """Test BOOL/TINYINT(1) compatability and reflection."""
+        """Test BOOL/TINYINT(1) compatibility and reflection."""
 
         meta = MetaData(testing.db)
         bool_table = Table(
@@ -487,22 +527,22 @@ class TypesTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             # if needed, can break out the eq_() just to check for
             # timestamps that are within a few seconds of "now" 
             # using timedelta.
-            
+
             now = testing.db.execute("select now()").scalar()
-            
+
             # TIMESTAMP without NULL inserts current time when passed
             # NULL.  when not passed, generates 0000-00-00 quite
             # annoyingly.
             ts_table.insert().execute({'t1':now, 't2':None})
             ts_table.insert().execute({'t1':None, 't2':None})
-            
+
             eq_(
                 ts_table.select().execute().fetchall(),
                 [(now, now), (None, now)]
             )
         finally:
             meta.drop_all()
-            
+
     def test_year(self):
         """Exercise YEAR."""
 
@@ -532,7 +572,6 @@ class TypesTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         finally:
             meta.drop_all()
 
-
     def test_set(self):
         """Exercise the SET type."""
 
@@ -625,7 +664,7 @@ class TypesTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
         assert_raises(exc.SQLError, enum_table.insert().execute, 
                         e1=None, e2=None, e3=None, e4=None)
-        
+
         assert_raises(exc.InvalidRequestError, enum_table.insert().execute,
                                         e1='c', e2='c', e2generic='c', e3='c',
                                         e4='c', e5='c', e5generic='c', e6='c')
@@ -665,7 +704,7 @@ class TypesTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
         eq_(res, expected)
         enum_table.drop()
-    
+
     def test_unicode_enum(self):
         unicode_engine = utf8_engine()
         metadata = MetaData(unicode_engine)
@@ -698,7 +737,7 @@ class TypesTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
                     (u'réveillé', u'drôle') #, u'S’il') # eh ? 
         finally:
             metadata.drop_all()
-        
+
     def test_enum_compile(self):
         e1 = Enum('x', 'y', 'z', name='somename')
         t1 = Table('sometable', MetaData(), Column('somecolumn', e1))
@@ -711,7 +750,7 @@ class TypesTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
                             "CREATE TABLE sometable (somecolumn "
                             "VARCHAR(1), CHECK (somecolumn IN ('x', "
                             "'y', 'z')))")
-        
+
     @testing.exclude('mysql', '<', (4,), "3.23 can't handle an ENUM of ''")
     @testing.uses_deprecated('Manually quoting ENUM value literals')
     def test_enum_parse(self):
@@ -763,6 +802,9 @@ class ReflectionTest(TestBase, AssertsExecutionResults):
             Column('c4', TIMESTAMP, DefaultClause('2009-04-05 12:00:00'
                    )),
             Column('c5', TIMESTAMP),
+            Column('c6', TIMESTAMP,
+                   DefaultClause(sql.text("CURRENT_TIMESTAMP "
+                                          "ON UPDATE CURRENT_TIMESTAMP"))),
             )
         def_table.create()
         try:
@@ -780,6 +822,13 @@ class ReflectionTest(TestBase, AssertsExecutionResults):
         assert str(reflected.c.c3.server_default.arg) == "'abc'"
         assert str(reflected.c.c4.server_default.arg) \
             == "'2009-04-05 12:00:00'"
+        assert reflected.c.c5.default is None
+        assert reflected.c.c5.server_default is None
+        assert reflected.c.c6.default is None
+        eq_(
+            str(reflected.c.c6.server_default.arg).upper(),
+            "CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP"
+        )
         reflected.create()
         try:
             reflected2 = Table('mysql_def', MetaData(testing.db),
@@ -791,6 +840,13 @@ class ReflectionTest(TestBase, AssertsExecutionResults):
         assert str(reflected2.c.c3.server_default.arg) == "'abc'"
         assert str(reflected2.c.c4.server_default.arg) \
             == "'2009-04-05 12:00:00'"
+        assert reflected.c.c5.default is None
+        assert reflected.c.c5.server_default is None
+        assert reflected.c.c6.default is None
+        eq_(
+            str(reflected.c.c6.server_default.arg).upper(),
+            "CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP"
+        )
 
     def test_reflection_with_table_options(self):
         comment = r"""Comment types type speedily ' " \ '' Fun!"""
@@ -1026,7 +1082,7 @@ class SQLTest(TestBase, AssertsCompiledSQL):
         eq_(
             gen(True, ['high_priority', sql.text('sql_cache')]),
             'SELECT high_priority sql_cache DISTINCT q')
-        
+
     def test_backslash_escaping(self):
         self.assert_compile(
             sql.column('foo').like('bar', escape='\\'),
@@ -1040,7 +1096,7 @@ class SQLTest(TestBase, AssertsCompiledSQL):
             "foo LIKE %s ESCAPE '\\'",
             dialect=dialect
         )
-        
+
     def test_limit(self):
         t = sql.table('t', sql.column('col1'), sql.column('col2'))
 
@@ -1055,7 +1111,7 @@ class SQLTest(TestBase, AssertsCompiledSQL):
             select([t]).offset(10),
             "SELECT t.col1, t.col2 FROM t  LIMIT 10, 18446744073709551615"
             )
-    
+
     def test_varchar_raise(self):
         for type_ in (
             String,
@@ -1068,7 +1124,7 @@ class SQLTest(TestBase, AssertsCompiledSQL):
         ):
             type_ = sqltypes.to_instance(type_)
             assert_raises(exc.InvalidRequestError, type_.compile, dialect=mysql.dialect())
-            
+
     def test_update_limit(self):
         t = sql.table('t', sql.column('col1'), sql.column('col2'))
 
@@ -1094,7 +1150,7 @@ class SQLTest(TestBase, AssertsCompiledSQL):
 
     def test_sysdate(self):
         self.assert_compile(func.sysdate(), "SYSDATE()")
-        
+
     def test_cast(self):
         t = sql.table('t', sql.column('col'))
         m = mysql
@@ -1186,7 +1242,7 @@ class SQLTest(TestBase, AssertsCompiledSQL):
 
         for type_, expected in specs:
             self.assert_compile(cast(t.c.col, type_), expected)
-    
+
     def test_no_cast_pre_4(self):
         self.assert_compile(
                     cast(Column('foo', Integer), String),
@@ -1199,7 +1255,7 @@ class SQLTest(TestBase, AssertsCompiledSQL):
                     "foo",
                     dialect=dialect
             )
-        
+
     def test_extract(self):
         t = sql.table('t', sql.column('col1'))
 
@@ -1212,24 +1268,24 @@ class SQLTest(TestBase, AssertsCompiledSQL):
         self.assert_compile(
             select([extract('milliseconds', t.c.col1)]),
             "SELECT EXTRACT(millisecond FROM t.col1) AS anon_1 FROM t")
-    
+
     def test_too_long_index(self):
         exp = 'ix_zyrenian_zyme_zyzzogeton_zyzzogeton_zyrenian_zyme_zyz_5cd2'
         tname = 'zyrenian_zyme_zyzzogeton_zyzzogeton'
         cname = 'zyrenian_zyme_zyzzogeton_zo'
-        
+
         t1 = Table(tname, MetaData(), 
                     Column(cname, Integer, index=True),
                 )
         ix1 = list(t1.indexes)[0]
-        
+
         self.assert_compile(
             schema.CreateIndex(ix1),
             "CREATE INDEX %s "
             "ON %s (%s)" % (exp, tname, cname),
             dialect=mysql.dialect()
         )
-        
+
     def test_innodb_autoincrement(self):
         t1 = Table('sometable', MetaData(), Column('assigned_id',
                    Integer(), primary_key=True, autoincrement=False),
@@ -1254,7 +1310,7 @@ class SQLTest(TestBase, AssertsCompiledSQL):
 
 class SQLModeDetectionTest(TestBase):
     __only_on__ = 'mysql'
-    
+
     def _options(self, modes):
         class SetOptions(object):
             def first_connect(self, con, record):
@@ -1263,7 +1319,7 @@ class SQLModeDetectionTest(TestBase):
                 cursor = con.cursor()
                 cursor.execute("set sql_mode='%s'" % (",".join(modes)))
         return engines.testing_engine(options={"listeners":[SetOptions()]})
-        
+
     def test_backslash_escapes(self):
         engine = self._options(['NO_BACKSLASH_ESCAPES'])
         c = engine.connect()
@@ -1291,7 +1347,7 @@ class SQLModeDetectionTest(TestBase):
         assert not engine.dialect._backslash_escapes
         c.close()
         engine.dispose()
-        
+
 class RawReflectionTest(TestBase):
     def setup(self):
         dialect = mysql.dialect()
@@ -1323,7 +1379,7 @@ class ExecutionTest(TestBase):
         meta.reflect(cx)
         eq_(cx.dialect._connection_charset, charset)
         cx.close()
-    
+
     def test_sysdate(self):
         d = testing.db.scalar(func.sysdate())
         assert isinstance(d, datetime.datetime)
@@ -1379,7 +1435,7 @@ class MatchTest(TestBase, AssertsCompiledSQL):
         self.assert_compile(
             matchtable.c.title.match('somstr'),
             "MATCH (matchtable.title) AGAINST (%s IN BOOLEAN MODE)" % format)
-    
+
     @testing.fails_on('mysql+mysqldb', 'uses format')
     @testing.fails_on('mysql+oursql', 'uses format')
     @testing.fails_on('mysql+pyodbc', 'uses format')
