@@ -17,7 +17,7 @@ import logging
 
 class SequenceTest(TestBase, AssertsCompiledSQL):
 
-    def test_basic(self):
+    def test_format(self):
         seq = Sequence('my_seq_no_schema')
         dialect = postgresql.PGDialect()
         assert dialect.identifier_preparer.format_sequence(seq) \
@@ -29,6 +29,24 @@ class SequenceTest(TestBase, AssertsCompiledSQL):
         assert dialect.identifier_preparer.format_sequence(seq) \
             == '"Some_Schema"."My_Seq"'
 
+    @testing.only_on('postgresql', 'foo')
+    @testing.provide_metadata
+    def test_reverse_eng_name(self):
+        engine = engines.testing_engine(options=dict(implicit_returning=False))
+        for tname, cname in [
+            ('tb1' * 30, 'abc'),
+            ('tb2', 'abc' * 30),
+            ('tb3' * 30, 'abc' * 30),
+            ('tb4', 'abc'),
+        ]:
+            t = Table(tname[:57],
+                metadata,
+                Column(cname[:57], Integer, primary_key=True)
+            )
+            t.create(engine)
+            r = engine.execute(t.insert())
+            assert r.inserted_primary_key == [1]
+
 class CompileTest(TestBase, AssertsCompiledSQL):
 
     __dialect__ = postgresql.dialect()
@@ -56,7 +74,7 @@ class CompileTest(TestBase, AssertsCompiledSQL):
                             'RETURNING length(mytable.name) AS length_1'
                             , dialect=dialect)
 
-        
+
     def test_insert_returning(self):
         dialect = postgresql.dialect()
         table1 = table('mytable',
@@ -83,7 +101,7 @@ class CompileTest(TestBase, AssertsCompiledSQL):
                             'INSERT INTO mytable (name) VALUES '
                             '(%(name)s) RETURNING length(mytable.name) '
                             'AS length_1', dialect=dialect)
-    
+
     @testing.uses_deprecated('.*argument is deprecated.  Please use '
                              'statement.returning.*')
     def test_old_returning_names(self):
@@ -109,7 +127,7 @@ class CompileTest(TestBase, AssertsCompiledSQL):
                             'INSERT INTO mytable (name) VALUES '
                             '(%(name)s) RETURNING mytable.myid, '
                             'mytable.name', dialect=dialect)
-        
+
     def test_create_partial_index(self):
         m = MetaData()
         tbl = Table('testtbl', m, Column('data', Integer))
@@ -202,6 +220,16 @@ class CompileTest(TestBase, AssertsCompiledSQL):
                                     'anon_1 FROM t' % (field,
                                     compiled_expr))
 
+    def test_reserved_words(self):
+        table = Table("pg_table", MetaData(),
+            Column("col1", Integer),
+            Column("variadic", Integer))
+        x = select([table.c.col1, table.c.variadic])
+
+        self.assert_compile(x, 
+            '''SELECT pg_table.col1, pg_table."variadic" FROM pg_table''')
+
+
 class FloatCoercionTest(TablesTest, AssertsExecutionResults):
     __only_on__ = 'postgresql'
     __dialect__ = postgresql.dialect()
@@ -228,7 +256,7 @@ class FloatCoercionTest(TablesTest, AssertsExecutionResults):
             {'data':52},
             {'data':9},
         )
-    
+
     @testing.resolve_artifact_names
     def test_float_coercion(self):
         for type_, result in [
@@ -251,7 +279,7 @@ class FloatCoercionTest(TablesTest, AssertsExecutionResults):
                 ])
             ).scalar()
             eq_(round_decimal(ret, 9), result)
-    
+
     @testing.provide_metadata
     def test_arrays(self):
         t1 = Table('t', metadata, 
@@ -261,13 +289,13 @@ class FloatCoercionTest(TablesTest, AssertsExecutionResults):
             Column('q', postgresql.ARRAY(Numeric))
         )
         metadata.create_all()
-        t1.insert().execute(x=[5], y=[5], z=[6], q=[6.4])
+        t1.insert().execute(x=[5], y=[5], z=[6], q=[decimal.Decimal("6.4")])
         row = t1.select().execute().first()
         eq_(
             row, 
             ([5], [5], [6], [decimal.Decimal("6.4")])
         )
-        
+
 class EnumTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
     __only_on__ = 'postgresql'
@@ -296,7 +324,7 @@ class EnumTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
                             "CREATE TABLE sometable (somecolumn "
                             "VARCHAR(1), CHECK (somecolumn IN ('x', "
                             "'y', 'z')))")
-    
+
     @testing.fails_on('postgresql+zxjdbc',
                       'zxjdbc fails on ENUM: column "XXX" is of type '
                       'XXX but expression is of type character varying')
@@ -319,7 +347,7 @@ class EnumTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         finally:
             metadata.drop_all()
             metadata.drop_all()
-    
+
     def test_name_required(self):
         metadata = MetaData(testing.db)
         etype = Enum('four', 'five', 'six', metadata=metadata)
@@ -341,7 +369,7 @@ class EnumTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
                     Enum(u'réveillé', u'drôle', u'S’il',
                             name='onetwothreetype'))
         )
-        
+
         metadata.create_all()
         try:
             t1.insert().execute(value=u'drôle')
@@ -412,6 +440,24 @@ class EnumTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             assert not testing.db.dialect.has_type(testing.db,
                     'fourfivesixtype')
 
+    def test_no_support(self):
+        def server_version_info(self):
+            return (8, 2)
+
+        e = engines.testing_engine()
+        dialect = e.dialect
+        dialect._get_server_version_info = server_version_info
+
+        assert dialect.supports_native_enum
+        e.connect()
+        assert not dialect.supports_native_enum
+
+        # initialize is called again on new pool
+        e.dispose()
+        e.connect()
+        assert not dialect.supports_native_enum
+
+
     def test_reflection(self):
         metadata = MetaData(testing.db)
         etype = Enum('four', 'five', 'six', name='fourfivesixtype',
@@ -456,7 +502,25 @@ class EnumTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             assert t2.c.value2.type.schema == 'test_schema'
         finally:
             metadata.drop_all()
-        
+
+class NumericInterpretationTest(TestBase):
+
+
+    def test_numeric_codes(self):
+        from sqlalchemy.dialects.postgresql import pg8000, psycopg2, base
+        from decimal import Decimal
+
+        for dialect in (pg8000.dialect(), psycopg2.dialect()):
+
+            typ = Numeric().dialect_impl(dialect)
+            for code in base._INT_TYPES + base._FLOAT_TYPES + \
+                        base._DECIMAL_TYPES:
+                proc = typ.result_processor(dialect, code)
+                val = 23.7
+                if proc is not None:
+                    val = proc(val)
+                assert val in (23.7, Decimal("23.7"))
+
 class InsertTest(TestBase, AssertsExecutionResults):
 
     __only_on__ = 'postgresql'
@@ -504,7 +568,7 @@ class InsertTest(TestBase, AssertsExecutionResults):
             assert_raises_message(exc.DBAPIError,
                                   'violates not-null constraint',
                                   eng.execute, t2.insert())
-        
+
     def test_sequence_insert(self):
         table = Table('testtable', metadata, Column('id', Integer,
                       Sequence('my_seq'), primary_key=True),
@@ -909,7 +973,10 @@ class DomainReflectionTest(TestBase, AssertsExecutionResults):
         con = testing.db.connect()
         for ddl in \
             'CREATE DOMAIN testdomain INTEGER NOT NULL DEFAULT 42', \
-            'CREATE DOMAIN test_schema.testdomain INTEGER DEFAULT 0':
+            'CREATE DOMAIN test_schema.testdomain INTEGER DEFAULT 0', \
+            "CREATE TYPE testtype AS ENUM ('test')", \
+            'CREATE DOMAIN enumdomain AS testtype'\
+            :
             try:
                 con.execute(ddl)
             except exc.SQLError, e:
@@ -923,6 +990,8 @@ class DomainReflectionTest(TestBase, AssertsExecutionResults):
         con.execute('CREATE TABLE crosschema (question integer, answer '
                     'test_schema.testdomain)')
 
+        con.execute('CREATE TABLE enum_test (id integer, data enumdomain)')
+
     @classmethod
     def teardown_class(cls):
         con = testing.db.connect()
@@ -931,6 +1000,9 @@ class DomainReflectionTest(TestBase, AssertsExecutionResults):
         con.execute('DROP TABLE crosschema')
         con.execute('DROP DOMAIN testdomain')
         con.execute('DROP DOMAIN test_schema.testdomain')
+        con.execute("DROP TABLE enum_test")
+        con.execute("DROP DOMAIN enumdomain")
+        con.execute("DROP TYPE testtype")
 
     def test_table_is_reflected(self):
         metadata = MetaData(testing.db)
@@ -947,6 +1019,14 @@ class DomainReflectionTest(TestBase, AssertsExecutionResults):
         assert not table.columns.answer.nullable, \
             'Expected reflected column to not be nullable.'
 
+    def test_enum_domain_is_reflected(self):
+        metadata = MetaData(testing.db)
+        table = Table('enum_test', metadata, autoload=True)
+        eq_(
+            table.c.data.type.enums,
+            ('test', )
+        )
+
     def test_table_is_reflected_test_schema(self):
         metadata = MetaData(testing.db)
         table = Table('testtable', metadata, autoload=True,
@@ -990,6 +1070,7 @@ class DomainReflectionTest(TestBase, AssertsExecutionResults):
         finally:
             postgresql.PGDialect.ischema_names = ischema_names
 
+
 class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
 
     __only_on__ = 'postgresql'
@@ -1133,6 +1214,33 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             'SELECT DISTINCT ON (mytable.id, mytable.a) mytable.id, '
             'mytable.a \nFROM mytable')
 
+    @testing.fails_if(lambda: True, "[ticket:2142]")
+    def test_distinct_on_subquery(self):
+        t1 = Table('mytable1', MetaData(testing.db), Column('id',
+                  Integer, primary_key=True), Column('a', String(8)))
+        t2 = Table('mytable2', MetaData(testing.db), Column('id',
+                  Integer, primary_key=True), Column('a', String(8)))
+
+        sq = select([t1]).alias()
+        q = select([t2.c.id,sq.c.id], distinct=sq.c.id).where(t2.c.id==sq.c.id)
+        self.assert_compile(
+            q,
+            "SELECT DISTINCT ON (anon_1.id) mytable2.id, anon_1.id "
+            "FROM mytable2, (SELECT mytable1.id AS id, mytable1.a AS a "
+            "FROM mytable1) AS anon_1 "
+            "WHERE mytable2.id = anon_1.id"
+            )
+
+        sq = select([t1]).alias('sq')
+        q = select([t2.c.id,sq.c.id], distinct=sq.c.id).where(t2.c.id==sq.c.id)
+        self.assert_compile(
+            q,
+            "SELECT DISTINCT ON (sq.id) mytable2.id, sq.id "
+            "FROM mytable2, (SELECT mytable1.id AS id, mytable1.a AS a "
+            "FROM mytable1) AS sq "
+            "WHERE mytable2.id = sq.id"
+            )
+
     def test_schema_reflection(self):
         """note: this test requires that the 'test_schema' schema be
         separate and accessible by the test user"""
@@ -1313,6 +1421,27 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
             warnings.warn = capture_warnings._orig_showwarning
             m1.drop_all()
 
+    @testing.provide_metadata
+    def test_index_reflection_modified(self):
+        """reflect indexes when a column name has changed - PG 9 
+        does not update the name of the column in the index def.
+        [ticket:2141]
+
+        """
+
+        t1 = Table('t', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('x', Integer)
+        )
+        metadata.create_all()
+        conn = testing.db.connect().execution_options(autocommit=True)
+        conn.execute("CREATE INDEX idx1 ON t (x)")
+        conn.execute("ALTER TABLE t RENAME COLUMN x to y")
+
+        ind = testing.db.dialect.get_indexes(conn, "t", None)
+        eq_(ind, [{'unique': False, 'column_names': [u'y'], 'name': u'idx1'}])
+        conn.close()
+
     @testing.fails_on('postgresql+pypostgresql',
                       'pypostgresql bombs on multiple calls')
     def test_set_isolation_level(self):
@@ -1325,7 +1454,7 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
                             isolation_level='SERIALIZABLE')
         eq_(eng.execute('show transaction isolation level').scalar(),
             'serializable')
-        
+
         # check that it stays
         conn = eng.connect()
         eq_(conn.execute('show transaction isolation level').scalar(),
@@ -1336,7 +1465,7 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         eq_(conn.execute('show transaction isolation level').scalar(),
             'serializable')
         conn.close()
-        
+
         eng = create_engine(testing.db.url, isolation_level='FOO')
         if testing.db.driver == 'zxjdbc':
             exception_cls = eng.dialect.dbapi.Error
@@ -1354,11 +1483,11 @@ class MiscTest(TestBase, AssertsExecutionResults, AssertsCompiledSQL):
         stmt = text("select cast('hi' as char) as hi", typemap={'hi'
                     : Numeric})
         assert_raises(exc.InvalidRequestError, testing.db.execute, stmt)
-        
+
 class TimezoneTest(TestBase):
 
     """Test timezone-aware datetimes.
-    
+
     psycopg will return a datetime with a tzinfo attached to it, if
     postgresql returns it.  python then will not let you compare a
     datetime with a tzinfo to a datetime that doesnt have one.  this
@@ -1450,11 +1579,11 @@ class TimePrecisionTest(TestBase, AssertsCompiledSQL):
             self.assert_compile(type_, expected)
 
     @testing.only_on('postgresql', 'DB specific feature')
+    @testing.provide_metadata
     def test_reflection(self):
-        m1 = MetaData(testing.db)
         t1 = Table(
             't1',
-            m1,
+            metadata,
             Column('c1', postgresql.TIME()),
             Column('c2', postgresql.TIME(precision=5)),
             Column('c3', postgresql.TIME(timezone=True, precision=5)),
@@ -1464,24 +1593,21 @@ class TimePrecisionTest(TestBase, AssertsCompiledSQL):
                    precision=5)),
             )
         t1.create()
-        try:
-            m2 = MetaData(testing.db)
-            t2 = Table('t1', m2, autoload=True)
-            eq_(t2.c.c1.type.precision, None)
-            eq_(t2.c.c2.type.precision, 5)
-            eq_(t2.c.c3.type.precision, 5)
-            eq_(t2.c.c4.type.precision, None)
-            eq_(t2.c.c5.type.precision, 5)
-            eq_(t2.c.c6.type.precision, 5)
-            eq_(t2.c.c1.type.timezone, False)
-            eq_(t2.c.c2.type.timezone, False)
-            eq_(t2.c.c3.type.timezone, True)
-            eq_(t2.c.c4.type.timezone, False)
-            eq_(t2.c.c5.type.timezone, False)
-            eq_(t2.c.c6.type.timezone, True)
-        finally:
-            t1.drop()
-        
+        m2 = MetaData(testing.db)
+        t2 = Table('t1', m2, autoload=True)
+        eq_(t2.c.c1.type.precision, None)
+        eq_(t2.c.c2.type.precision, 5)
+        eq_(t2.c.c3.type.precision, 5)
+        eq_(t2.c.c4.type.precision, None)
+        eq_(t2.c.c5.type.precision, 5)
+        eq_(t2.c.c6.type.precision, 5)
+        eq_(t2.c.c1.type.timezone, False)
+        eq_(t2.c.c2.type.timezone, False)
+        eq_(t2.c.c3.type.timezone, True)
+        eq_(t2.c.c4.type.timezone, False)
+        eq_(t2.c.c5.type.timezone, False)
+        eq_(t2.c.c6.type.timezone, True)
+
 class ArrayTest(TestBase, AssertsExecutionResults):
 
     __only_on__ = 'postgresql'
@@ -1492,8 +1618,8 @@ class ArrayTest(TestBase, AssertsExecutionResults):
         metadata = MetaData(testing.db)
         arrtable = Table('arrtable', metadata, Column('id', Integer,
                          primary_key=True), Column('intarr',
-                         postgresql.PGArray(Integer)), Column('strarr',
-                         postgresql.PGArray(Unicode()), nullable=False))
+                         postgresql.ARRAY(Integer)), Column('strarr',
+                         postgresql.ARRAY(Unicode()), nullable=False))
         metadata.create_all()
 
     def teardown(self):
@@ -1506,8 +1632,8 @@ class ArrayTest(TestBase, AssertsExecutionResults):
     def test_reflect_array_column(self):
         metadata2 = MetaData(testing.db)
         tbl = Table('arrtable', metadata2, autoload=True)
-        assert isinstance(tbl.c.intarr.type, postgresql.PGArray)
-        assert isinstance(tbl.c.strarr.type, postgresql.PGArray)
+        assert isinstance(tbl.c.intarr.type, postgresql.ARRAY)
+        assert isinstance(tbl.c.strarr.type, postgresql.ARRAY)
         assert isinstance(tbl.c.intarr.type.item_type, Integer)
         assert isinstance(tbl.c.strarr.type.item_type, String)
 
@@ -1575,7 +1701,7 @@ class ArrayTest(TestBase, AssertsExecutionResults):
 
         footable = Table('foo', metadata, Column('id', Integer,
                          primary_key=True), Column('intarr',
-                         postgresql.PGArray(Integer), nullable=True))
+                         postgresql.ARRAY(Integer), nullable=True))
         mapper(Foo, footable)
         metadata.create_all()
         sess = create_session()
@@ -1608,13 +1734,45 @@ class ArrayTest(TestBase, AssertsExecutionResults):
         sess.add(foo)
         sess.flush()
 
+    @testing.provide_metadata
+    def test_tuple_flag(self):
+        assert_raises_message(
+            exc.ArgumentError, 
+            "mutable must be set to False if as_tuple is True.",
+            postgresql.ARRAY, Integer, as_tuple=True)
+
+        t1 = Table('t1', metadata,
+            Column('id', Integer, primary_key=True),
+            Column('data', postgresql.ARRAY(String(5), as_tuple=True, mutable=False)),
+            Column('data2', postgresql.ARRAY(Numeric(asdecimal=False), as_tuple=True, mutable=False)),
+        )
+        metadata.create_all()
+        testing.db.execute(t1.insert(), id=1, data=["1","2","3"], data2=[5.4, 5.6])
+        testing.db.execute(t1.insert(), id=2, data=["4", "5", "6"], data2=[1.0])
+        testing.db.execute(t1.insert(), id=3, data=[["4", "5"], ["6", "7"]], data2=[[5.4, 5.6], [1.0, 1.1]])
+
+        r = testing.db.execute(t1.select().order_by(t1.c.id)).fetchall()
+        eq_(
+            r, 
+            [
+                (1, ('1', '2', '3'), (5.4, 5.6)), 
+                (2, ('4', '5', '6'), (1.0,)), 
+                (3, (('4', '5'), ('6', '7')), ((5.4, 5.6), (1.0, 1.1)))
+            ]
+        )
+        # hashable
+        eq_(
+            set(row[1] for row in r),
+            set([('1', '2', '3'), ('4', '5', '6'), (('4', '5'), ('6', '7'))])
+        )
+
 class TimestampTest(TestBase, AssertsExecutionResults):
     __only_on__ = 'postgresql'
 
     def test_timestamp(self):
         engine = testing.db
         connection = engine.connect()
-        
+
         s = select(["timestamp '2007-12-25'"])
         result = connection.execute(s).first()
         eq_(result[0], datetime.datetime(2007, 12, 25, 0, 0))
@@ -1762,17 +1920,17 @@ class ServerSideCursorsTest(TestBase, AssertsExecutionResults):
         finally:
             test_table.drop(checkfirst=True)
 
-class SpecialTypesTest(TestBase, ComparesTables):
+class SpecialTypesTest(TestBase, ComparesTables, AssertsCompiledSQL):
     """test DDL and reflection of PG-specific types """
-    
+
     __only_on__ = 'postgresql'
     __excluded_on__ = (('postgresql', '<', (8, 3, 0)),)
-    
+
     @classmethod
     def setup_class(cls):
         global metadata, table
         metadata = MetaData(testing.db)
-        
+
         # create these types so that we can issue
         # special SQL92 INTERVAL syntax
         class y2m(types.UserDefinedType, postgresql.INTERVAL):
@@ -1782,38 +1940,129 @@ class SpecialTypesTest(TestBase, ComparesTables):
         class d2s(types.UserDefinedType, postgresql.INTERVAL):
             def get_col_spec(self):
                 return "INTERVAL DAY TO SECOND"
-            
+
         table = Table('sometable', metadata,
-            Column('id', postgresql.PGUuid, primary_key=True),
-            Column('flag', postgresql.PGBit),
-            Column('addr', postgresql.PGInet),
-            Column('addr2', postgresql.PGMacAddr),
-            Column('addr3', postgresql.PGCidr),
+            Column('id', postgresql.UUID, primary_key=True),
+            Column('flag', postgresql.BIT),
+            Column('bitstring', postgresql.BIT(4)),
+            Column('addr', postgresql.INET),
+            Column('addr2', postgresql.MACADDR),
+            Column('addr3', postgresql.CIDR),
             Column('doubleprec', postgresql.DOUBLE_PRECISION),
             Column('plain_interval', postgresql.INTERVAL),
             Column('year_interval', y2m()),
             Column('month_interval', d2s()),
             Column('precision_interval', postgresql.INTERVAL(precision=3))
         )
-        
+
         metadata.create_all()
-        
+
         # cheat so that the "strict type check"
         # works
         table.c.year_interval.type = postgresql.INTERVAL()
         table.c.month_interval.type = postgresql.INTERVAL()
-    
+
     @classmethod
     def teardown_class(cls):
         metadata.drop_all()
-    
+
     def test_reflection(self):
         m = MetaData(testing.db)
         t = Table('sometable', m, autoload=True)
-        
+
         self.assert_tables_equal(table, t, strict_types=True)
         assert t.c.plain_interval.type.precision is None
         assert t.c.precision_interval.type.precision == 3
+        assert t.c.bitstring.type.length == 4
+
+    def test_bit_compile(self):
+        pairs = [(postgresql.BIT(), 'BIT(1)'),
+                 (postgresql.BIT(5), 'BIT(5)'),
+                 (postgresql.BIT(varying=True), 'BIT VARYING'),
+                 (postgresql.BIT(5, varying=True), 'BIT VARYING(5)'),
+                ]
+        for type_, expected in pairs:
+            self.assert_compile(type_, expected)
+
+    @testing.provide_metadata
+    def test_bit_reflection(self):
+        t1 = Table('t1', metadata,
+        Column('bit1', postgresql.BIT()),
+        Column('bit5', postgresql.BIT(5)),
+        Column('bitvarying', postgresql.BIT(varying=True)),
+        Column('bitvarying5', postgresql.BIT(5, varying=True)),
+        )
+        t1.create()
+        m2 = MetaData(testing.db)
+        t2 = Table('t1', m2, autoload=True)
+        eq_(t2.c.bit1.type.length, 1)
+        eq_(t2.c.bit1.type.varying, False)
+        eq_(t2.c.bit5.type.length, 5)
+        eq_(t2.c.bit5.type.varying, False)
+        eq_(t2.c.bitvarying.type.length, None)
+        eq_(t2.c.bitvarying.type.varying, True)
+        eq_(t2.c.bitvarying5.type.length, 5)
+        eq_(t2.c.bitvarying5.type.varying, True)
+
+class UUIDTest(TestBase):
+    """Test the bind/return values of the UUID type."""
+
+    __only_on__ = 'postgresql'
+
+    @testing.requires.python25
+    @testing.fails_on('postgresql+pg8000', 'No support for UUID type')
+    def test_uuid_string(self):
+        import uuid
+        self._test_round_trip(
+            Table('utable', MetaData(), 
+                Column('data', postgresql.UUID())
+            ),
+            str(uuid.uuid4()),
+            str(uuid.uuid4())
+        )
+
+    @testing.requires.python25
+    @testing.fails_on('postgresql+pg8000', 'No support for UUID type')
+    def test_uuid_uuid(self):
+        import uuid
+        self._test_round_trip(
+            Table('utable', MetaData(), 
+                Column('data', postgresql.UUID(as_uuid=True))
+            ),
+            uuid.uuid4(),
+            uuid.uuid4()
+        )
+
+    def test_no_uuid_available(self):
+        from sqlalchemy.dialects.postgresql import base
+        uuid_type = base._python_UUID
+        base._python_UUID = None
+        try:
+            assert_raises(
+                NotImplementedError,
+                postgresql.UUID, as_uuid=True
+            )
+        finally:
+            base._python_UUID = uuid_type
+
+    def setup(self):
+        self.conn = testing.db.connect()
+        trans = self.conn.begin()
+
+    def teardown(self):
+        self.conn.close()
+
+    def _test_round_trip(self, utable, value1, value2):
+        utable.create(self.conn)
+        self.conn.execute(utable.insert(), {'data':value1})
+        self.conn.execute(utable.insert(), {'data':value2})
+        r = self.conn.execute(
+                select([utable.c.data]).
+                    where(utable.c.data != value1)
+                )
+        eq_(r.fetchone()[0], value2)
+        eq_(r.fetchone(), None)
+
 
 class MatchTest(TestBase, AssertsCompiledSQL):
 
@@ -1880,12 +2129,14 @@ class MatchTest(TestBase, AssertsCompiledSQL):
                 )).execute().fetchall()
         eq_([3], [r.id for r in results])
 
+    @testing.requires.english_locale_on_postgresql
     def test_simple_derivative_match(self):
         results = \
             matchtable.select().where(matchtable.c.title.match('nutshells'
                 )).execute().fetchall()
         eq_([5], [r.id for r in results])
 
+    @testing.requires.english_locale_on_postgresql
     def test_or_match(self):
         results1 = \
             matchtable.select().where(or_(matchtable.c.title.match('nutshells'
@@ -1898,6 +2149,7 @@ class MatchTest(TestBase, AssertsCompiledSQL):
                 )).order_by(matchtable.c.id).execute().fetchall()
         eq_([3, 5], [r.id for r in results2])
 
+    @testing.requires.english_locale_on_postgresql
     def test_and_match(self):
         results1 = \
             matchtable.select().where(and_(matchtable.c.title.match('python'
@@ -1910,6 +2162,7 @@ class MatchTest(TestBase, AssertsCompiledSQL):
                 )).execute().fetchall()
         eq_([5], [r.id for r in results2])
 
+    @testing.requires.english_locale_on_postgresql
     def test_match_across_joins(self):
         results = matchtable.select().where(and_(cattable.c.id
                 == matchtable.c.category_id,
@@ -1917,3 +2170,33 @@ class MatchTest(TestBase, AssertsCompiledSQL):
                 matchtable.c.title.match('nutshells'
                 )))).order_by(matchtable.c.id).execute().fetchall()
         eq_([1, 3, 5], [r.id for r in results])
+
+
+class TupleTest(TestBase):
+    __only_on__ = 'postgresql'
+
+    def test_tuple_containment(self):
+
+        for test, exp in [
+            ([('a', 'b')], True),
+            ([('a', 'c')], False),
+            ([('f', 'q'), ('a', 'b')], True),
+            ([('f', 'q'), ('a', 'c')], False)
+        ]:
+            eq_(
+                testing.db.execute(
+                    select([
+                            tuple_(
+                                literal_column("'a'"), 
+                                literal_column("'b'")
+                            ).\
+                                in_([
+                                    tuple_(*[
+                                            literal_column("'%s'" % letter) 
+                                            for letter in elem
+                                        ]) for elem in test
+                                ])
+                            ])
+                ).scalar(),
+                exp
+            )
