@@ -16,7 +16,7 @@ class TestTypes(TestBase, AssertsExecutionResults):
 
     def test_boolean(self):
         """Test that the boolean only treats 1 as True
-        
+
         """
 
         meta = MetaData(testing.db)
@@ -214,7 +214,7 @@ class DialectTest(TestBase, AssertsExecutionResults):
 
     def test_extra_reserved_words(self):
         """Tests reserved words in identifiers.
-        
+
         'true', 'false', and 'column' are undocumented reserved words
         when used as column identifiers (as of 3.5.1).  Covering them
         here to ensure they remain in place if the dialect's
@@ -384,6 +384,57 @@ class SQLTest(TestBase, AssertsCompiledSQL):
                                 "SELECT CAST(STRFTIME('%s', t.col1) AS "
                                 "INTEGER) AS anon_1 FROM t" % subst)
 
+    def test_constraints_with_schemas(self):
+        metadata = MetaData()
+        t1 = Table('t1', metadata, 
+                        Column('id', Integer, primary_key=True),
+                        schema='master')
+        t2 = Table('t2', metadata, 
+                        Column('id', Integer, primary_key=True),
+                        Column('t1_id', Integer, ForeignKey('master.t1.id')),
+                        schema='master'
+                    )
+        t3 = Table('t3', metadata, 
+                        Column('id', Integer, primary_key=True),
+                        Column('t1_id', Integer, ForeignKey('master.t1.id')),
+                        schema='alternate'
+                    )
+        t4 = Table('t4', metadata, 
+                        Column('id', Integer, primary_key=True),
+                        Column('t1_id', Integer, ForeignKey('master.t1.id')),
+                    )
+
+        # schema->schema, generate REFERENCES with no schema name
+        self.assert_compile(
+            schema.CreateTable(t2),
+                "CREATE TABLE master.t2 ("
+                "id INTEGER NOT NULL, "
+                "t1_id INTEGER, "
+                "PRIMARY KEY (id), "
+                "FOREIGN KEY(t1_id) REFERENCES t1 (id)"
+                ")"
+        )
+
+        # schema->different schema, don't generate REFERENCES
+        self.assert_compile(
+            schema.CreateTable(t3),
+                "CREATE TABLE alternate.t3 ("
+                "id INTEGER NOT NULL, "
+                "t1_id INTEGER, "
+                "PRIMARY KEY (id)"
+                ")"
+        )
+
+        # same for local schema
+        self.assert_compile(
+            schema.CreateTable(t4),
+                "CREATE TABLE t4 ("
+                "id INTEGER NOT NULL, "
+                "t1_id INTEGER, "
+                "PRIMARY KEY (id)"
+                ")"
+        )
+
 
 class InsertTest(TestBase, AssertsExecutionResults):
 
@@ -585,3 +636,24 @@ class TestAutoIncrement(TestBase, AssertsCompiledSQL):
                             'CREATE TABLE noautoinctable (id INTEGER '
                             'NOT NULL, x INTEGER, PRIMARY KEY (id))',
                             dialect=sqlite.dialect())
+
+
+class ReflectHeadlessFKsTest(TestBase):
+    __only_on__ = ('sqlite', )
+
+    def setup(self):
+        testing.db.execute("CREATE TABLE a (id INTEGER PRIMARY KEY)")
+        testing.db.execute("CREATE TABLE b (id INTEGER PRIMARY KEY REFERENCES a)")
+
+    def teardown(self):
+        testing.db.execute("drop table b")
+        testing.db.execute("drop table a")
+
+    def test_reflect_tables_fk_no_colref(self):
+        meta = MetaData()
+        a = Table('a', meta, autoload=True, autoload_with=testing.db)
+        b = Table('b', meta, autoload=True, autoload_with=testing.db)
+
+        assert b.c.id.references(a.c.id)
+
+
