@@ -36,10 +36,10 @@ class MockCursor(object):
 mock_dbapi = MockDBAPI()
 
 
-class PoolTestBase(TestBase):    
+class PoolTestBase(TestBase):
     def setup(self):
         pool.clear_managers()
-        
+
     @classmethod
     def teardown_class(cls):
        pool.clear_managers()
@@ -82,17 +82,17 @@ class PoolTest(PoolTestBase):
         expected = [(1, )]
         for row in cursor:
             eq_(row, expected.pop(0))
-    
+
     def test_no_connect_on_recreate(self):
         def creator():
             raise Exception("no creates allowed")
-        
+
         for cls in (pool.SingletonThreadPool, pool.StaticPool, 
                     pool.QueuePool, pool.NullPool, pool.AssertionPool):
             p = cls(creator=creator)
             p.dispose()
             p.recreate()
-        
+
             mock_dbapi = MockDBAPI()
             p = cls(creator=mock_dbapi.connect)
             conn = p.connect()
@@ -100,8 +100,8 @@ class PoolTest(PoolTestBase):
             mock_dbapi.throw_error = True
             p.dispose()
             p.recreate()
-            
-            
+
+
     def testthreadlocal_del(self):
         self._do_testthreadlocal(useclose=False)
 
@@ -367,7 +367,7 @@ class PoolTest(PoolTestBase):
         snoop.assert_total(1, 1, 2, 1)
         c.close()
         snoop.assert_total(1, 1, 2, 2)
-    
+
     def test_listeners_callables(self):
         dbapi = MockDBAPI()
 
@@ -497,7 +497,7 @@ class QueuePoolTest(PoolTestBase):
         c1.close()
         lazy_gc()
         assert not pool._refs
-       
+
     def test_timeout(self):
         p = pool.QueuePool(creator=mock_dbapi.connect, pool_size=3,
                            max_overflow=0, use_threadlocal=False,
@@ -534,7 +534,7 @@ class QueuePoolTest(PoolTestBase):
                     timeouts.append(time.time() - now)
                     continue
                 time.sleep(4)
-                c1.close()  
+                c1.close()
 
         threads = []
         for i in xrange(10):
@@ -553,7 +553,7 @@ class QueuePoolTest(PoolTestBase):
 
     def _test_overflow(self, thread_count, max_overflow):
         gc_collect()
-        
+
         def creator():
             time.sleep(.05)
             return mock_dbapi.connect()
@@ -581,7 +581,7 @@ class QueuePoolTest(PoolTestBase):
             th.join()
  
         self.assert_(max(peaks) <= max_overflow)
-        
+
         lazy_gc()
         assert not pool._refs
  
@@ -606,6 +606,32 @@ class QueuePoolTest(PoolTestBase):
         lazy_gc()
         assert not pool._refs
 
+    def test_overflow_no_gc_tlocal(self):
+        self._test_overflow_no_gc(True)
+
+    def test_overflow_no_gc(self):
+        self._test_overflow_no_gc(False)
+
+    def _test_overflow_no_gc(self, threadlocal):
+        p = pool.QueuePool(creator=mock_dbapi.connect, pool_size=2,
+                           max_overflow=2, use_threadlocal=False)
+
+        # disable weakref collection of the 
+        # underlying connections
+        strong_refs = set()
+        def _conn():
+            c = p.connect()
+            strong_refs.add(c.connection)
+            return c
+
+        for j in xrange(5):
+            conns = [_conn() for i in xrange(4)]
+            for c in conns:
+                c.close()
+
+        still_opened = len([c for c in strong_refs if not c.closed])
+        eq_(still_opened, 2)
+
     def test_weakref_kaboom(self):
         p = pool.QueuePool(creator=mock_dbapi.connect, pool_size=3,
                            max_overflow=-1, use_threadlocal=True)
@@ -731,15 +757,31 @@ class QueuePoolTest(PoolTestBase):
 class SingletonThreadPoolTest(PoolTestBase):
 
     def test_cleanup(self):
+        self._test_cleanup(False)
+
+    def test_cleanup_no_gc(self):
+        self._test_cleanup(True)
+
+    def _test_cleanup(self, strong_refs):
         """test that the pool's connections are OK after cleanup() has
         been called."""
 
         p = pool.SingletonThreadPool(creator=mock_dbapi.connect,
                 pool_size=3)
 
+        if strong_refs:
+            sr = set()
+            def _conn():
+                c = p.connect()
+                sr.add(c.connection)
+                return c
+        else:
+            def _conn():
+                return p.connect()
+
         def checkout():
             for x in xrange(10):
-                c = p.connect()
+                c = _conn()
                 assert c
                 c.cursor()
                 c.close()
@@ -754,6 +796,10 @@ class SingletonThreadPoolTest(PoolTestBase):
             th.join()
         assert len(p._all_conns) == 3
 
+        if strong_refs:
+            still_opened = len([c for c in sr if not c.closed])
+            eq_(still_opened, 3)
+
 class NullPoolTest(PoolTestBase):
     def test_reconnect(self):
         dbapi = MockDBAPI()
