@@ -184,11 +184,11 @@ class CursorErrTest(TestBase):
 
     def setup(self):
         global db, dbapi
-        
+
         class MDBAPI(MockDBAPI):
             def connect(self, *args, **kwargs):
                 return MConn(self)
-            
+
         class MConn(MockConnection):
             def cursor(self):
                 return MCursor(self)
@@ -202,16 +202,16 @@ class CursorErrTest(TestBase):
         db = tsa.create_engine(
                     'postgresql://foo:bar@localhost/test', 
                     module=dbapi, _initialize=False)
-    
+
     def test_cursor_explode(self):
         conn = db.connect()
         result = conn.execute("select foo")
         result.close()
         conn.close()
-    
+
     def teardown(self):
         db.dispose()
-        
+
 engine = None
 class RealReconnectTest(TestBase):
     def setup(self):
@@ -221,6 +221,8 @@ class RealReconnectTest(TestBase):
     def teardown(self):
         engine.dispose()
 
+    @testing.fails_on('+informixdb',
+                      "Wrong error thrown, fix in informixdb?")
     def test_reconnect(self):
         conn = engine.connect()
 
@@ -256,19 +258,19 @@ class RealReconnectTest(TestBase):
         assert not conn.invalidated
 
         conn.close()
-    
+
     def test_invalidate_twice(self):
         conn = engine.connect()
         conn.invalidate()
         conn.invalidate()
-    
+
     def test_explode_in_initializer(self):
         engine = engines.testing_engine()
         def broken_initialize(connection):
             connection.execute("select fake_stuff from _fake_table")
-            
+
         engine.dialect.initialize = broken_initialize
-        
+
         # raises a DBAPIError, not an AttributeError
         assert_raises(exc.DBAPIError, engine.connect)
 
@@ -277,18 +279,20 @@ class RealReconnectTest(TestBase):
         engine.dispose()
 
         p1 = engine.pool
-        
+
         def is_disconnect(e):
             return True
-            
+
         engine.dialect.is_disconnect = is_disconnect
 
         # invalidate() also doesn't screw up
         assert_raises(exc.DBAPIError, engine.connect)
-        
+
         # pool was recreated
         assert engine.pool is not p1
-        
+
+    @testing.fails_on('+informixdb',
+                      "Wrong error thrown, fix in informixdb?")
     def test_null_pool(self):
         engine = \
             engines.reconnecting_engine(options=dict(poolclass=pool.NullPool))
@@ -306,7 +310,9 @@ class RealReconnectTest(TestBase):
         assert conn.invalidated
         eq_(conn.execute(select([1])).scalar(), 1)
         assert not conn.invalidated
-        
+
+    @testing.fails_on('+informixdb',
+                      "Wrong error thrown, fix in informixdb?")
     def test_close(self):
         conn = engine.connect()
         eq_(conn.execute(select([1])).scalar(), 1)
@@ -325,6 +331,8 @@ class RealReconnectTest(TestBase):
         conn = engine.connect()
         eq_(conn.execute(select([1])).scalar(), 1)
 
+    @testing.fails_on('+informixdb',
+                      "Wrong error thrown, fix in informixdb?")
     def test_with_transaction(self):
         conn = engine.connect()
         trans = conn.begin()
@@ -376,7 +384,7 @@ class RecycleTest(TestBase):
             conn = engine.contextual_connect()
             eq_(conn.execute(select([1])).scalar(), 1)
             conn.close()
-    
+
 meta, table, engine = None, None, None
 class InvalidateDuringResultTest(TestBase):
     def setup(self):
@@ -401,6 +409,8 @@ class InvalidateDuringResultTest(TestBase):
     @testing.fails_on('+pg8000',
                       "Buffers the result set and doesn't check for "
                       "connection close")
+    @testing.fails_on('+informixdb',
+                      "Wrong error thrown, fix in informixdb?")
     def test_invalidate_on_results(self):
         conn = engine.connect()
         result = conn.execute('select * from sometable')
