@@ -77,7 +77,7 @@ class TransactionTest(TestBase):
         result = connection.execute("select * from query_users")
         assert len(result.fetchall()) == 0
         connection.close()
-    
+
     def test_transaction_container(self):
 
         def go(conn, table, data):
@@ -93,7 +93,7 @@ class TransactionTest(TestBase):
                       {'user_id': 1, 'user_name': 'user3'}])
         eq_(testing.db.execute(users.select()).fetchall(), [(1, 'user1'
             )])
-        
+
     def test_nested_rollback(self):
         connection = testing.db.connect()
         try:
@@ -135,10 +135,10 @@ class TransactionTest(TestBase):
             conn2 = connection.execution_options(dummy=True)
             conn2.execute(users.insert(), user_id=2, user_name='user2')
             transaction.rollback()
-            eq_(connection.scalar("select count(1) from query_users"), 0)
+            eq_(connection.scalar("select count(*) from query_users"), 0)
         finally:
             connection.close()
-        
+
     def test_nesting(self):
         connection = testing.db.connect()
         transaction = connection.begin()
@@ -150,7 +150,7 @@ class TransactionTest(TestBase):
         connection.execute(users.insert(), user_id=5, user_name='user5')
         trans2.commit()
         transaction.rollback()
-        self.assert_(connection.scalar('select count(1) from '
+        self.assert_(connection.scalar('select count(*) from '
                      'query_users') == 0)
         result = connection.execute('select * from query_users')
         assert len(result.fetchall()) == 0
@@ -170,7 +170,7 @@ class TransactionTest(TestBase):
         assert connection.in_transaction()
         transaction.commit()
         assert not connection.in_transaction()
-        self.assert_(connection.scalar('select count(1) from '
+        self.assert_(connection.scalar('select count(*) from '
                      'query_users') == 5)
         result = connection.execute('select * from query_users')
         assert len(result.fetchall()) == 5
@@ -190,7 +190,7 @@ class TransactionTest(TestBase):
         assert connection.in_transaction()
         transaction.close()
         assert not connection.in_transaction()
-        self.assert_(connection.scalar('select count(1) from '
+        self.assert_(connection.scalar('select count(*) from '
                      'query_users') == 0)
         result = connection.execute('select * from query_users')
         assert len(result.fetchall()) == 0
@@ -383,7 +383,7 @@ class AutoRollbackTest(TestBase):
 class ExplicitAutoCommitTest(TestBase):
 
     """test the 'autocommit' flag on select() and text() objects.
-    
+
     Requires PostgreSQL so that we may define a custom function which
     modifies the database. """
 
@@ -444,7 +444,7 @@ class ExplicitAutoCommitTest(TestBase):
         eq_(conn2.execute(select([foo.c.data])).fetchall(), [('data1',
             )])
 
-        # connection supercedes statement
+        # connection supersedes statement
 
         conn1.execution_options(autocommit=False).\
             execute(select([func.insert_foo('data2'
@@ -539,6 +539,42 @@ class TLTransactionTest(TestBase):
 
         tlengine.close()
 
+    def test_rollback_no_trans(self):
+        tlengine = create_engine(testing.db.url, strategy="threadlocal")
+
+        # shouldn't fail
+        tlengine.rollback()
+
+        tlengine.begin()
+        tlengine.rollback()
+
+        # shouldn't fail
+        tlengine.rollback()
+
+    def test_commit_no_trans(self):
+        tlengine = create_engine(testing.db.url, strategy="threadlocal")
+
+        # shouldn't fail
+        tlengine.commit()
+
+        tlengine.begin()
+        tlengine.rollback()
+
+        # shouldn't fail
+        tlengine.commit()
+
+    def test_prepare_no_trans(self):
+        tlengine = create_engine(testing.db.url, strategy="threadlocal")
+
+        # shouldn't fail
+        tlengine.prepare()
+
+        tlengine.begin()
+        tlengine.rollback()
+
+        # shouldn't fail
+        tlengine.prepare()
+
     def test_connection_close(self):
         """test that when connections are closed for real, transactions
         are rolled back and disposed."""
@@ -602,9 +638,30 @@ class TLTransactionTest(TestBase):
         finally:
             external_connection.close()
 
+    def test_with_interface(self):
+        trans = tlengine.begin()
+        tlengine.execute(users.insert(), user_id=1, user_name='user1')
+        tlengine.execute(users.insert(), user_id=2, user_name='user2')
+        trans.commit()
+
+        trans = tlengine.begin()
+        tlengine.execute(users.insert(), user_id=3, user_name='user3')
+        trans.__exit__(Exception, "fake", None)
+        trans = tlengine.begin()
+        tlengine.execute(users.insert(), user_id=4, user_name='user4')
+        trans.__exit__(None, None, None)
+        eq_(
+            tlengine.execute(users.select().order_by(users.c.user_id)).fetchall(),
+            [
+                (1, 'user1'),
+                (2, 'user2'),
+                (4, 'user4'),
+            ]
+        )
+
     def test_commits(self):
         connection = tlengine.connect()
-        assert connection.execute('select count(1) from query_users'
+        assert connection.execute('select count(*) from query_users'
                                   ).scalar() == 0
         connection.close()
         connection = tlengine.contextual_connect()
@@ -697,7 +754,7 @@ class TLTransactionTest(TestBase):
         tlengine.rollback()
         try:
             self.assert_(external_connection.scalar(
-                        'select count(1) from query_users'
+                        'select count(*) from query_users'
                          ) == 0)
         finally:
             external_connection.close()
@@ -719,7 +776,7 @@ class TLTransactionTest(TestBase):
         tlengine.commit()
         try:
             self.assert_(external_connection.scalar(
-                        'select count(1) from query_users'
+                        'select count(*) from query_users'
                          ) == 5)
         finally:
             external_connection.close()
@@ -751,7 +808,7 @@ class TLTransactionTest(TestBase):
         conn.close()
         try:
             self.assert_(external_connection.scalar(
-                        'select count(1) from query_users'
+                        'select count(*) from query_users'
                          ) == 0)
         finally:
             external_connection.close()
@@ -778,7 +835,7 @@ class TLTransactionTest(TestBase):
         connection.close()
         try:
             self.assert_(external_connection.scalar(
-                        'select count(1) from query_users'
+                        'select count(*) from query_users'
                          ) == 0)
         finally:
             external_connection.close()
@@ -954,11 +1011,11 @@ class ForUpdateTest(TestBase):
     @testing.requires.independent_connections
     def test_queued_update(self):
         """Test SELECT FOR UPDATE with concurrent modifications.
-        
+
         Runs concurrent modifications on a single row in the users
         table, with each mutator trying to increment a value stored in
         user_name.
-        
+
         """
 
         db = testing.db
