@@ -1,13 +1,16 @@
 from sqlalchemy.test.testing import eq_, assert_raises
 import copy
-import gc
 import pickle
 
 from sqlalchemy import *
 from sqlalchemy.orm import *
 from sqlalchemy.orm.collections import collection
 from sqlalchemy.ext.associationproxy import *
+from sqlalchemy.ext.associationproxy import _AssociationList
 from sqlalchemy.test import *
+from sqlalchemy.test.util import gc_collect
+from sqlalchemy.sql import not_
+from test.orm import _base
 
 
 class DictCollection(dict):
@@ -88,7 +91,7 @@ class _CollectionOperations(TestBase):
                     self.name = name
 
         mapper(Parent, parents_table, properties={
-            '_children': relation(Child, lazy=False,
+            '_children': relationship(Child, lazy='joined',
                                   collection_class=collection_class)})
         mapper(Child, children_table)
 
@@ -607,14 +610,77 @@ class CustomObjectTest(_CollectionOperations):
         p = self.roundtrip(p)
         self.assert_(len(list(p.children)) == 1)
 
-        # We didn't provide an alternate _AssociationList implementation for
-        # our ObjectCollection, so indexing will fail.
+        # We didn't provide an alternate _AssociationList implementation
+        # for our ObjectCollection, so indexing will fail.
+
         try:
             v = p.children[1]
             self.fail()
         except TypeError:
             pass
 
+class ProxyFactoryTest(ListTest):
+    def setup(self):
+        metadata = MetaData(testing.db)
+
+        parents_table = Table('Parent', metadata,
+                              Column('id', Integer, primary_key=True,
+                                     test_needs_autoincrement=True),
+                              Column('name', String(128)))
+        children_table = Table('Children', metadata,
+                               Column('id', Integer, primary_key=True,
+                                      test_needs_autoincrement=True),
+                               Column('parent_id', Integer,
+                                      ForeignKey('Parent.id')),
+                               Column('foo', String(128)),
+                               Column('name', String(128)))
+
+        class CustomProxy(_AssociationList):
+            def __init__(
+                self,
+                lazy_collection,
+                creator,
+                value_attr,
+                parent,
+                ):
+                getter, setter = parent._default_getset(lazy_collection)
+                _AssociationList.__init__(
+                    self,
+                    lazy_collection,
+                    creator,
+                    getter,
+                    setter,
+                    parent,
+                    )
+
+        class Parent(object):
+            children = association_proxy('_children', 'name', 
+                        proxy_factory=CustomProxy, 
+                        proxy_bulk_set=CustomProxy.extend
+                    )
+
+            def __init__(self, name):
+                self.name = name
+
+        class Child(object):
+            def __init__(self, name):
+                self.name = name
+
+        mapper(Parent, parents_table, properties={
+            '_children': relationship(Child, lazy='joined',
+                                  collection_class=list)})
+        mapper(Child, children_table)
+
+        metadata.create_all()
+
+        self.metadata = metadata
+        self.session = create_session()
+        self.Parent, self.Child = Parent, Child
+
+    def test_sequence_ops(self):
+        self._test_sequence_ops()
+
+
 class ScalarTest(TestBase):
     def test_scalar_proxy(self):
         metadata = MetaData(testing.db)
@@ -648,7 +714,7 @@ class ScalarTest(TestBase):
                     setattr(self, attr, kw[attr])
 
         mapper(Parent, parents_table, properties={
-            'child': relation(Child, lazy=False,
+            'child': relationship(Child, lazy='joined',
                               backref='parent', uselist=False)})
         mapper(Child, children_table)
 
@@ -783,7 +849,7 @@ class LazyLoadTest(TestBase):
         Parent, Child = self.Parent, self.Child
 
         mapper(Parent, self.table, properties={
-            '_children': relation(Child, lazy=True,
+            '_children': relationship(Child, lazy='select',
                                   collection_class=list)})
 
         p = Parent('p')
@@ -801,7 +867,7 @@ class LazyLoadTest(TestBase):
         Parent, Child = self.Parent, self.Child
 
         mapper(Parent, self.table, properties={
-            '_children': relation(Child, lazy=False,
+            '_children': relationship(Child, lazy='joined',
                                   collection_class=list)})
 
         p = Parent('p')
@@ -816,7 +882,7 @@ class LazyLoadTest(TestBase):
         Parent, Child = self.Parent, self.Child
 
         mapper(Parent, self.table, properties={
-            '_children': relation(Child, lazy=True, uselist=False)})
+            '_children': relationship(Child, lazy='select', uselist=False)})
 
 
         p = Parent('p')
@@ -831,7 +897,7 @@ class LazyLoadTest(TestBase):
         Parent, Child = self.Parent, self.Child
 
         mapper(Parent, self.table, properties={
-            '_children': relation(Child, lazy=False, uselist=False)})
+            '_children': relationship(Child, lazy='joined', uselist=False)})
 
 
         p = Parent('p')
@@ -844,106 +910,270 @@ class LazyLoadTest(TestBase):
 
 
 class ReconstitutionTest(TestBase):
+
     def setup(self):
         metadata = MetaData(testing.db)
-        parents = Table('parents', metadata,
-                        Column('id', Integer, primary_key=True,
-                               test_needs_autoincrement=True),
-                        Column('name', String(30)))
-        children = Table('children', metadata,
-                         Column('id', Integer, primary_key=True,
-                                test_needs_autoincrement=True),
-                         Column('parent_id', Integer, ForeignKey('parents.id')),
-                         Column('name', String(30)))
+        parents = Table('parents', metadata, Column('id', Integer,
+                        primary_key=True,
+                        test_needs_autoincrement=True), Column('name',
+                        String(30)))
+        children = Table('children', metadata, Column('id', Integer,
+                         primary_key=True,
+                         test_needs_autoincrement=True),
+                         Column('parent_id', Integer,
+                         ForeignKey('parents.id')), Column('name',
+                         String(30)))
         metadata.create_all()
         parents.insert().execute(name='p1')
-
-
         self.metadata = metadata
         self.parents = parents
         self.children = children
-        
+
     def teardown(self):
         self.metadata.drop_all()
         clear_mappers()
 
     def test_weak_identity_map(self):
-        mapper(Parent, self.parents, properties=dict(children=relation(Child)))
+        mapper(Parent, self.parents,
+               properties=dict(children=relationship(Child)))
         mapper(Child, self.children)
-
         session = create_session(weak_identity_map=True)
 
         def add_child(parent_name, child_name):
-            parent = (session.query(Parent).
-                      filter_by(name=parent_name)).one()
+            parent = \
+                session.query(Parent).filter_by(name=parent_name).one()
             parent.kids.append(child_name)
 
-
         add_child('p1', 'c1')
-        gc.collect()
+        gc_collect()
         add_child('p1', 'c2')
-
         session.flush()
         p = session.query(Parent).filter_by(name='p1').one()
         assert set(p.kids) == set(['c1', 'c2']), p.kids
 
     def test_copy(self):
-        mapper(Parent, self.parents, properties=dict(children=relation(Child)))
+        mapper(Parent, self.parents,
+               properties=dict(children=relationship(Child)))
         mapper(Child, self.children)
-
         p = Parent('p1')
         p.kids.extend(['c1', 'c2'])
         p_copy = copy.copy(p)
         del p
-        gc.collect()
-
+        gc_collect()
         assert set(p_copy.kids) == set(['c1', 'c2']), p.kids
 
     def test_pickle_list(self):
-        mapper(Parent, self.parents, properties=dict(children=relation(Child)))
+        mapper(Parent, self.parents,
+               properties=dict(children=relationship(Child)))
         mapper(Child, self.children)
-
         p = Parent('p1')
         p.kids.extend(['c1', 'c2'])
-
         r1 = pickle.loads(pickle.dumps(p))
         assert r1.kids == ['c1', 'c2']
-
         r2 = pickle.loads(pickle.dumps(p.kids))
         assert r2 == ['c1', 'c2']
 
     def test_pickle_set(self):
-        mapper(Parent, self.parents, properties=dict(children=relation(Child, collection_class=set)))
+        mapper(Parent, self.parents,
+               properties=dict(children=relationship(Child,
+               collection_class=set)))
         mapper(Child, self.children)
-
         p = Parent('p1')
         p.kids.update(['c1', 'c2'])
-
         r1 = pickle.loads(pickle.dumps(p))
         assert r1.kids == set(['c1', 'c2'])
-
         r2 = pickle.loads(pickle.dumps(p.kids))
         assert r2 == set(['c1', 'c2'])
 
     def test_pickle_dict(self):
-        mapper(Parent, self.parents, properties=dict(
-                    children=relation(KVChild, collection_class=collections.mapped_collection(PickleKeyFunc('name')))
-                ))
+        mapper(Parent, self.parents,
+               properties=dict(children=relationship(KVChild,
+               collection_class=
+                    collections.mapped_collection(PickleKeyFunc('name')))))
         mapper(KVChild, self.children)
-
         p = Parent('p1')
-        p.kids.update({'c1':'v1', 'c2':'v2'})
-        assert p.kids == {'c1':'c1', 'c2':'c2'}
-        
+        p.kids.update({'c1': 'v1', 'c2': 'v2'})
+        assert p.kids == {'c1': 'c1', 'c2': 'c2'}
         r1 = pickle.loads(pickle.dumps(p))
-        assert r1.kids == {'c1':'c1', 'c2':'c2'}
-
+        assert r1.kids == {'c1': 'c1', 'c2': 'c2'}
         r2 = pickle.loads(pickle.dumps(p.kids))
-        assert r2 == {'c1':'c1', 'c2':'c2'}
+        assert r2 == {'c1': 'c1', 'c2': 'c2'}
 
 class PickleKeyFunc(object):
     def __init__(self, name):
         self.name = name
-    
+
     def __call__(self, obj):
-        return getattr(obj, self.name)
\ No newline at end of file
+        return getattr(obj, self.name)
+
+class ComparatorTest(_base.MappedTest):
+
+    run_inserts = 'once'
+    run_deletes = None
+    run_setup_mappers = 'once'
+    run_setup_classes = 'once'
+
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('userkeywords', metadata, Column('keyword_id', Integer,
+              ForeignKey('keywords.id'), primary_key=True),
+              Column('user_id', Integer, ForeignKey('users.id')))
+        Table('users', metadata, Column('id', Integer,
+              primary_key=True, test_needs_autoincrement=True),
+              Column('name', String(64)))
+        Table('keywords', metadata, Column('id', Integer,
+              primary_key=True, test_needs_autoincrement=True),
+              Column('keyword', String(64)))
+
+    @classmethod
+    def setup_classes(cls):
+        class User(_base.ComparableEntity):
+            def __init__(self, name):
+                self.name = name
+
+            keywords = association_proxy('user_keywords', 'keyword',
+                    creator=lambda k: UserKeyword(keyword=k))
+
+        class Keyword(_base.ComparableEntity):
+            def __init__(self, keyword):
+                self.keyword = keyword
+
+            user = association_proxy('user_keyword', 'user')
+
+        class UserKeyword(_base.ComparableEntity):
+            def __init__(self, user=None, keyword=None):
+                self.user = user
+                self.keyword = keyword
+
+    @classmethod
+    @testing.resolve_artifact_names
+    def setup_mappers(cls):
+        mapper(User, users)
+        mapper(Keyword, keywords, properties={'user_keyword'
+               : relationship(UserKeyword, uselist=False)})
+        mapper(UserKeyword, userkeywords, properties={'user'
+               : relationship(User, backref='user_keywords'), 'keyword'
+               : relationship(Keyword)})
+
+    @classmethod
+    @testing.resolve_artifact_names
+    def insert_data(cls):
+        session = sessionmaker()()
+        words = (
+            'quick',
+            'brown',
+            'fox',
+            'jumped',
+            'over',
+            'the',
+            'lazy',
+            )
+        for ii in range(4):
+            user = User('user%d' % ii)
+            session.add(user)
+            for jj in words[ii:ii + 3]:
+                user.keywords.append(Keyword(jj))
+        orphan = Keyword('orphan')
+        orphan.user_keyword = UserKeyword(keyword=orphan, user=None)
+        session.add(orphan)
+        session.commit()
+        cls.u = user
+        cls.kw = user.keywords[0]
+        cls.session = session
+
+    def _equivalent(self, q_proxy, q_direct):
+        eq_(q_proxy.all(), q_direct.all())
+
+    @testing.resolve_artifact_names
+    def test_filter_any_kwarg(self):
+        self._equivalent(self.session.query(User).
+                    filter(User.keywords.any(keyword='jumped'
+                         )),
+                         self.session.query(User).filter(
+                                User.user_keywords.any(
+                            UserKeyword.keyword.has(keyword='jumped'
+                         ))))
+
+    @testing.resolve_artifact_names
+    def test_filter_has_kwarg(self):
+        self._equivalent(self.session.query(Keyword).
+                    filter(Keyword.user.has(name='user2'
+                         )),
+                         self.session.query(Keyword).
+                            filter(Keyword.user_keyword.has(
+                            UserKeyword.user.has(name='user2'
+                         ))))
+
+    @testing.resolve_artifact_names
+    def test_filter_any_criterion(self):
+        self._equivalent(self.session.query(User).
+                    filter(User.keywords.any(Keyword.keyword
+                         == 'jumped')),
+                         self.session.query(User).
+                            filter(User.user_keywords.any(
+                            UserKeyword.keyword.has(Keyword.keyword
+                         == 'jumped'))))
+
+    @testing.resolve_artifact_names
+    def test_filter_has_criterion(self):
+        self._equivalent(self.session.query(Keyword).
+                filter(Keyword.user.has(User.name
+                         == 'user2')),
+                         self.session.query(Keyword).
+                            filter(Keyword.user_keyword.has(
+                                UserKeyword.user.has(User.name
+                         == 'user2'))))
+
+    @testing.resolve_artifact_names
+    def test_filter_contains(self):
+        self._equivalent(self.session.query(User).
+        filter(User.keywords.contains(self.kw)),
+                         self.session.query(User).
+                         filter(User.user_keywords.any(keyword=self.kw)))
+
+    @testing.resolve_artifact_names
+    def test_filter_eq(self):
+        self._equivalent(self.session.query(Keyword).filter(Keyword.user
+                         == self.u),
+                         self.session.query(Keyword).
+                         filter(Keyword.user_keyword.has(user=self.u)))
+
+    @testing.resolve_artifact_names
+    def test_filter_ne(self):
+        self._equivalent(self.session.query(Keyword).filter(Keyword.user
+                         != self.u),
+                         self.session.query(Keyword).
+                         filter(not_(Keyword.user_keyword.has(user=self.u))))
+
+    @testing.resolve_artifact_names
+    def test_filter_eq_null(self):
+        self._equivalent(self.session.query(Keyword).filter(Keyword.user
+                         == None),
+                         self.session.query(Keyword).
+                            filter(Keyword.user_keyword.has(UserKeyword.user
+                         == None)))
+
+    @testing.resolve_artifact_names
+    def test_filter_scalar_contains_fails(self):
+        assert_raises(exceptions.InvalidRequestError, lambda : \
+                      Keyword.user.contains(self.u))
+
+    @testing.resolve_artifact_names
+    def test_filter_scalar_any_fails(self):
+        assert_raises(exceptions.InvalidRequestError, lambda : \
+                      Keyword.user.any(name='user2'))
+
+    @testing.resolve_artifact_names
+    def test_filter_collection_has_fails(self):
+        assert_raises(exceptions.InvalidRequestError, lambda : \
+                      User.keywords.has(keyword='quick'))
+
+    @testing.resolve_artifact_names
+    def test_filter_collection_eq_fails(self):
+        assert_raises(exceptions.InvalidRequestError, lambda : \
+                      User.keywords == self.kw)
+
+    @testing.resolve_artifact_names
+    def test_filter_collection_ne_fails(self):
+        assert_raises(exceptions.InvalidRequestError, lambda : \
+                      User.keywords != self.kw)
