@@ -4,7 +4,7 @@ from sqlalchemy.orm.collections import collection
 from sqlalchemy.orm.interfaces import AttributeExtension
 from sqlalchemy import exc as sa_exc
 from sqlalchemy.test import *
-from sqlalchemy.test.testing import eq_, ne_, assert_raises
+from sqlalchemy.test.testing import eq_, ne_, assert_raises, assert_raises_message
 from test.orm import _base
 from sqlalchemy.test.util import gc_collect
 from sqlalchemy.util import cmp, jython
@@ -109,13 +109,13 @@ class AttributesTest(_base.ORMTest):
         self.assert_(len(o4.mt2) == 1)
         self.assert_(o4.mt2[0].a == 'abcde')
         self.assert_(o4.mt2[0].b is None)
-    
+
     def test_state_gc(self):
         """test that InstanceState always has a dict, even after host object gc'ed."""
-        
+
         class Foo(object):
             pass
-        
+
         attributes.register_class(Foo)
         f = Foo()
         state = attributes.instance_state(f)
@@ -125,7 +125,29 @@ class AttributesTest(_base.ORMTest):
         gc_collect()
         assert state.obj() is None
         assert state.dict == {}
-        
+
+    def test_object_dereferenced_error(self):
+        class Foo(object):
+            pass
+        class Bar(object):
+            def __init__(self):
+                gc_collect()
+
+        attributes.register_class(Foo)
+        attributes.register_class(Bar)
+        attributes.register_attribute(Foo, 
+                                    'bars', 
+                                    uselist=True, 
+                                    useobject=True)
+
+        assert_raises_message(
+            sa_exc.SAWarning,
+            "Can't emit change event for attribute "
+            "'Foo.bars' - parent object of type <Foo> "
+            "has been garbage collected.",
+            lambda: Foo().bars.append(Bar())
+        )
+
     def test_deferred(self):
         class Foo(object):pass
 
@@ -220,24 +242,24 @@ class AttributesTest(_base.ORMTest):
         a.email_address = 'foo@bar.com'
         u.addresses.append(a)
         self.assert_(u.user_id == 7 and u.user_name == 'heythere' and u.addresses[0].email_address == 'lala@123.com' and u.addresses[1].email_address == 'foo@bar.com')
-    
+
     def test_extension_commit_attr(self):
         """test that an extension which commits attribute history
         maintains the end-result history.
-        
+
         This won't work in conjunction with some unitofwork extensions.
-        
+
         """
-        
+
         class Foo(_base.ComparableEntity):
             pass
         class Bar(_base.ComparableEntity):
             pass
-        
+
         class ReceiveEvents(AttributeExtension):
             def __init__(self, key):
                 self.key = key
-                
+
             def append(self, state, child, initiator):
                 if commit:
                     state.commit_all(state.dict)
@@ -257,34 +279,34 @@ class AttributesTest(_base.ORMTest):
         attributes.register_class(Bar)
 
         b1, b2, b3, b4 = Bar(id='b1'), Bar(id='b2'), Bar(id='b3'), Bar(id='b4')
-        
+
         def loadcollection(**kw):
             if kw.get('passive') is attributes.PASSIVE_NO_FETCH:
                 return attributes.PASSIVE_NO_RESULT
             return [b1, b2]
-        
+
         def loadscalar(**kw):
             if kw.get('passive') is attributes.PASSIVE_NO_FETCH:
                 return attributes.PASSIVE_NO_RESULT
             return b2
-            
+
         attributes.register_attribute(Foo, 'bars', 
                                uselist=True, 
                                useobject=True, 
                                callable_=lambda o:loadcollection,
                                extension=[ReceiveEvents('bars')])
-                               
+
         attributes.register_attribute(Foo, 'bar', 
                               uselist=False, 
                               useobject=True, 
                               callable_=lambda o:loadscalar,
                               extension=[ReceiveEvents('bar')])
-                              
+
         attributes.register_attribute(Foo, 'scalar', 
                             uselist=False, 
                             useobject=False, extension=[ReceiveEvents('scalar')])
-        
-            
+
+
         def create_hist():
             def hist(key, shouldmatch, fn, *arg):
                 attributes.instance_state(f1).commit_all(attributes.instance_dict(f1))
@@ -301,7 +323,7 @@ class AttributesTest(_base.ORMTest):
             hist('scalar', True, setattr, f1, 'scalar', 5)
             hist('scalar', True, setattr, f1, 'scalar', None)
             hist('scalar', True, setattr, f1, 'scalar', 4)
-        
+
         histories = []
         commit = False
         create_hist()
@@ -317,7 +339,7 @@ class AttributesTest(_base.ORMTest):
                 eq_(woc, wic)
             else:
                 ne_(woc, wic)
-        
+
     def test_extension_lazyload_assertion(self):
         class Foo(_base.BasicEntity):
             pass
@@ -343,7 +365,7 @@ class AttributesTest(_base.ORMTest):
         def func1(**kw):
             if kw.get('passive') is attributes.PASSIVE_NO_FETCH:
                 return attributes.PASSIVE_NO_RESULT
-            
+
             return [bar1, bar2, bar3]
 
         attributes.register_attribute(Foo, 'bars', uselist=True, callable_=lambda o:func1, useobject=True, extension=[ReceiveEvents()])
@@ -351,20 +373,20 @@ class AttributesTest(_base.ORMTest):
 
         x = Foo()
         assert_raises(AssertionError, Bar(id=4).foos.append, x)
-        
+
         x.bars
         b = Bar(id=4)
         b.foos.append(x)
         attributes.instance_state(x).expire_attributes(attributes.instance_dict(x), ['bars'])
         assert_raises(AssertionError, b.foos.remove, x)
-        
-        
+
+
     def test_scalar_listener(self):
         # listeners on ScalarAttributeImpl and MutableScalarAttributeImpl aren't used normally.
         # test that they work for the benefit of user extensions
         class Foo(object):
             pass
-        
+
         results = []
         class ReceiveEvents(AttributeExtension):
             def append(self, state, child, initiator):
@@ -376,11 +398,11 @@ class AttributesTest(_base.ORMTest):
             def set(self, state, child, oldchild, initiator):
                 results.append(("set", state.obj(), child, oldchild))
                 return child
-        
+
         attributes.register_class(Foo)
         attributes.register_attribute(Foo, 'x', uselist=False, mutable_scalars=False, useobject=False, extension=ReceiveEvents())
         attributes.register_attribute(Foo, 'y', uselist=False, mutable_scalars=True, useobject=False, copy_function=lambda x:x, extension=ReceiveEvents())
-        
+
         f = Foo()
         f.x = 5
         f.x = 17
@@ -388,7 +410,7 @@ class AttributesTest(_base.ORMTest):
         f.y = [1,2,3]
         f.y = [4,5,6]
         del f.y
-        
+
         eq_(results, [
             ('set', f, 5, None),
             ('set', f, 17, 5),
@@ -397,12 +419,12 @@ class AttributesTest(_base.ORMTest):
             ('set', f, [4,5,6], [1,2,3]),
             ('remove', f, [4,5,6])
         ])
-        
-        
+
+
     def test_lazytrackparent(self):
         """test that the "hasparent" flag works properly 
            when lazy loaders and backrefs are used
-           
+
         """
 
         class Post(object):pass
@@ -621,7 +643,7 @@ class AttributesTest(_base.ORMTest):
 
         attributes.unregister_attribute(Foo, "collection")
         assert not attributes.manager_of_class(Foo).is_instrumented("collection")
-        
+
         try:
             attributes.register_attribute(Foo, "collection", uselist=True, typecallable=dict, useobject=True)
             assert False
@@ -671,11 +693,11 @@ class UtilTest(_base.ORMTest):
 
         class Bar(object):
             pass
-        
+
         attributes.register_class(Foo)
         attributes.register_class(Bar)
         attributes.register_attribute(Foo, "coll", uselist=True, useobject=True)
-    
+
         f1 = Foo()
         b1 = Bar()
         b2 = Bar()
@@ -687,20 +709,24 @@ class UtilTest(_base.ORMTest):
         eq_(attributes.get_history(f1, "coll"), ([b1], [], []))
         attributes.set_committed_value(f1, "coll", [b2])
         eq_(attributes.get_history(f1, "coll"), ((), [b2], ()))
-        
+
         attributes.del_attribute(f1, "coll")
         assert "coll" not in f1.__dict__
 
 class BackrefTest(_base.ORMTest):
 
-    def test_manytomany(self):
+    def test_m2m(self):
         class Student(object):pass
         class Course(object):pass
 
         attributes.register_class(Student)
         attributes.register_class(Course)
-        attributes.register_attribute(Student, 'courses', uselist=True, extension=attributes.GenericBackrefExtension('students'), useobject=True)
-        attributes.register_attribute(Course, 'students', uselist=True, extension=attributes.GenericBackrefExtension('courses'), useobject=True)
+        attributes.register_attribute(Student, 'courses', uselist=True,
+                extension=attributes.GenericBackrefExtension('students'
+                ), useobject=True)
+        attributes.register_attribute(Course, 'students', uselist=True,
+                extension=attributes.GenericBackrefExtension('courses'
+                ), useobject=True)
 
         s = Student()
         c = Course()
@@ -717,14 +743,18 @@ class BackrefTest(_base.ORMTest):
         s1.courses.remove(c)
         self.assert_(c.students == [s2,s3])
 
-    def test_onetomany(self):
+    def test_o2m(self):
         class Post(object):pass
         class Blog(object):pass
 
         attributes.register_class(Post)
         attributes.register_class(Blog)
-        attributes.register_attribute(Post, 'blog', uselist=False, extension=attributes.GenericBackrefExtension('posts'), trackparent=True, useobject=True)
-        attributes.register_attribute(Blog, 'posts', uselist=True, extension=attributes.GenericBackrefExtension('blog'), trackparent=True, useobject=True)
+        attributes.register_attribute(Post, 'blog', uselist=False,
+                extension=attributes.GenericBackrefExtension('posts'),
+                trackparent=True, useobject=True)
+        attributes.register_attribute(Blog, 'posts', uselist=True,
+                extension=attributes.GenericBackrefExtension('blog'),
+                trackparent=True, useobject=True)
         b = Blog()
         (p1, p2, p3) = (Post(), Post(), Post())
         b.posts.append(p1)
@@ -748,13 +778,17 @@ class BackrefTest(_base.ORMTest):
         p5.blog = None
         del p5.blog
 
-    def test_onetoone(self):
+    def test_o2o(self):
         class Port(object):pass
         class Jack(object):pass
         attributes.register_class(Port)
         attributes.register_class(Jack)
-        attributes.register_attribute(Port, 'jack', uselist=False, extension=attributes.GenericBackrefExtension('port'), useobject=True)
-        attributes.register_attribute(Jack, 'port', uselist=False, extension=attributes.GenericBackrefExtension('jack'), useobject=True)
+        attributes.register_attribute(Port, 'jack', uselist=False,
+                extension=attributes.GenericBackrefExtension('port'),
+                useobject=True)
+        attributes.register_attribute(Jack, 'port', uselist=False,
+                extension=attributes.GenericBackrefExtension('jack'),
+                useobject=True)
         p = Port()
         j = Jack()
         p.jack = j
@@ -764,6 +798,96 @@ class BackrefTest(_base.ORMTest):
         j.port = None
         self.assert_(p.jack is None)
 
+    def test_symmetric_o2o_inheritance(self):
+        """Test that backref 'initiator' catching goes against
+        a token that is global to all InstrumentedAttribute objects
+        within a particular class, not just the indvidual IA object
+        since we use distinct objects in an inheritance scenario.
+
+        """
+        class Parent(object):
+            pass
+        class Child(object):
+            pass
+        class SubChild(Child):
+            pass
+
+        p_token = object()
+        c_token = object()
+
+        attributes.register_class(Parent)
+        attributes.register_class(Child)
+        attributes.register_class(SubChild)
+        attributes.register_attribute(Parent, 'child', uselist=False,
+                extension=attributes.GenericBackrefExtension('parent'),
+                parent_token = p_token,
+                useobject=True)
+        attributes.register_attribute(Child, 'parent', uselist=False,
+                extension=attributes.GenericBackrefExtension('child'),
+                parent_token = c_token,
+                useobject=True)
+        attributes.register_attribute(SubChild, 'parent',
+                uselist=False,
+                extension=attributes.GenericBackrefExtension('child'),
+                parent_token = c_token,
+                useobject=True)
+
+        p1 = Parent()
+        c1 = Child()
+        p1.child = c1
+
+        c2 = SubChild()
+        c2.parent = p1
+
+    def test_symmetric_o2m_inheritance(self):
+        class Parent(object):
+            pass
+        class SubParent(Parent):
+            pass
+        class Child(object):
+            pass
+
+        p_token = object()
+        c_token = object()
+
+        attributes.register_class(Parent)
+        attributes.register_class(SubParent)
+        attributes.register_class(Child)
+        attributes.register_attribute(Parent, 'children', uselist=True,
+                extension=attributes.GenericBackrefExtension('parent'),
+                parent_token = p_token,
+                useobject=True)
+        attributes.register_attribute(SubParent, 'children', uselist=True,
+                extension=attributes.GenericBackrefExtension('parent'),
+                parent_token = p_token,
+                useobject=True)
+        attributes.register_attribute(Child, 'parent', uselist=False,
+                extension=attributes.GenericBackrefExtension('children'),
+                parent_token = c_token,
+                useobject=True)
+
+        p1 = Parent()
+        p2 = SubParent()
+        c1 = Child()
+
+        p1.children.append(c1)
+
+        assert c1.parent is p1
+        assert c1 in p1.children
+
+        p2.children.append(c1)
+        assert c1.parent is p2
+
+        # note its still in p1.children -
+        # the event model currently allows only
+        # one level deep.  without the parent_token,
+        # it keeps going until a ValueError is raised
+        # and this condition changes.
+        assert c1 in p1.children
+
+
+
+
 class PendingBackrefTest(_base.ORMTest):
     def setup(self):
         global Post, Blog, called, lazy_load
@@ -807,7 +931,7 @@ class PendingBackrefTest(_base.ORMTest):
 
         b = Blog("blog 1")
         p = Post("post 4")
-        
+
         p.blog = b
         p = Post("post 5")
         p.blog = b
@@ -817,41 +941,46 @@ class PendingBackrefTest(_base.ORMTest):
         # calling backref calls the callable, populates extra posts
         assert b.posts == [p1, p2, p3, Post("post 4"), Post("post 5")]
         assert called[0] == 1
-    
+
     def test_lazy_history(self):
         global lazy_load
 
         p1, p2, p3 = Post("post 1"), Post("post 2"), Post("post 3")
         lazy_load = [p1, p2, p3]
-        
+
         b = Blog("blog 1")
         p = Post("post 4")
         p.blog = b
-        
+
         p4 = Post("post 5")
         p4.blog = b
         assert called[0] == 0
         eq_(attributes.instance_state(b).get_history('posts'), ([p, p4], [p1, p2, p3], []))
         assert called[0] == 1
 
-    def test_lazy_remove(self):
-        global lazy_load
-        called[0] = 0
-        lazy_load = []
-
+    def test_state_on_add_remove(self):
         b = Blog("blog 1")
         p = Post("post 1")
         p.blog = b
-        assert called[0] == 0
-
-        lazy_load = [p]
-
         p.blog = None
+
+        eq_(called[0], 0)
+        eq_(b.posts, [])
+        eq_(called[0], 1)
+
+    def test_pending_combines_with_lazy(self):
+        global lazy_load
+        b = Blog("blog 1")
+        p = Post("post 1")
         p2 = Post("post 2")
-        p2.blog = b
-        assert called[0] == 0
-        assert b.posts == [p2]
-        assert called[0] == 1
+        p.blog = b
+        lazy_load = [p, p2]
+        # lazy loaded + pending get added together.
+        # This isn't seen often with the ORM due
+        # to usual practices surrounding the 
+        # load/flush/load cycle.
+        eq_(b.posts, [p, p2, p])
+        eq_(called[0], 1)
 
     def test_normal_load(self):
         global lazy_load
@@ -903,7 +1032,7 @@ class HistoryTest(_base.ORMTest):
         f = Foo()
         f.someattr = 3
         eq_(Foo.someattr.impl.get_committed_value(attributes.instance_state(f), attributes.instance_dict(f)), None)
-        
+
         attributes.instance_state(f).commit(attributes.instance_dict(f), ['someattr'])
         eq_(Foo.someattr.impl.get_committed_value(attributes.instance_state(f), attributes.instance_dict(f)), 3)
 
@@ -965,8 +1094,8 @@ class HistoryTest(_base.ORMTest):
         eq_(attributes.get_state_history(attributes.instance_state(f), 'someattr'), (['one'], (), ()))
         f.someattr = 'two'
         eq_(attributes.get_state_history(attributes.instance_state(f), 'someattr'), (['two'], (), ()))
-        
-        
+
+
     def test_mutable_scalar(self):
         class Foo(_base.BasicEntity):
             pass
@@ -1240,10 +1369,10 @@ class HistoryTest(_base.ORMTest):
 
         attributes.instance_state(f).commit_all(attributes.instance_dict(f))
         eq_(attributes.get_state_history(attributes.instance_state(f), 'someattr'), ((), [hi, there, hi], ()))
-        
+
         f.someattr = []
         eq_(attributes.get_state_history(attributes.instance_state(f), 'someattr'), ([], [], [hi, there, hi]))
-        
+
     def test_collections_via_backref(self):
         class Foo(_base.BasicEntity):
             pass
@@ -1468,10 +1597,10 @@ class HistoryTest(_base.ORMTest):
 class ListenerTest(_base.ORMTest):
     def test_receive_changes(self):
         """test that Listeners can mutate the given value.
-        
+
         This is a rudimentary test which would be better suited by a full-blown inclusion
         into collection.py.
-        
+
         """
         class Foo(object):
             pass
@@ -1493,7 +1622,7 @@ class ListenerTest(_base.ORMTest):
         attributes.register_attribute(Foo, 'barlist', uselist=True, useobject=True, extension=AlteringListener())
         attributes.register_attribute(Foo, 'barset', typecallable=set, uselist=True, useobject=True, extension=AlteringListener())
         attributes.register_attribute(Bar, 'data', uselist=False, useobject=False)
-        
+
         f1 = Foo()
         f1.data = "some data"
         eq_(f1.data, "some data modified")
@@ -1502,8 +1631,8 @@ class ListenerTest(_base.ORMTest):
         f1.barlist.append(b1)
         assert b1.data == "some bar"
         assert f1.barlist[0].data == "some bar appended"
-        
+
         f1.barset.add(b1)
         assert f1.barset.pop().data == "some bar appended"
-    
-    
+
+
