@@ -9,7 +9,7 @@ from sqlalchemy.test.schema import Table
 from sqlalchemy.test.schema import Column
 from sqlalchemy.orm import mapper, relationship, create_session, \
                         attributes, deferred, exc as orm_exc, defer, undefer,\
-                        strategies, state, lazyload, backref
+                        strategies, state, lazyload, backref, Session
 from test.orm import _base, _fixtures
 
 
@@ -92,7 +92,7 @@ class ExpireTest(_fixtures.FixtureTest):
         # trick the "deleted" flag so we can re-add for the sake
         # of this test
         del attributes.instance_state(u).deleted
-        
+
         # add it back
         s.add(u)
         # nope, raises ObjectDeletedError
@@ -101,20 +101,20 @@ class ExpireTest(_fixtures.FixtureTest):
         # do a get()/remove u from session again
         assert s.query(User).get(10) is None
         assert u not in s
-        
+
         s.rollback()
 
         assert u in s
-        # but now its back, rollback has occured, the _remove_newly_deleted
+        # but now its back, rollback has occurred, the _remove_newly_deleted
         # is reverted
         eq_(u.name, 'chuck')
 
     @testing.resolve_artifact_names
     def test_deferred(self):
         """test that unloaded, deferred attributes aren't included in the expiry list."""
-        
+
         mapper(Order, orders, properties={'description':deferred(orders.c.description)})
-        
+
         s = create_session()
         o1 = s.query(Order).first()
         assert 'description' not in o1.__dict__
@@ -122,7 +122,7 @@ class ExpireTest(_fixtures.FixtureTest):
         assert o1.isopen is not None
         assert 'description' not in o1.__dict__
         assert o1.description
-        
+
     @testing.resolve_artifact_names
     def test_lazyload_autoflushes(self):
         mapper(User, users, properties={
@@ -148,19 +148,23 @@ class ExpireTest(_fixtures.FixtureTest):
 
     @testing.resolve_artifact_names
     def test_refresh_collection_exception(self):
-        """test graceful failure for currently unsupported immediate refresh of a collection"""
-        
+        """test graceful failure for currently unsupported 
+        immediate refresh of a collection"""
+
         mapper(User, users, properties={
             'addresses':relationship(Address, order_by=addresses.c.email_address)
         })
         mapper(Address, addresses)
         s = create_session(autoflush=True, autocommit=False)
         u = s.query(User).get(8)
-        assert_raises_message(sa_exc.InvalidRequestError, "properties specified for refresh", s.refresh, u, ['addresses'])
-        
+        assert_raises_message(sa_exc.InvalidRequestError, 
+                        "properties specified for refresh", 
+                        s.refresh, u, ['addresses'])
+
         # in contrast to a regular query with no columns
-        assert_raises_message(sa_exc.InvalidRequestError, "no columns with which to SELECT", s.query().all)
-        
+        assert_raises_message(sa_exc.InvalidRequestError, 
+                        "no columns with which to SELECT", s.query().all)
+
     @testing.resolve_artifact_names
     def test_refresh_cancels_expire(self):
         mapper(User, users)
@@ -225,7 +229,7 @@ class ExpireTest(_fixtures.FixtureTest):
         assert 'name' not in u.__dict__
         sess.add(u)
         assert u.name == 'jack'
-    
+
     @testing.resolve_artifact_names
     def test_no_instance_key_no_pk(self):
         # same as test_no_instance_key, but the PK columns
@@ -240,8 +244,8 @@ class ExpireTest(_fixtures.FixtureTest):
         assert 'name' not in u.__dict__
         sess.add(u)
         assert_raises(sa_exc.InvalidRequestError, getattr, u, 'name')
-        
-        
+
+
     @testing.resolve_artifact_names
     def test_expire_preserves_changes(self):
         """test that the expire load operation doesn't revert post-expire changes"""
@@ -346,7 +350,7 @@ class ExpireTest(_fixtures.FixtureTest):
     def test_refresh_cascade_pending(self):
         cascade = 'save-update, refresh-expire'
         self._test_cascade_to_pending(cascade, False)
-        
+
     @testing.resolve_artifact_names
     def _test_cascade_to_pending(self, cascade, expire_or_refresh):
         mapper(User, users, properties={
@@ -357,7 +361,7 @@ class ExpireTest(_fixtures.FixtureTest):
 
         u = s.query(User).get(8)
         a = Address(id=12, email_address='foobar')
-        
+
         u.addresses.append(a)
         if expire_or_refresh:
             s.expire(u)
@@ -367,7 +371,7 @@ class ExpireTest(_fixtures.FixtureTest):
             assert a not in s
         else:
             assert a in s
-        
+
         assert a not in u.addresses
         s.flush()
 
@@ -732,7 +736,8 @@ class ExpireTest(_fixtures.FixtureTest):
     @testing.resolve_artifact_names
     def test_expire_all(self):
         mapper(User, users, properties={
-            'addresses':relationship(Address, backref='user', lazy='joined'),
+            'addresses':relationship(Address, backref='user', lazy='joined', 
+                                    order_by=addresses.c.id),
             })
         mapper(Address, addresses)
 
@@ -754,9 +759,9 @@ class ExpireTest(_fixtures.FixtureTest):
         """Behavioral test to verify the current activity of loader callables."""
 
         mapper(User, users)
-        
+
         sess = create_session()
-        
+
         # deferred attribute option, gets the LoadDeferredColumns
         # callable
         u1 = sess.query(User).options(defer(User.name)).first()
@@ -764,14 +769,14 @@ class ExpireTest(_fixtures.FixtureTest):
                     attributes.instance_state(u1).callables['name'],
                     strategies.LoadDeferredColumns
                 )
-                
+
         # expire the attr, it gets the InstanceState callable
         sess.expire(u1, ['name'])
         assert isinstance(
                     attributes.instance_state(u1).callables['name'],
                     state.InstanceState
                 )
-                
+
         # load it, callable is gone
         u1.name
         assert 'name' not in attributes.instance_state(u1).callables
@@ -784,28 +789,28 @@ class ExpireTest(_fixtures.FixtureTest):
                     attributes.instance_state(u1).callables['name'],
                     state.InstanceState
                 )
-        
+
         # load over it.  everything normal.
         sess.query(User).first()
         assert 'name' not in attributes.instance_state(u1).callables
-        
+
         sess.expunge_all()
         u1 = sess.query(User).first()
         # for non present, still expires the same way
         del u1.name
         sess.expire(u1)
         assert 'name' in attributes.instance_state(u1).callables
-        
+
     @testing.resolve_artifact_names
     def test_state_deferred_to_col(self):
         """Behavioral test to verify the current activity of loader callables."""
-        
+
         mapper(User, users, properties={'name':deferred(users.c.name)})
 
         sess = create_session()
         u1 = sess.query(User).options(undefer(User.name)).first()
         assert 'name' not in attributes.instance_state(u1).callables
-        
+
         # mass expire, the attribute was loaded, 
         # the attribute gets the callable
         sess.expire(u1)
@@ -817,7 +822,7 @@ class ExpireTest(_fixtures.FixtureTest):
         # load it, callable is gone
         u1.name
         assert 'name' not in attributes.instance_state(u1).callables
-        
+
         # mass expire, attribute was loaded but then deleted,
         # the callable goes away - the state wants to flip 
         # it back to its "deferred" loader.
@@ -842,7 +847,7 @@ class ExpireTest(_fixtures.FixtureTest):
 
         mapper(User, users, properties={'addresses':relationship(Address, lazy='noload')})
         mapper(Address, addresses)
-        
+
         sess = create_session()
         u1 = sess.query(User).options(lazyload(User.addresses)).first()
         assert isinstance(
@@ -855,11 +860,11 @@ class ExpireTest(_fixtures.FixtureTest):
                     attributes.instance_state(u1).callables['addresses'],
                     strategies.LoadLazyAttribute
                 )
-        
+
         # load over it.  callable goes away.
         sess.query(User).first()
         assert 'addresses' not in attributes.instance_state(u1).callables
-        
+
         sess.expunge_all()
         u1 = sess.query(User).options(lazyload(User.addresses)).first()
         sess.expire(u1, ['addresses'])
@@ -867,13 +872,13 @@ class ExpireTest(_fixtures.FixtureTest):
                     attributes.instance_state(u1).callables['addresses'],
                     strategies.LoadLazyAttribute
                 )
-        
+
         # load the attr, goes away
         u1.addresses
         assert 'addresses' not in attributes.instance_state(u1).callables
-        
-        
-        
+
+
+
 class PolymorphicExpireTest(_base.MappedTest):
     run_inserts = 'once'
     run_deletes = None
@@ -886,7 +891,8 @@ class PolymorphicExpireTest(_base.MappedTest):
            Column('person_id', Integer, primary_key=True,
                   test_needs_autoincrement=True),
            Column('name', String(50)),
-           Column('type', String(30)))
+           Column('type', String(30)),
+           )
 
         engineers = Table('engineers', metadata,
            Column('person_id', Integer, ForeignKey('people.person_id'),
@@ -914,11 +920,15 @@ class PolymorphicExpireTest(_base.MappedTest):
             {'person_id':3, 'status':'old engineer'},
         )
 
+    @classmethod
     @testing.resolve_artifact_names
-    def test_poly_deferred(self):
+    def setup_mappers(cls):
         mapper(Person, people, polymorphic_on=people.c.type, polymorphic_identity='person')
         mapper(Engineer, engineers, inherits=Person, polymorphic_identity='engineer')
 
+    @testing.resolve_artifact_names
+    def test_poly_deferred(self):
+
         sess = create_session()
         [p1, e1, e2] = sess.query(Person).order_by(people.c.person_id).all()
 
@@ -953,12 +963,40 @@ class PolymorphicExpireTest(_base.MappedTest):
         self.assert_sql_count(testing.db, go, 2)
         eq_(Engineer.name.get_history(e1), (['new engineer name'],(), ['engineer1']))
 
+    @testing.resolve_artifact_names
+    def test_no_instance_key(self):
+
+        sess = create_session()
+        e1 = sess.query(Engineer).get(2)
+
+        sess.expire(e1, attribute_names=['name'])
+        sess.expunge(e1)
+        attributes.instance_state(e1).key = None
+        assert 'name' not in e1.__dict__
+        sess.add(e1)
+        assert e1.name == 'engineer1'
+
+    @testing.resolve_artifact_names
+    def test_no_instance_key(self):
+        # same as test_no_instance_key, but the PK columns
+        # are absent.  ensure an error is raised.
+        sess = create_session()
+        e1 = sess.query(Engineer).get(2)
+
+        sess.expire(e1, attribute_names=['name', 'person_id'])
+        sess.expunge(e1)
+        attributes.instance_state(e1).key = None
+        assert 'name' not in e1.__dict__
+        sess.add(e1)
+        assert_raises(sa_exc.InvalidRequestError, getattr, e1, 'name')
+
+
 class ExpiredPendingTest(_fixtures.FixtureTest):
     run_define_tables = 'once'
     run_setup_classes = 'once'
     run_setup_mappers = None
     run_inserts = None
-    
+
     @testing.resolve_artifact_names
     def test_expired_pending(self):
         mapper(User, users, properties={
@@ -970,7 +1008,7 @@ class ExpiredPendingTest(_fixtures.FixtureTest):
         a1 = Address(email_address='a1')
         sess.add(a1)
         sess.flush()
-        
+
         u1 = User(name='u1')
         a1.user = u1
         sess.flush()
@@ -987,17 +1025,17 @@ class ExpiredPendingTest(_fixtures.FixtureTest):
         # expire u1.addresses again.  this expires
         # "pending" as well.
         sess.expire(u1, ['addresses'])
-        
+
         # insert a new row
         sess.execute(addresses.insert(), dict(email_address='a3', user_id=u1.id))
-        
+
         # only two addresses pulled from the DB, no "pending"
         assert len(u1.addresses) == 2
-        
+
         sess.flush()
         sess.expire_all()
         assert len(u1.addresses) == 3
-    
+
 
 class RefreshTest(_fixtures.FixtureTest):
 
@@ -1092,7 +1130,7 @@ class RefreshTest(_fixtures.FixtureTest):
     @testing.fails_on('maxdb', 'FIXME: unknown')
     @testing.resolve_artifact_names
     def test_refresh2(self):
-        """test a hang condition that was occuring on expire/refresh"""
+        """test a hang condition that was occurring on expire/refresh"""
 
         s = create_session()
         mapper(Address, addresses)
