@@ -1,11 +1,10 @@
-import testenv; testenv.configure_for_tests()
 import time
 from datetime import datetime
 
 from sqlalchemy import *
 from sqlalchemy.orm import *
-from testlib import *
-from testlib.profiling import profiled
+from sqlalchemy.test import *
+from sqlalchemy.test.profiling import profiled
 
 class Item(object):
     def __repr__(self):
@@ -62,14 +61,14 @@ def define_tables():
 @profiled('mapper')
 def setup_mappers():
     mapper(Item, items, properties={
-            'subitems': relation(SubItem, backref='item', lazy=True)
+            'subitems': relationship(SubItem, backref='item', lazy='select')
             })
     mapper(SubItem, subitems)
     mapper(Customer, customers, properties={
-            'purchases': relation(Purchase, lazy=True, backref='customer')
+            'purchases': relationship(Purchase, lazy='select', backref='customer')
             })
     mapper(Purchase, purchases, properties={
-            'items': relation(Item, lazy=True, secondary=purchaseitems)
+            'items': relationship(Item, lazy='select', secondary=purchaseitems)
             })
 
 @profiled('inserts')
@@ -153,8 +152,8 @@ def run_queries():
     q = session.query(Purchase). \
         order_by(desc(Purchase.purchase_date)). \
         limit(50).\
-        options(eagerload('items'), eagerload('items.subitems'),
-                eagerload('customer'))
+        options(joinedload('items'), joinedload('items.subitems'),
+                joinedload('customer'))
 
     report = []
     # "write" the report.  pretend it's going to a web template or something,
