@@ -1,10 +1,11 @@
 from sqlalchemy.test.testing import eq_, assert_raises, assert_raises_message
 import datetime
 from sqlalchemy import Sequence, Column, func
+from sqlalchemy.schema import CreateSequence, DropSequence
 from sqlalchemy.sql import select, text
 import sqlalchemy as sa
-from sqlalchemy.test import testing
-from sqlalchemy import MetaData, Integer, String, ForeignKey, Boolean
+from sqlalchemy.test import testing, engines
+from sqlalchemy import MetaData, Integer, String, ForeignKey, Boolean, exc
 from sqlalchemy.test.schema import Table
 from sqlalchemy.test.testing import eq_
 from test.sql import _base
@@ -37,7 +38,7 @@ class DefaultTest(testing.TestBase):
                 # since its a "branched" connection
                 conn.close()
 
-        use_function_defaults = testing.against('postgres', 'mssql', 'maxdb')
+        use_function_defaults = testing.against('postgresql', 'mssql', 'maxdb')
         is_oracle = testing.against('oracle')
 
         # select "count(1)" returns different results on different DBs also
@@ -284,7 +285,7 @@ class DefaultTest(testing.TestBase):
     @testing.fails_on('firebird', 'Data type unknown')
     def test_insertmany(self):
         # MySQL-Python 1.2.2 breaks functions in execute_many :(
-        if (testing.against('mysql') and
+        if (testing.against('mysql+mysqldb') and
             testing.db.dialect.dbapi.version_info[:3] == (1, 2, 2)):
             return
 
@@ -301,15 +302,24 @@ class DefaultTest(testing.TestBase):
              (53, 'imthedefault', f, ts, ts, ctexec, True, False,
               12, today, 'py')])
 
+    def test_missing_many_param(self):
+        assert_raises_message(exc.InvalidRequestError, 
+            "A value is required for bind parameter 'col7', in parameter group 1",
+            t.insert().execute,
+            {'col4':7, 'col7':12, 'col8':19},
+            {'col4':7, 'col8':19},
+            {'col4':7, 'col7':12, 'col8':19},
+        )
+
     def test_insert_values(self):
         t.insert(values={'col3':50}).execute()
         l = t.select().execute()
-        eq_(50, l.fetchone()['col3'])
+        eq_(50, l.first()['col3'])
 
     @testing.fails_on('firebird', 'Data type unknown')
     def test_updatemany(self):
         # MySQL-Python 1.2.2 breaks functions in execute_many :(
-        if (testing.against('mysql') and
+        if (testing.against('mysql+mysqldb') and
             testing.db.dialect.dbapi.version_info[:3] == (1, 2, 2)):
             return
 
@@ -337,11 +347,11 @@ class DefaultTest(testing.TestBase):
     @testing.fails_on('firebird', 'Data type unknown')
     def test_update(self):
         r = t.insert().execute()
-        pk = r.last_inserted_ids()[0]
+        pk = r.inserted_primary_key[0]
         t.update(t.c.col1==pk).execute(col4=None, col5=None)
         ctexec = currenttime.scalar()
         l = t.select(t.c.col1==pk).execute()
-        l = l.fetchone()
+        l = l.first()
         eq_(l,
             (pk, 'im the update', f2, None, None, ctexec, True, False,
              13, datetime.date.today(), 'py'))
@@ -350,43 +360,12 @@ class DefaultTest(testing.TestBase):
     @testing.fails_on('firebird', 'Data type unknown')
     def test_update_values(self):
         r = t.insert().execute()
-        pk = r.last_inserted_ids()[0]
+        pk = r.inserted_primary_key[0]
         t.update(t.c.col1==pk, values={'col3': 55}).execute()
         l = t.select(t.c.col1==pk).execute()
-        l = l.fetchone()
+        l = l.first()
         eq_(55, l['col3'])
 
-    @testing.fails_on_everything_except('postgres')
-    def test_passive_override(self):
-        """
-        Primarily for postgres, tests that when we get a primary key column
-        back from reflecting a table which has a default value on it, we
-        pre-execute that DefaultClause upon insert, even though DefaultClause
-        says "let the database execute this", because in postgres we must have
-        all the primary key values in memory before insert; otherwise we can't
-        locate the just inserted row.
-
-        """
-        # TODO: move this to dialect/postgres
-        try:
-            meta = MetaData(testing.db)
-            testing.db.execute("""
-             CREATE TABLE speedy_users
-             (
-                 speedy_user_id   SERIAL     PRIMARY KEY,
-
-                 user_name        VARCHAR    NOT NULL,
-                 user_password    VARCHAR    NOT NULL
-             );
-            """, None)
-
-            t = Table("speedy_users", meta, autoload=True)
-            t.insert().execute(user_name='user', user_password='lala')
-            l = t.select().execute().fetchall()
-            eq_(l, [(1, 'user', 'lala')])
-        finally:
-            testing.db.execute("drop table speedy_users", None)
-
 
 class PKDefaultTest(_base.TablesTest):
     __requires__ = ('subqueries',)
@@ -401,17 +380,26 @@ class PKDefaultTest(_base.TablesTest):
                      default=sa.select([func.max(t2.c.nextid)]).as_scalar()),
               Column('data', String(30)))
 
-    @testing.fails_on('mssql', 'FIXME: unknown')
+    @testing.requires.returning
+    def test_with_implicit_returning(self):
+        self._test(True)
+
+    def test_regular(self):
+        self._test(False)
+
     @testing.resolve_artifact_names
-    def test_basic(self):
-        t2.insert().execute(nextid=1)
-        r = t1.insert().execute(data='hi')
-        eq_([1], r.last_inserted_ids())
-
-        t2.insert().execute(nextid=2)
-        r = t1.insert().execute(data='there')
-        eq_([2], r.last_inserted_ids())
+    def _test(self, returning):
+        if not returning and not testing.db.dialect.implicit_returning:
+            engine = testing.db
+        else:
+            engine = engines.testing_engine(options={'implicit_returning':returning})
+        engine.execute(t2.insert(), nextid=1)
+        r = engine.execute(t1.insert(), data='hi')
+        eq_([1], r.inserted_primary_key)
 
+        engine.execute(t2.insert(), nextid=2)
+        r = engine.execute(t1.insert(), data='there')
+        eq_([2], r.inserted_primary_key)
 
 class PKIncrementTest(_base.TablesTest):
     run_define_tables = 'each'
@@ -430,29 +418,31 @@ class PKIncrementTest(_base.TablesTest):
     def _test_autoincrement(self, bind):
         ids = set()
         rs = bind.execute(aitable.insert(), int1=1)
-        last = rs.last_inserted_ids()[0]
+        last = rs.inserted_primary_key[0]
         self.assert_(last)
         self.assert_(last not in ids)
         ids.add(last)
 
         rs = bind.execute(aitable.insert(), str1='row 2')
-        last = rs.last_inserted_ids()[0]
+        last = rs.inserted_primary_key[0]
         self.assert_(last)
         self.assert_(last not in ids)
         ids.add(last)
 
         rs = bind.execute(aitable.insert(), int1=3, str1='row 3')
-        last = rs.last_inserted_ids()[0]
+        last = rs.inserted_primary_key[0]
         self.assert_(last)
         self.assert_(last not in ids)
         ids.add(last)
 
         rs = bind.execute(aitable.insert(values={'int1':func.length('four')}))
-        last = rs.last_inserted_ids()[0]
+        last = rs.inserted_primary_key[0]
         self.assert_(last)
         self.assert_(last not in ids)
         ids.add(last)
 
+        eq_(ids, set([1,2,3,4]))
+
         eq_(list(bind.execute(aitable.select().order_by(aitable.c.id))),
             [(1, 1, None), (2, None, 'row 2'), (3, 3, 'row 3'), (4, 4, None)])
 
@@ -487,7 +477,7 @@ class EmptyInsertTest(testing.TestBase):
         t1 = Table('t1', metadata,
                 Column('is_true', Boolean, server_default=('1')))
         metadata.create_all()
-        
+
         try:
             result = t1.insert().execute()
             eq_(1, select([func.count(text('*'))], from_obj=t1).scalar())
@@ -510,8 +500,8 @@ class AutoIncrementTest(_base.TablesTest):
         single.create()
 
         r = single.insert().execute()
-        id_ = r.last_inserted_ids()[0]
-        assert id_ is not None
+        id_ = r.inserted_primary_key[0]
+        eq_(id_, 1)
         eq_(1, sa.select([func.count(sa.text('*'))], from_obj=single).scalar())
 
     def test_autoincrement_fk(self):
@@ -522,7 +512,7 @@ class AutoIncrementTest(_base.TablesTest):
         nodes.create()
 
         r = nodes.insert().execute(data='foo')
-        id_ = r.last_inserted_ids()[0]
+        id_ = r.inserted_primary_key[0]
         nodes.insert().execute(data='bar', parent_id=id_)
 
     @testing.fails_on('sqlite', 'FIXME: unknown')
@@ -535,7 +525,7 @@ class AutoIncrementTest(_base.TablesTest):
 
 
         try:
-            # postgres + mysql strict will fail on first row,
+            # postgresql + mysql strict will fail on first row,
             # mysql in legacy mode fails on second row
             nonai.insert().execute(data='row 1')
             nonai.insert().execute(data='row 2')
@@ -546,10 +536,10 @@ class AutoIncrementTest(_base.TablesTest):
         nonai.insert().execute(id=1, data='row 1')
 
 
-class SequenceTest(testing.TestBase):
-    __requires__ = ('sequences',)
+class SequenceTest(testing.TestBase, testing.AssertsCompiledSQL):
 
     @classmethod
+    @testing.requires.sequences
     def setup_class(cls):
         global cartitems, sometable, metadata
         metadata = MetaData(testing.db)
@@ -567,31 +557,87 @@ class SequenceTest(testing.TestBase):
 
         metadata.create_all()
 
-    def testseqnonpk(self):
+
+    def test_compile(self):
+        self.assert_compile(
+            CreateSequence(Sequence('foo_seq')),
+            "CREATE SEQUENCE foo_seq",
+            use_default_dialect=True,
+        )
+
+        self.assert_compile(
+            CreateSequence(Sequence('foo_seq', start=5)),
+            "CREATE SEQUENCE foo_seq START WITH 5",
+            use_default_dialect=True,
+        )
+
+        self.assert_compile(
+            CreateSequence(Sequence('foo_seq', increment=2)),
+            "CREATE SEQUENCE foo_seq INCREMENT BY 2",
+            use_default_dialect=True,
+        )
+
+        self.assert_compile(
+            CreateSequence(Sequence('foo_seq', increment=2, start=5)),
+            "CREATE SEQUENCE foo_seq INCREMENT BY 2 START WITH 5",
+            use_default_dialect=True,
+        )
+
+        self.assert_compile(
+            DropSequence(Sequence('foo_seq')),
+            "DROP SEQUENCE foo_seq",
+            use_default_dialect=True,
+        )
+
+
+    @testing.fails_on('firebird', 'no FB support for start/increment')
+    @testing.requires.sequences
+    def test_start_increment(self):
+        for seq in (
+                Sequence('foo_seq'), 
+                Sequence('foo_seq', start=8), 
+                Sequence('foo_seq', increment=5)):
+            seq.create(testing.db)
+            try:
+                values = [
+                    testing.db.execute(seq) for i in range(3)
+                ]
+                start = seq.start or 1
+                inc = seq.increment or 1
+                assert values == list(xrange(start, start + inc * 3, inc))
+
+            finally:
+                seq.drop(testing.db)
+
+    @testing.requires.sequences
+    def test_seq_nonpk(self):
         """test sequences fire off as defaults on non-pk columns"""
 
-        result = sometable.insert().execute(name="somename")
-        assert 'id' in result.postfetch_cols()
+        engine = engines.testing_engine(options={'implicit_returning':False})
+        result = engine.execute(sometable.insert(), name="somename")
 
-        result = sometable.insert().execute(name="someother")
-        assert 'id' in result.postfetch_cols()
+        assert set(result.postfetch_cols()) == set([sometable.c.obj_id])
+
+        result = engine.execute(sometable.insert(), name="someother")
+        assert set(result.postfetch_cols()) == set([sometable.c.obj_id])
 
         sometable.insert().execute(
             {'name':'name3'},
             {'name':'name4'})
-        eq_(sometable.select().execute().fetchall(),
+        eq_(sometable.select().order_by(sometable.c.id).execute().fetchall(),
             [(1, "somename", 1),
              (2, "someother", 2),
              (3, "name3", 3),
              (4, "name4", 4)])
 
-    def testsequence(self):
+    @testing.requires.sequences
+    def test_sequence(self):
         cartitems.insert().execute(description='hi')
         cartitems.insert().execute(description='there')
         r = cartitems.insert().execute(description='lala')
 
-        assert r.last_inserted_ids() and r.last_inserted_ids()[0] is not None
-        id_ = r.last_inserted_ids()[0]
+        assert r.inserted_primary_key and r.inserted_primary_key[0] is not None
+        id_ = r.inserted_primary_key[0]
 
         eq_(1,
             sa.select([func.count(cartitems.c.cart_id)],
@@ -603,6 +649,7 @@ class SequenceTest(testing.TestBase):
     @testing.fails_on('maxdb', 'FIXME: unknown')
     # maxdb db-api seems to double-execute NEXTVAL internally somewhere,
     # throwing off the numbers for these tests...
+    @testing.requires.sequences
     def test_implicit_sequence_exec(self):
         s = Sequence("my_sequence", metadata=MetaData(testing.db))
         s.create()
@@ -613,6 +660,7 @@ class SequenceTest(testing.TestBase):
             s.drop()
 
     @testing.fails_on('maxdb', 'FIXME: unknown')
+    @testing.requires.sequences
     def teststandalone_explicit(self):
         s = Sequence("my_sequence")
         s.create(bind=testing.db)
@@ -622,6 +670,7 @@ class SequenceTest(testing.TestBase):
         finally:
             s.drop(testing.db)
 
+    @testing.requires.sequences
     def test_checkfirst(self):
         s = Sequence("my_sequence")
         s.create(testing.db, checkfirst=False)
@@ -630,11 +679,13 @@ class SequenceTest(testing.TestBase):
         s.drop(testing.db, checkfirst=True)
 
     @testing.fails_on('maxdb', 'FIXME: unknown')
+    @testing.requires.sequences
     def teststandalone2(self):
-        x = cartitems.c.cart_id.sequence.execute()
+        x = cartitems.c.cart_id.default.execute()
         self.assert_(1 <= x <= 4)
 
     @classmethod
+    @testing.requires.sequences
     def teardown_class(cls):
         metadata.drop_all()
 
